"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const util_1 = require("util");
const fs = require("fs");
const os = require("os");
const path = require("path");
const Logger_1 = require("./Logger");
const Constants_1 = require("./constants/Constants");
var MCUtil;
(function (MCUtil) {
    function uppercaseFirstChar(input) {
        return input.charAt(0).toUpperCase() + input.slice(1);
    }
    MCUtil.uppercaseFirstChar = uppercaseFirstChar;
    /**
     * Returns a wrapper promise which runs the given promise with the given timeout.
     * If the timeout expires before the given promise is fulfilled, the wrapper promise rejects with the given message.
     *
     * If the promise resolves or rejects before the timeout,
     * the wrapper promise resolves or rejects with the same result as the inner promise.
     */
    function promiseWithTimeout(promise, timeoutMS, rejectMsg) {
        return new Promise((resolve, reject) => {
            setTimeout(() => reject(rejectMsg), timeoutMS);
            promise
                .then((result) => resolve(result))
                .catch((err) => reject(err));
        });
    }
    MCUtil.promiseWithTimeout = promiseWithTimeout;
    function isGoodDate(date) {
        return !isNaN(date.valueOf());
    }
    MCUtil.isGoodDate = isGoodDate;
    function isGoodStatusCode(statusCode) {
        return statusCode != null && !isNaN(statusCode) && statusCode >= 200 && statusCode < 400;
    }
    MCUtil.isGoodStatusCode = isGoodStatusCode;
    /**
     * Extract the hostname from a URL with authority hostname:9090, for example.
     * If there's no port, return the whole authority.
     */
    function getHostnameFrom(url) {
        const authority = url.authority;
        const colonIndex = authority.indexOf(":"); // non-nls
        if (colonIndex === -1) {
            return authority;
        }
        return authority.substring(0, colonIndex);
    }
    MCUtil.getHostnameFrom = getHostnameFrom;
    function isGoodPort(port) {
        return port != null && !isNaN(port) && Number.isInteger(port) && port > 0 && port < 65536;
    }
    MCUtil.isGoodPort = isGoodPort;
    function errToString(err, isOidc = false) {
        if (err.toString() === err) {
            // string errors don't need any change
            return err.toString();
        }
        if (isOidc) {
            return err.error_description || err.error || err.message || JSON.stringify(err);
        }
        if (err.error) {
            if (err.error.info) {
                const infoMsg = err.error.info.message;
                if (infoMsg) {
                    return infoMsg;
                }
            }
            const msg = err.error.msg || err.error.message;
            if (msg) {
                return msg;
            }
            const jsoErr = JSON.stringify(err.error);
            if (jsoErr !== "{}") {
                return jsoErr;
            }
        }
        if (err.message) {
            return err.message;
        }
        return "Unknown error";
    }
    MCUtil.errToString = errToString;
    const charsToRemove = "·/_,:;";
    const toRemoveRx = new RegExp(charsToRemove.split("").join("|"), "g");
    /**
     * Not a 'normal' slug function, but makes strings look nice and normal and kebab-cased.
     * Replace url-unfriendly characters, spaces and '.'s with '-'.
     *
     * Inspired by https://medium.com/@mhagemann/the-ultimate-way-to-slugify-a-url-string-in-javascript-b8e4a0d849e1
     */
    function slug(s) {
        return s.toLowerCase()
            .replace(/\s+/g, "-") // spaces to -
            .replace(/\./g, "-") // literal . to -
            .replace(/\(|\)/g, "") // remove ( and )
            .replace(toRemoveRx, "") // remove other special chars
            // .replace(/[^\w\-]+/g, "")    // remove all non-words
            .replace(/\-\-+/g, "-") // replace multiple - with single
            .replace(/^-+/, "") // trim - from start
            .replace(/-+$/, ""); // trim - from end
    }
    MCUtil.slug = slug;
    let currentOS;
    function getOS() {
        // Get OS once, then cache it
        if (currentOS) {
            return currentOS;
        }
        const platf = process.platform;
        // https://nodejs.org/api/process.html#process_process_platform
        if (platf === "win32") {
            currentOS = "windows";
        }
        else if (platf === "darwin") {
            currentOS = "darwin";
        }
        else if (platf === "linux") {
            currentOS = "linux";
        }
        else {
            Logger_1.default.w("Potentially unsupported platform: " + platf);
            // our 'best guess' is linux
            currentOS = "linux";
        }
        return currentOS;
    }
    MCUtil.getOS = getOS;
    /**
     * C:\\Users\\... -> /C/Users/
     */
    function fsPathToContainerPath(fsPath) {
        const pathStr = fsPath instanceof vscode.Uri ? fsPath.fsPath : fsPath;
        if (getOS() !== "windows") {
            return pathStr;
        }
        const driveLetter = pathStr.charAt(0);
        // we have to convert C:\<path> to /C/<path>
        const containerPath = pathStr.substring(2).replace(/\\/g, "/");
        return `/${driveLetter}${containerPath}`;
    }
    MCUtil.fsPathToContainerPath = fsPathToContainerPath;
    /**
     * /C/Users/... -> C:/Users/
     */
    function containerPathToFsPath(containerPath) {
        if (getOS() !== "windows") {
            return containerPath;
        }
        let deviceLetter;
        if (containerPath.startsWith("/")) {
            deviceLetter = containerPath.substring(1, 2);
        }
        else {
            deviceLetter = containerPath.charAt(0);
        }
        return `${deviceLetter}:${containerPath.substring(2)}`;
    }
    MCUtil.containerPathToFsPath = containerPathToFsPath;
    /**
     * List of files whose presence indicates we are in a CW workspace or project.
     * This should match this list in the workspaceContains activation events in package.json.
     */
    const CW_FILES = [
        // workspace files
        ".idc",
        ".projects",
        ".extensions",
        // project files
        Constants_1.default.PROJ_SETTINGS_FILE_NAME,
    ];
    /**
     * Returns true if any of the user's workspace folders look like a Codewind workspace, or a Codewind project,
     * by checking for existence of the CW_FILES.
     */
    function isUserInCwWorkspaceOrProject() {
        return __awaiter(this, void 0, void 0, function* () {
            const wsFolders = vscode.workspace.workspaceFolders;
            if (wsFolders == null || wsFolders.length === 0) {
                return false;
            }
            const checkWsFolderPromises = wsFolders.map((wsFolder) => isCwWorkspaceOrProject(wsFolder.uri.fsPath));
            return (yield Promise.all(checkWsFolderPromises)).some((result) => result);
        });
    }
    MCUtil.isUserInCwWorkspaceOrProject = isUserInCwWorkspaceOrProject;
    function isCwWorkspaceOrProject(dirPath) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield util_1.promisify(fs.readdir)(dirPath))
                .some((file) => CW_FILES.includes(file.toString()));
        });
    }
    function getCWWorkspacePath() {
        if (MCUtil.getOS() === "windows") {
            return "C:\\codewind-workspace";
        }
        return path.join(os.homedir(), "codewind-workspace");
    }
    MCUtil.getCWWorkspacePath = getCWWorkspacePath;
    function getCWDataPath() {
        if (MCUtil.getOS() === "windows") {
            return "C:\\codewind-data";
        }
        return path.join(os.homedir(), "codewind-data");
    }
    MCUtil.getCWDataPath = getCWDataPath;
    /**
     * Joins a string array into a user-friendly list.
     * Eg, `joinWithAnd([ "tim", "erin", "john" ])` => "tim, erin, and john"
     */
    function joinWithAnd(strings) {
        strings = strings.filter((s) => {
            // remove undefined/empty
            if (!s) {
                Logger_1.default.w(`Refusing to joinWithAnd empty or undefined string`);
                return false;
            }
            return true;
        });
        let joined = strings.join(" ");
        if (strings.length > 1) {
            const lastItem = strings[strings.length - 1];
            joined = joined.replace(lastItem, "and " + lastItem);
        }
        return joined;
    }
    MCUtil.joinWithAnd = joinWithAnd;
    function isDevEnv() {
        return process.env[Constants_1.default.CW_ENV_VAR] === Constants_1.default.CW_ENV_DEV;
    }
    MCUtil.isDevEnv = isDevEnv;
    function promptForProjectDir(btnLabel, defaultUri) {
        return __awaiter(this, void 0, void 0, function* () {
            const selectedDirs = yield vscode.window.showOpenDialog({
                canSelectFiles: false,
                canSelectFolders: true,
                canSelectMany: false,
                openLabel: btnLabel,
                defaultUri
            });
            if (selectedDirs == null) {
                return undefined;
            }
            // canSelectMany is false so we just use [0]
            const selectedDir = selectedDirs[0];
            const cwDataPath = MCUtil.getCWDataPath();
            if (selectedDir.fsPath.startsWith(cwDataPath)) {
                vscode.window.showErrorMessage(`You cannot create or add a project under ${cwDataPath}. Select a different directory.`);
                return undefined;
            }
            return selectedDir;
        });
    }
    MCUtil.promptForProjectDir = promptForProjectDir;
})(MCUtil || (MCUtil = {}));
exports.default = MCUtil;
//# sourceMappingURL=MCUtil.js.map