"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CLIWrapper_1 = require("./CLIWrapper");
const Logger_1 = require("../../Logger");
const MCUtil_1 = require("../../MCUtil");
const CLICommands_1 = require("./CLICommands");
var CLICommandRunner;
(function (CLICommandRunner) {
    function status() {
        return __awaiter(this, void 0, void 0, function* () {
            const statusObj = yield CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.STATUS);
            // The CLI will leave out these fields if they are empty, but an empty array is easier to deal with.
            if (statusObj["installed-versions"] == null) {
                statusObj["installed-versions"] = [];
            }
            if (statusObj.started == null) {
                statusObj.started = [];
            }
            return statusObj;
        });
    }
    CLICommandRunner.status = status;
    function createProject(connectionID, projectPath, url) {
        return __awaiter(this, void 0, void 0, function* () {
            return CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.PROJECT.CREATE, [
                projectPath,
                "--conid", connectionID,
                "--url", url
            ]);
        });
    }
    CLICommandRunner.createProject = createProject;
    /**
     * Test the path given to determine the project type Codewind should use.
     */
    function detectProjectType(connectionID, projectPath, desiredType) {
        return __awaiter(this, void 0, void 0, function* () {
            const args = [
                projectPath,
                "--conid", connectionID,
            ];
            if (desiredType) {
                args.push("--type", desiredType);
            }
            return CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.PROJECT.CREATE, args, `Processing ${projectPath}...`);
        });
    }
    CLICommandRunner.detectProjectType = detectProjectType;
    /**
     * @returns The newly created project's inf content.
     */
    function bindProject(connectionID, projectName, projectPath, detectedType) {
        return __awaiter(this, void 0, void 0, function* () {
            const bindRes = yield CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.PROJECT.BIND, [
                "--conid", connectionID,
                "--name", projectName,
                "--language", detectedType.language,
                "--type", detectedType.projectType,
                "--path", projectPath,
            ]);
            if (bindRes.error_description) {
                throw new Error(bindRes.error_description);
            }
            else if (!bindRes.projectID) {
                // should never happen
                throw new Error(`Failed to bind ${projectName}; no project ID was returned.`);
            }
            Logger_1.default.i(`Bound new project ${projectName} with ID ${bindRes.projectID}`);
            return bindRes;
        });
    }
    CLICommandRunner.bindProject = bindProject;
    /**
     * Perform a workspace upgrade from a version older than 0.6
     */
    function upgrade() {
        return __awaiter(this, void 0, void 0, function* () {
            return CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.UPGRADE, [
                "--ws", MCUtil_1.default.getCWWorkspacePath(),
            ]);
        });
    }
    CLICommandRunner.upgrade = upgrade;
    ///// Connection management commands
    /**
     * @returns The data for the new Connection
     */
    function addConnection(label, url, username) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.CONNECTIONS.ADD, [
                "--label", label,
                "--url", url,
                "--username", username,
            ]);
        });
    }
    CLICommandRunner.addConnection = addConnection;
    /**
     * @returns The data for all current connections, except Local
     */
    function getRemoteConnections() {
        return __awaiter(this, void 0, void 0, function* () {
            const connections = yield CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.CONNECTIONS.LIST);
            if (!connections.connections) {
                // the local connection should at least be there
                Logger_1.default.e(`Received no connections back from connections list`);
                return [];
            }
            return connections.connections.filter((conn) => conn.id !== "local");
        });
    }
    CLICommandRunner.getRemoteConnections = getRemoteConnections;
    /**
     *
     * @returns The data for all current connections, after removal.
     */
    function removeConnection(id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.CONNECTIONS.REMOVE, ["--conid", id]);
        });
    }
    CLICommandRunner.removeConnection = removeConnection;
    function updateConnection(newData) {
        return __awaiter(this, void 0, void 0, function* () {
            yield CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.CONNECTIONS.UPDATE, [
                "--conid", newData.id,
                "--label", newData.label,
                "--url", newData.url,
                "--username", newData.username
            ]);
        });
    }
    CLICommandRunner.updateConnection = updateConnection;
    ///// Template source management commands - These should only be used by the TemplateSourceList
    function addTemplateSource(connectionID, url, name, descr) {
        return __awaiter(this, void 0, void 0, function* () {
            const args = [
                "--conid", connectionID,
                "--url", url,
                "--name", name,
            ];
            if (descr) {
                args.push("--description", descr);
            }
            return CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.TEMPLATE_SOURCES.ADD, args);
        });
    }
    CLICommandRunner.addTemplateSource = addTemplateSource;
    function getTemplateSources(connectionID, showProgress) {
        return __awaiter(this, void 0, void 0, function* () {
            const progress = showProgress ? undefined : "Fetching template sources...";
            return CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.TEMPLATE_SOURCES.LIST, [
                "--conid", connectionID,
            ], progress);
        });
    }
    CLICommandRunner.getTemplateSources = getTemplateSources;
    function removeTemplateSource(connectionID, url) {
        return __awaiter(this, void 0, void 0, function* () {
            return CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.TEMPLATE_SOURCES.REMOVE, [
                "--conid", connectionID,
                "--url", url
            ]);
        });
    }
    CLICommandRunner.removeTemplateSource = removeTemplateSource;
    ///// Auth/credential commands
    function updateKeyringCredentials(connectionID, username, password) {
        return __awaiter(this, void 0, void 0, function* () {
            // in the success case, the output is just an OK status
            yield CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.AUTHENTICATION.KEYRING_UPDATE, [
                "--conid", connectionID,
                "--username", username,
                "--password", password
            ]);
        });
    }
    CLICommandRunner.updateKeyringCredentials = updateKeyringCredentials;
    CLICommandRunner.INVALID_CREDS_ERR = "Invalid user credentials";
    function getAccessToken(connectionID, username) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield CLIWrapper_1.default.cliExec(CLICommands_1.CLICommands.AUTHENTICATION.GET_SECTOKEN, [
                    "--conid", connectionID,
                    "--username", username,
                ]);
                return result;
            }
            catch (err) {
                if (err.toString().includes(CLICommandRunner.INVALID_CREDS_ERR)) {
                    // simplify the error
                    throw new Error(CLICommandRunner.INVALID_CREDS_ERR);
                }
                throw err;
            }
        });
    }
    CLICommandRunner.getAccessToken = getAccessToken;
})(CLICommandRunner = exports.CLICommandRunner || (exports.CLICommandRunner = {}));
//# sourceMappingURL=CLICommandRunner.js.map