"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const path = require("path");
const child_process = require("child_process");
const fs = require("fs");
const os = require("os");
const MCUtil_1 = require("../../MCUtil");
const Logger_1 = require("../../Logger");
const CLILifecycleWrapper_1 = require("./local/CLILifecycleWrapper");
const CLILifecycleCommands_1 = require("./local/CLILifecycleCommands");
const Constants_1 = require("../../constants/Constants");
const BIN_DIR = "bin";
const CLI_EXECUTABLE = "cwctl";
const CLI_EXECUTABLE_WIN = "cwctl.exe";
const CLI_PREREQS = {
    [CLI_EXECUTABLE]: ["appsody"],
    [CLI_EXECUTABLE_WIN]: ["appsody.exe"]
};
const cliOutputChannel = vscode.window.createOutputChannel("Codewind");
var CLIWrapper;
(function (CLIWrapper) {
    // check error against this to see if it's a real error or just a user cancellation
    CLIWrapper.CLI_CMD_CANCELLED = "Cancelled";
    /**
     * Returns the location of the executable as within the extension. It cannot be run from this location - see initialize()
     */
    function getInternalExecutable() {
        const platform = MCUtil_1.default.getOS();
        const executable = platform === "windows" ? CLI_EXECUTABLE_WIN : CLI_EXECUTABLE;
        return path.join(global.__extRoot, BIN_DIR, platform, executable);
    }
    // abs path to copied-out executable. Set and returned by initialize()
    let _cwctlPath;
    /**
     * Copies the CLI to somewhere writeable if not already done, and sets exectablePath.
     * @returns The path to the CLI executable after moving it to a writeable directory.
     */
    function initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            if (_cwctlPath) {
                return _cwctlPath;
            }
            // The executable is copied out to eg ~/.codewind/0.7.0/cwctl
            const cwctlSourcePath = getInternalExecutable();
            const binarySourceDir = path.dirname(cwctlSourcePath);
            const cwctlBasename = path.basename(cwctlSourcePath);
            const dotCodewindPath = path.join(os.homedir(), Constants_1.default.DOT_CODEWIND_DIR);
            const binaryTargetDir = path.join(dotCodewindPath, global.extVersion);
            // fails on windows, see note about electron https://github.com/nodejs/node/issues/24698#issuecomment-486405542
            // await promisify(fs.mkdir)(binaryTargetDir, { recursive: true });
            try {
                yield fs.promises.access(dotCodewindPath);
            }
            catch (err) {
                yield fs.promises.mkdir(dotCodewindPath);
                Logger_1.default.d(`Created ${dotCodewindPath}`);
            }
            try {
                yield fs.promises.access(binaryTargetDir);
            }
            catch (err) {
                yield fs.promises.mkdir(binaryTargetDir);
                Logger_1.default.d(`Created ${binaryTargetDir}`);
            }
            const cwctlTargetPath = path.join(binaryTargetDir, cwctlBasename);
            Logger_1.default.d(`Copying ${cwctlSourcePath} to ${cwctlTargetPath}`);
            yield fs.promises.copyFile(cwctlSourcePath, cwctlTargetPath);
            _cwctlPath = cwctlTargetPath;
            Logger_1.default.d("Copying CLI prerequisites");
            for (const prereq of CLI_PREREQS[cwctlBasename]) {
                const source = path.join(binarySourceDir, prereq);
                const target = path.join(binaryTargetDir, prereq);
                Logger_1.default.d(`Copying ${source} to ${target}`);
                yield fs.promises.copyFile(source, target);
            }
            Logger_1.default.i("Binary copy-out succeeded to " + _cwctlPath);
            cliOutputChannel.appendLine(`cwctl is available at ${_cwctlPath}`);
            return _cwctlPath;
        });
    }
    CLIWrapper.initialize = initialize;
    function getExecutablePath() {
        return __awaiter(this, void 0, void 0, function* () {
            if (_cwctlPath) {
                return _cwctlPath;
            }
            return initialize();
        });
    }
    CLIWrapper.getExecutablePath = getExecutablePath;
    const PASSWORD_ARG = "--password";
    function cliExec(cmd, args = [], progressPrefix) {
        return __awaiter(this, void 0, void 0, function* () {
            const executablePath = yield initialize();
            args = cmd.command.concat(args);
            if (!(cmd instanceof CLILifecycleCommands_1.CLILifecycleCommand)) {
                args.unshift("--insecure");
            }
            args.unshift("--json");
            // cmdStr will be the full command, eg `cwctl --insecure project create <path> --url <url>`
            // is generally only used for debugging
            let cmdStr = [path.basename(executablePath), ...args].join(" ");
            if (cmdStr.includes(PASSWORD_ARG)) {
                const words = cmdStr.split(" ");
                const pwIndex = words.findIndex((word) => word === PASSWORD_ARG) + 1;
                words[pwIndex] = "*********";
                cmdStr = words.join(" ");
            }
            Logger_1.default.i(`Running CLI command: ${cmdStr}`);
            // CLI output and err are echoed to a user-visible outputchannel.
            // We hide install output because it's thousands of lines of a progress bar, and sectoken because the token should not be exposed.
            cliOutputChannel.appendLine(`==> Run ${cmdStr}`);
            const echoOutput = !cmd.censorOutput;
            if (!echoOutput) {
                cliOutputChannel.appendLine(`<Output hidden>`);
            }
            const executableDir = path.dirname(executablePath);
            const cwctlProcess = child_process.spawn(executablePath, args, {
                cwd: executableDir
            });
            const cwctlExecPromise = new Promise((resolve, reject) => {
                cwctlProcess.on("error", (err) => {
                    return reject(err);
                });
                let outStr = "";
                let errStr = "";
                cwctlProcess.stdout.on("data", (chunk) => {
                    const str = chunk.toString();
                    if (echoOutput) {
                        cliOutputChannel.append(str);
                    }
                    outStr += str;
                });
                cwctlProcess.stderr.on("data", (chunk) => {
                    const str = chunk.toString();
                    cliOutputChannel.append(str);
                    errStr += str;
                });
                cwctlProcess.on("close", (code) => {
                    if (code == null) {
                        // this happens in SIGTERM case, not sure what else may cause it
                        Logger_1.default.w(`CLI command ${cmdStr} did not exit normally`);
                        Logger_1.default.w(`Stdout:`, outStr);
                        if (errStr) {
                            Logger_1.default.e(`Stderr:`, errStr);
                        }
                        resolve(outStr);
                    }
                    else if (code !== 0) {
                        Logger_1.default.e(`Error running ${cmdStr}`);
                        Logger_1.default.e("Stdout:", outStr.trim());
                        if (errStr) {
                            Logger_1.default.e("Stderr:", errStr.trim());
                        }
                        let errMsg = `Error running ${path.basename(_cwctlPath)} ${cmd.command.join(" ")}`;
                        if (cmd.hasJSONOutput && isProbablyJSON(outStr)) {
                            const asObj = JSON.parse(outStr);
                            if (asObj.error_description) {
                                errMsg += `: ${asObj.error_description}`;
                            }
                        }
                        if (errStr) {
                            errMsg += `: ${errStr}`;
                        }
                        return reject(errMsg);
                    }
                    else {
                        let successMsg = `Successfully ran CLI command ${cmdStr}`;
                        if (echoOutput) {
                            successMsg += `, Output was:\n` + outStr.trimRight();
                        }
                        Logger_1.default.d(successMsg);
                        if (!cmd.hasJSONOutput) {
                            return resolve(outStr);
                        }
                        if (!outStr || !isProbablyJSON(outStr)) {
                            Logger_1.default.e(`Missing expected JSON output from CLI command, output was "${outStr}"`);
                            return resolve({});
                        }
                        // Log.d("CLI object output:", outStr);
                        const obj = JSON.parse(outStr);
                        if (obj.error_description) {
                            return reject(obj.error_description);
                        }
                        return resolve(obj);
                    }
                });
            }).finally(() => {
                cliOutputChannel.appendLine(`==> End ${cmdStr}`);
            });
            const hasProgress = cmd instanceof CLILifecycleCommands_1.CLILifecycleCommand || progressPrefix;
            if (hasProgress) {
                return vscode.window.withProgress({
                    cancellable: cmd.cancellable,
                    location: vscode.ProgressLocation.Notification,
                    title: progressPrefix,
                }, (progress, token) => {
                    // only lifecycle commands show updating progress, for now
                    if (cmd instanceof CLILifecycleCommands_1.CLILifecycleCommand) {
                        CLILifecycleWrapper_1.CLILifecycleWrapper.updateProgress(cmd, cwctlProcess.stdout, progress);
                    }
                    token.onCancellationRequested((_e) => {
                        cwctlProcess.kill();
                    });
                    return cwctlExecPromise;
                });
            }
            else {
                return cwctlExecPromise;
            }
        });
    }
    CLIWrapper.cliExec = cliExec;
    function isProbablyJSON(str) {
        return str.startsWith("{") || str.startsWith("[");
    }
    function isCancellation(err) {
        return MCUtil_1.default.errToString(err) === CLIWrapper.CLI_CMD_CANCELLED;
    }
    CLIWrapper.isCancellation = isCancellation;
    function showCLIError(errMsg) {
        const viewErrBtn = "View Output";
        vscode.window.showErrorMessage(errMsg, viewErrBtn)
            .then((res) => {
            if (res === viewErrBtn) {
                cliOutputChannel.show();
            }
        });
    }
    CLIWrapper.showCLIError = showCLIError;
})(CLIWrapper || (CLIWrapper = {}));
exports.default = CLIWrapper;
//# sourceMappingURL=CLIWrapper.js.map