"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2020 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Project_1 = require("../project/Project");
const MCSocket_1 = require("./MCSocket");
const Logger_1 = require("../../Logger");
const CWEnvironment_1 = require("./CWEnvironment");
const MCUtil_1 = require("../../MCUtil");
const Requester_1 = require("../project/Requester");
const Constants_1 = require("../../constants/Constants");
const codewind_filewatcher_1 = require("codewind-filewatcher");
const Logger_2 = require("codewind-filewatcher/lib/Logger");
const LocalCodewindManager_1 = require("./local/LocalCodewindManager");
const CodewindEventListener_1 = require("./CodewindEventListener");
const CLIWrapper_1 = require("./CLIWrapper");
const ConnectionState_1 = require("./ConnectionState");
const TemplateSourceList_1 = require("./TemplateSourceList");
exports.LOCAL_CONNECTION_ID = "local";
class Connection {
    constructor(
    /**
     * Connection ID as returned by `cwctl connections add`
     */
    id, 
    /**
     * URL as returned by `cwctl status` in local, or to Codewind Gatekeeper for remote
     */
    url, 
    /**
     * User-provided label, or hard-coded local label
     */
    label, isRemote) {
        this.id = id;
        this.url = url;
        this.label = label;
        this.isRemote = isRemote;
        this.cwVersion = CWEnvironment_1.default.UNKNOWN_VERSION;
        this.hasConnected = false;
        this._projects = [];
        this.templateSourcesList = new TemplateSourceList_1.default(this);
        this.hasInitialized = false;
        this._hasHadPushRegistry = false;
        this.onConnect = () => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`${this} onConnect`);
            if (this.isConnected) {
                // we already know we're connected, nothing to do until we disconnect
                return;
            }
            if (this.hasConnected) {
                // things to do on reconnect, but not initial connect, go here
                this._projects.forEach((p) => p.onConnectionReconnect());
            }
            this.hasConnected = true;
            Logger_1.default.d(`${this} is now connected`);
            if (this.hasInitialized) {
                Logger_1.default.d(`${this} is now ready - initialize finished before connect`);
                this.setState(ConnectionState_1.ConnectionStates.READY);
            }
            this.onChange();
        });
        this.onDisconnect = () => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`${this} onDisconnect`);
            if (this._state === ConnectionState_1.ConnectionStates.NETWORK_ERROR) {
                // we already know we're disconnected, nothing to do until we reconnect
                return;
            }
            this.setState(ConnectionState_1.ConnectionStates.NETWORK_ERROR);
            this._projects.forEach((p) => p.onConnectionDisconnect());
            this._projects = [];
            Logger_1.default.d(`${this} is now disconnected`);
            this.onChange();
        });
        Logger_1.default.d(`Creating new connection ${this.label} @ ${this.url}`);
        this._state = ConnectionState_1.ConnectionStates.INITIALIZING;
        this.host = this.getHost(url);
        this.enable()
            .catch((err) => {
            const errMsg = `Error initializing ${this.label}:`;
            Logger_1.default.e(errMsg, err);
            vscode.window.showErrorMessage(`${errMsg} ${MCUtil_1.default.errToString(err)}`);
        });
    }
    get enabled() {
        return this.state !== ConnectionState_1.ConnectionStates.DISABLED;
    }
    get state() {
        return this._state;
    }
    setState(newState) {
        Logger_1.default.d(`${this.label} is now ${newState}`);
        this._state = newState;
        this.onChange();
    }
    get isConnected() {
        return this._state.isConnected;
    }
    enable() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i(`${this.label} starting base enable`);
            const readyTimeoutS = 60;
            const ready = yield Requester_1.default.waitForReady(this, readyTimeoutS);
            if (!ready) {
                throw new Error(`${this.label} connected, but was not ready after ${readyTimeoutS} seconds. Try reconnecting to, or restarting, this Codewind instance.`);
            }
            const envData = yield CWEnvironment_1.default.getEnvData(this);
            this.cwVersion = envData.version;
            // onConnect will be called on initial socket connect,
            // which does the initial projects population and sets the state to Connected
            this._socket = new MCSocket_1.default(this, envData.socketNamespace);
            const initFWProm = this.initFileWatcher();
            yield initFWProm;
            try {
                Logger_1.default.d("Updating projects list after ready");
                yield this.updateProjects();
            }
            catch (err) {
                Logger_1.default.e("Error updating projects list after ready", err);
            }
            this.hasInitialized = true;
            if (this._socket.isReady) {
                Logger_1.default.d(`${this} is now ready - enable finished after connect`);
                this.setState(ConnectionState_1.ConnectionStates.READY);
            }
            this.onChange(this);
            Logger_1.default.d(`${this} finished base enable`);
        });
    }
    disable() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Disable connection " + this);
            const fwDisposeProm = new Promise((resolve) => {
                if (this.fileWatcher) {
                    this.fileWatcher.dispose();
                }
                resolve();
            });
            yield Promise.all([
                fwDisposeProm,
                // disposing the socket will result in onDisconnect being called
                this._socket ? this._socket.dispose() : Promise.resolve(),
                this._projects.map((p) => p.dispose()),
            ]);
            this.hasConnected = false;
            this.hasInitialized = false;
            this._socket = undefined;
            this.fileWatcher = undefined;
            this._projects = [];
            this.onChange(this);
        });
    }
    dispose() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.disable();
        });
    }
    toString() {
        return `${this.label}`;
    }
    initFileWatcher() {
        return __awaiter(this, void 0, void 0, function* () {
            if (global.isTheia) {
                Logger_1.default.i("In theia; no filewatcher required");
                return;
            }
            Logger_1.default.i("Establishing file watcher");
            const cliPath = yield CLIWrapper_1.default.getExecutablePath();
            this.fileWatcher = yield this.createFileWatcher(cliPath);
            Logger_2.LogSettings.getInstance().setOutputLogsToScreen(false);
            Logger_1.default.i(`${this.label} File watcher is established`);
        });
    }
    createFileWatcher(cliPath) {
        return __awaiter(this, void 0, void 0, function* () {
            return codewind_filewatcher_1.CreateFileWatcher(this.url.toString(), Logger_1.default.getLogDir, undefined, cliPath);
        });
    }
    getHost(url) {
        if (global.isTheia) {
            // On theia we have to use the che ingress
            // something like CHE_API_EXTERNAL=http://che-eclipse-che.9.28.239.191.nip.io/api
            const cheExternalUrlStr = process.env[Constants_1.default.CHE_API_EXTERNAL_ENVVAR];
            Logger_1.default.d(`${Constants_1.default.CHE_API_EXTERNAL_ENVVAR}=${cheExternalUrlStr}`);
            if (cheExternalUrlStr != null) {
                // we only want the authority component.
                const cheExternalUrl = vscode.Uri.parse(cheExternalUrlStr);
                const authority = cheExternalUrl.authority;
                if (authority) {
                    Logger_1.default.i("Setting connection host in Theia to " + authority);
                    return authority;
                }
            }
            Logger_1.default.e(`${Constants_1.default.CHE_API_EXTERNAL_ENVVAR} is not set in the environment or was invalid: falling back to default host`);
        }
        return MCUtil_1.default.getHostnameFrom(url);
    }
    /**
     * Call this whenever the tree needs to be updated - ie when this connection or any of its projects changes.
     */
    onChange(changed) {
        return __awaiter(this, void 0, void 0, function* () {
            if (changed == null) {
                changed = this;
            }
            // Log.d(`Connection ${this.mcUri} changed`);
            CodewindEventListener_1.default.onChange(changed);
            if (!this.isRemote) {
                CodewindEventListener_1.default.onChange(LocalCodewindManager_1.default.instance);
            }
        });
    }
    get projects() {
        return this._projects;
    }
    hasProjectAtPath(path) {
        return this.projects.some((proj) => proj.localPath.fsPath === path.fsPath);
    }
    updateProjects() {
        return __awaiter(this, void 0, void 0, function* () {
            const projectsData = yield Requester_1.default.getProjects(this);
            const oldProjects = this._projects;
            this._projects = [];
            const initPromises = [];
            for (const projectInfo of projectsData) {
                // This is a hard-coded exception for a backend bug where projects get stuck in the Deleting or Validating state
                // and don't go away until they're deleted from the workspace and codewind is restarted.
                if (projectInfo.action === "deleting" || projectInfo.action === "validating") { // non-nls
                    Logger_1.default.e("Project is in a bad state and won't be displayed:", projectInfo);
                    continue;
                }
                let project;
                // If we already have a Project object for this project, just update it, don't make a new object
                const existing = oldProjects.find((p) => p.id === projectInfo.projectID);
                if (existing != null) {
                    project = existing;
                    existing.update(projectInfo);
                    // Log.d("Reuse project " + project.name);
                }
                else {
                    try {
                        project = new Project_1.default(projectInfo, this);
                        initPromises.push(project.initPromise);
                        Logger_1.default.d("New project " + project.name);
                    }
                    catch (err) {
                        Logger_1.default.e(`Error creating new project with ID ${projectInfo.id} name ${projectInfo.name}`, err);
                        vscode.window.showErrorMessage(`Error with new project ${projectInfo.name}: ${MCUtil_1.default.errToString(err)}`);
                    }
                }
                if (project) {
                    this._projects.push(project);
                }
            }
            // Log.d("Awaiting init promises " + Date.now());
            yield Promise.all(initPromises);
            Logger_1.default.d("Done projects update");
            this.onChange();
            return this._projects;
        });
    }
    getProjectByID(projectID) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = this._projects.find((project) => project.id === projectID);
            if (result == null) {
                // Logger.logE(`Couldn't find project with ID ${projectID} on connection ${this}`);
            }
            return result;
        });
    }
    /**
     * Returns if this connection's CW instance is running in Kube.
     * This is the case for remote connections, or the local connection in theia.
     */
    get isKubeConnection() {
        return this.isRemote || global.isTheia;
    }
    needsPushRegistry() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isKubeConnection) {
                // The local connection does not ever need a push registry since the images are deployed to docker for desktop
                return false;
            }
            else if (this._hasHadPushRegistry) {
                // Once the push registry is configured once, we skip that step to save time.
                // if we had one and then the user removed it, codewind-style builds will fail, but the user was warned
                return false;
            }
            // const pushRegistryRes = await vscode.window.withProgress({
            //     cancellable: false,
            //     location: vscode.ProgressLocation.Notification,
            //     title: `Checking for image push registry...`,
            // }, () => {
            //     return Requester.getPushRegistry(this);
            // });
            const pushRegistryRes = yield Requester_1.default.getPushRegistry(this);
            const hasPushRegistry = pushRegistryRes.imagePushRegistry;
            if (hasPushRegistry) {
                this._hasHadPushRegistry = true;
            }
            // If the imagePushRegistry IS set, we do NOT need a push registry (since we already have one)
            return !hasPushRegistry;
        });
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.updateProjects();
        });
    }
    get description() {
        return `(${this.projects.length} projects)`;
    }
    // QuickPick detail
    get detail() {
        return this.url.toString();
    }
    get socketURI() {
        return this._socket ? this._socket.uri : undefined;
    }
    ///// Webview management
    onDidOpenSourcesPage(page) {
        this._sourcesPage = page;
    }
    onDidOpenRegistriesPage(page) {
        this._registriesPage = page;
    }
    get sourcesPage() {
        return this._sourcesPage;
    }
    get registriesPage() {
        return this._registriesPage;
    }
    onDidCloseSourcesPage() {
        this._sourcesPage = undefined;
    }
    onDidCloseRegistriesPage() {
        this._registriesPage = undefined;
    }
}
exports.default = Connection;
//# sourceMappingURL=Connection.js.map