"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Connection_1 = require("./Connection");
const Logger_1 = require("../../Logger");
const CodewindEventListener_1 = require("./CodewindEventListener");
const MCUtil_1 = require("../../MCUtil");
const RemoteConnection_1 = require("./RemoteConnection");
const CLICommandRunner_1 = require("./CLICommandRunner");
const ConnectionMemento_1 = require("./ConnectionMemento");
class ConnectionManager {
    constructor() {
        this._connections = [];
    }
    static get instance() {
        return ConnectionManager._instance || (ConnectionManager._instance = new ConnectionManager());
    }
    activate() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield vscode.window.withProgress({
                    cancellable: false,
                    location: vscode.ProgressLocation.Window,
                    title: "Loading Codewind connections...",
                }, () => {
                    return ConnectionMemento_1.ConnectionMemento.loadSavedConnections();
                });
            }
            catch (err) {
                Logger_1.default.e(`Error loading remote connections`, err);
                vscode.window.showErrorMessage(`Error loading remote connections: ${MCUtil_1.default.errToString(err)}`);
                return;
            }
            CodewindEventListener_1.default.onChange(undefined);
        });
    }
    dispose() {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all([
                this.connections.map((conn) => conn.dispose()),
            ]);
        });
    }
    get connections() {
        return this._connections;
    }
    get remoteConnections() {
        return this.connections.filter((conn) => conn instanceof RemoteConnection_1.default);
    }
    createRemoteConnection(ingressUrl, label, username, password) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i("Recreating new connection to " + ingressUrl);
            const existing = this.remoteConnections.find((conn) => {
                return conn.url.toString() === ingressUrl.toString();
            });
            if (existing) {
                const alreadyExistsMsg = `${existing.label} is already connected to ${ingressUrl.toString()}`;
                throw new Error(alreadyExistsMsg);
            }
            const newConnID = yield ConnectionMemento_1.ConnectionMemento.addConnection(label, ingressUrl, username);
            yield CLICommandRunner_1.CLICommandRunner.updateKeyringCredentials(newConnID, username, password);
            const newData = {
                id: newConnID,
                url: ingressUrl.toString(),
                label: label,
                username: username,
            };
            const newConnection = new RemoteConnection_1.default(ingressUrl, newData);
            yield this.onNewConnection(newConnection);
            return newConnection;
        });
    }
    /**
     * Set up a remote connection that was loaded from `cwctl connections`
     */
    loadRemoteConnection(connectionData) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i("Recreating connection to " + connectionData.url);
            const ingressUrl = vscode.Uri.parse(connectionData.url);
            const loadedConnection = new RemoteConnection_1.default(ingressUrl, connectionData);
            yield this.onNewConnection(loadedConnection);
            return loadedConnection;
        });
    }
    connectLocal(url) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.connections[0] && !this.connections[0].isRemote) {
                return this.connections[0];
            }
            Logger_1.default.i("Creating connection to " + url);
            const label = global.isTheia ? "Codewind in Che" : "Local Codewind";
            const newConnection = new Connection_1.default(Connection_1.LOCAL_CONNECTION_ID, url, label, false);
            this.onNewConnection(newConnection);
            return newConnection;
        });
    }
    onNewConnection(newConnection) {
        return __awaiter(this, void 0, void 0, function* () {
            if (newConnection.isRemote) {
                this.connections.push(newConnection);
            }
            else {
                // the local connection should always be first.
                this.connections.unshift(newConnection);
            }
            Logger_1.default.i("New Connection @ " + newConnection.url);
            // pass undefined here to refresh the tree from its root
            CodewindEventListener_1.default.onChange(undefined);
        });
    }
    removeConnection(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const indexToRemove = this.connections.indexOf(connection);
            if (indexToRemove === -1) {
                Logger_1.default.e(`Request to remove connection ${connection} but it doesn't exist!`);
                return false;
            }
            if (connection.isRemote) {
                yield CLICommandRunner_1.CLICommandRunner.removeConnection(connection.id);
            }
            connection.dispose();
            this.connections.splice(indexToRemove, 1);
            Logger_1.default.i("Removed connection", connection);
            CodewindEventListener_1.default.onChange(undefined);
            return true;
        });
    }
    /**
     * To be called on connection reconnect. Hits the environment endpoint at the given existing Connection's URI,
     * and returns if the response data matches the existing Connection.
     *
     * The given Connection will be destroyed if the data does not match (ie, this function returns `false`),
     * and thus must not do anything further.
     *
     */
    // public async verifyReconnect(connection: Connection): Promise<boolean> {
    //     Log.d("Verifying reconnect at " + connection.mcUri);
    //     let tries = 0;
    //     let newEnvData: CWEnvironment.IMCEnvData | undefined;
    //     // Sometimes this can execute before Portal is ready, resulting in a 404.
    //     while (newEnvData == null && tries < 10) {
    //         tries++;
    //         try {
    //             newEnvData = await CWEnvironment.getEnvData(connection.mcUri);
    //         }
    //         catch (err) {
    //             // wait briefly before trying again
    //             await new Promise( (resolve) => setTimeout(resolve, 250));
    //         }
    //     }
    //     if (newEnvData == null) {
    //         // I don't think this will ever happen
    //         Log.e("Couldn't get a good response from environment endpoint " + connection.mcUri);
    //         vscode.window.showErrorMessage(Translator.t(StringNamespaces.DEFAULT, "failedToReconnect", { uri: connection.mcUri }));
    //         await this.removeConnection(connection);
    //         return false;
    //     }
    //     if (CWEnvironment.envMatches(connection, newEnvData)) {
    //         // it's the same instance, so we don't have to do anything
    //         return true;
    //     }
    //     else {
    //         Log.d("Instance changed on reconnect!");
    //         await this.removeConnection(connection);
    //         // will also add the new Connection to this ConnectionManager
    //         // const newConnection = await activateConnectionCmd(MCUtil.getConnInfoFrom(connection.mcUri));
    //         const newConnection = await activateConnectionCmd();
    //         if (newConnection == null) {
    //             // should never happen
    //             Log.e("Failed to create new connection after verifyReconnect failure");
    //             return false;
    //         }
    //         const msg = Translator.t(StringNamespaces.DEFAULT, "versionChanged",
    //             { uri: connection.mcUri, oldVersion: connection.versionStr, newVersion: newConnection.versionStr }
    //         );
    //         vscode.window.showInformationMessage(msg);
    //         return false;
    //     }
    // }
    get allProjects() {
        return this.connections.reduce((allProjects, connection) => __awaiter(this, void 0, void 0, function* () {
            return (yield allProjects).concat(connection.projects);
        }), Promise.resolve([]));
    }
}
exports.default = ConnectionManager;
//# sourceMappingURL=ConnectionManager.js.map