"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const io = require("socket.io-client");
const Logger_1 = require("../../Logger");
const SocketEvents_1 = require("./SocketEvents");
const Validator_1 = require("../project/Validator");
const ProjectOverviewCmd_1 = require("../../command/project/ProjectOverviewCmd");
const Configurations_1 = require("../../constants/Configurations");
/**
 * Receives and reacts to socket events from Portal
 *
 * Each Connection has exactly one socket
 */
class MCSocket {
    /**
     * Create a SocketIO connection to the server at the given URI.
     * Can throw an error.
     *
     * @param namespace - Socket namespace. Must not start with a slash. Can be the empty string.
     */
    constructor(connection, namespace) {
        this.connection = connection;
        this._isConnected = false;
        this._isAuthorized = false;
        this.onProjectCreation = (payload) => __awaiter(this, void 0, void 0, function* () {
            // https://github.com/eclipse/codewind/issues/720#issuecomment-543801321
            // creation event is now, apparently, the same as changed event
            this.onProjectChanged(payload);
        });
        this.onProjectStatusChanged = (payload) => __awaiter(this, void 0, void 0, function* () {
            // Log.d("onProjectStatusChanged", payload);
            // portal emits the entire inf file with a statusChanged event, so we can treat this the same as projectChanged
            this.onProjectChanged(payload);
        });
        this.onProjectChanged = (payload) => __awaiter(this, void 0, void 0, function* () {
            // Log.d("onProjectChanged", payload);
            // Log.d(`PROJECT CHANGED name=${payload.name} appState=${payload.appStatus} ` +
            // `buildState=${payload.buildStatus} startMode=${payload.startMode}`);
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            project.update(payload);
        });
        this.onProjectClosed = (payload) => __awaiter(this, void 0, void 0, function* () {
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            yield project.clearValidationErrors();
            this.onProjectChanged(payload);
        });
        this.onProjectDeleted = (payload) => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Project deleted", payload.projectID);
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            yield project.onDeletionEvent(payload);
            this.connection.updateProjects();
        });
        this.onProjectRestarted = (payload) => __awaiter(this, void 0, void 0, function* () {
            // Log.d("PROJECT RESTARTED", payload);
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            project.onRestartEvent(payload);
        });
        this.onLogsListChanged = (payload) => __awaiter(this, void 0, void 0, function* () {
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            project.logManager.onLogsListChanged(payload);
        });
        this.onLogUpdate = (payload) => __awaiter(this, void 0, void 0, function* () {
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            // Log.d(`Received log ${payload.logName} of length ${payload.logs.length} with reset ${payload.reset}`);
            project.logManager.onNewLogs(payload);
        });
        this.onProjectValidated = (payload) => __awaiter(this, void 0, void 0, function* () {
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            if (payload.validationResults != null) {
                Validator_1.default.validate(project, payload.validationResults);
            }
            else {
                Logger_1.default.e("Backend didn't send result with validation event");
            }
        });
        this.onProjectSettingsChanged = (payload) => __awaiter(this, void 0, void 0, function* () {
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            // Log.d("projectSettingsChanged", payload);
            return project.onSettingsChangedEvent(payload);
        });
        this.onPushRegistryStatus = (payload) => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`Received push registry status`, payload);
            if (payload.msg) {
                if (payload.imagePushRegistryTest) {
                    vscode.window.showInformationMessage(payload.msg);
                }
                else {
                    vscode.window.showErrorMessage(payload.msg);
                }
            }
        });
        this.getProject = (payload) => __awaiter(this, void 0, void 0, function* () {
            const projectID = payload.projectID;
            if (projectID == null) {
                // Should never happen
                Logger_1.default.e("No projectID in socket event!", payload);
                return undefined;
            }
            if (this.refreshingProjectsProm) {
                yield this.refreshingProjectsProm;
            }
            const result = yield this.connection.getProjectByID(projectID);
            if (result == null) {
                Logger_1.default.w(`${this} received socket event for nonexistent project`, projectID);
                this.refreshingProjectsProm = this.connection.updateProjects();
                yield this.refreshingProjectsProm;
                this.refreshingProjectsProm = undefined;
                const newProject = yield this.connection.getProjectByID(projectID);
                if (newProject) {
                    Logger_1.default.i(`Project ${newProject.name} has been created`);
                    if (Configurations_1.CWConfigurations.OVERVIEW_ON_CREATION.get()) {
                        ProjectOverviewCmd_1.default(newProject);
                    }
                    if (!newProject.isInVSCodeWorkspace) {
                        vscode.window.showWarningMessage(`${newProject.name} is not in your VS Code workspace. ` +
                            `Right click the project in the Codewind view and run the Add Project to Workspace command to add it.`);
                    }
                    return newProject;
                }
                else {
                    Logger_1.default.e(`Still did not find project with ID ${projectID} after refreshing projects list, `
                        + `${this.connection.projects.length} projects were found`);
                }
            }
            return result;
        });
        this.uri = connection.url.toString();
        if (namespace) {
            if (!this.uri.endsWith("/")) {
                this.uri += "/";
            }
            this.uri += namespace;
        }
        Logger_1.default.i("Creating MCSocket for URI", this.uri);
        const usingHttps = connection.url.scheme === "https";
        const timeout = connection.isKubeConnection ? 15000 : 5000;
        const options = {
            rejectUnauthorized: !usingHttps,
            secure: usingHttps,
            timeout,
        };
        if (!connection.isRemote) {
            Logger_1.default.d(`${this} does not require authorization`);
            this._isAuthorized = true;
        }
        this.socket = io(this.uri, options);
        this.socket.connect();
        this.socket
            .on("connect", () => {
            this._isConnected = true;
            this.connection.onConnect();
        }) // non-nls
            .on("disconnect", () => {
            this._isConnected = false;
            this.connection.onDisconnect();
        }) // non-nls
            // .on(SocketEvents.Types.PROJECT_BOUND,           this.onProjectBound)
            .on(SocketEvents_1.default.Types.PROJECT_CREATED, this.onProjectCreation)
            .on(SocketEvents_1.default.Types.PROJECT_CHANGED, this.onProjectChanged)
            .on(SocketEvents_1.default.Types.PROJECT_STATUS_CHANGED, this.onProjectStatusChanged)
            .on(SocketEvents_1.default.Types.PROJECT_CLOSED, this.onProjectClosed)
            .on(SocketEvents_1.default.Types.PROJECT_DELETION, this.onProjectDeleted)
            .on(SocketEvents_1.default.Types.PROJECT_RESTART_RESULT, this.onProjectRestarted)
            .on(SocketEvents_1.default.Types.PROJECT_VALIDATED, this.onProjectValidated)
            .on(SocketEvents_1.default.Types.PROJECT_SETTING_CHANGED, this.onProjectSettingsChanged)
            .on(SocketEvents_1.default.Types.LOG_UPDATE, this.onLogUpdate)
            .on(SocketEvents_1.default.Types.LOGS_LIST_CHANGED, this.onLogsListChanged)
            .on(SocketEvents_1.default.Types.REGISTRY_STATUS, this.onPushRegistryStatus);
    }
    get isConnected() {
        return this._isConnected;
    }
    get isAuthorized() {
        return this._isAuthorized;
    }
    get isReady() {
        return this._isConnected && this._isAuthorized;
    }
    /**
     * This MUST be called when the connection is removed.
     * If there are multiple sockets listening on the same connection,
     * the callbacks will be fired multiple times for the same event, which will lead to serious misbehaviour.
     */
    dispose() {
        return __awaiter(this, void 0, void 0, function* () {
            this.connection.onDisconnect();
            this.socket.disconnect();
            this._isConnected = false;
            this._isAuthorized = false;
        });
    }
    authenticate(token) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i(`Authenticating ${this}`);
            return new Promise((resolve, reject) => {
                this.socket.emit("authentication", { token });
                const timeoutS = 10;
                const timeout = setTimeout(() => {
                    reject(`Socket at ${this.uri} did not respond to authentication request within ${timeoutS} seconds. Try refreshing the connection.`);
                }, timeoutS * 1000);
                this.socket.on("authenticated", () => {
                    Logger_1.default.i(`Successfully authenticated ${this}`);
                    // The authorization stays valid until we call socket.disconnect()
                    this._isAuthorized = true;
                    clearTimeout(timeout);
                    resolve();
                });
                this.socket.on("unauthorized", (payload) => {
                    Logger_1.default.e(`${this} received unauthorized event`, payload);
                    clearTimeout(timeout);
                    reject(payload.message);
                });
            });
        });
    }
    toString() {
        return "MCSocket @ " + this.uri; // not displayed to user        // non-nls
    }
}
exports.default = MCSocket;
//# sourceMappingURL=MCSocket.js.map