"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const ManageRegistriesCmd_1 = require("../../command/connection/ManageRegistriesCmd");
const InputUtil_1 = require("../../InputUtil");
const Requester_1 = require("../project/Requester");
const Logger_1 = require("../../Logger");
const MCUtil_1 = require("../../MCUtil");
class ContainerRegistry {
    constructor(address, username, _namespace = "") {
        this.address = address;
        this.username = username;
        this._namespace = _namespace;
        this._isPushRegistry = false;
    }
    toString() {
        return `${this.username}@${this.fullAddress}`;
    }
    get isPushRegistry() {
        return this._isPushRegistry;
    }
    set isPushRegistry(isPushRegistry) {
        this._isPushRegistry = isPushRegistry;
    }
    get namespace() {
        return this._namespace;
    }
    set namespace(ns) {
        this._namespace = ns;
    }
    get fullAddress() {
        if (this.namespace) {
            return `${this.address}/${this.namespace}`;
        }
        return this.address;
    }
}
exports.ContainerRegistry = ContainerRegistry;
var RegistryUtils;
(function (RegistryUtils) {
    /**
     * Returns if this connection MUST set up a push registry before it can build a project of this type.
     *
     * If the project type requires a push registry and the connection does not have a push registry,
     * shows an error message and returns true.
     */
    function doesNeedPushRegistry(internalType, connection) {
        return __awaiter(this, void 0, void 0, function* () {
            if (doesUsePushRegistry(internalType) && (yield connection.needsPushRegistry())) {
                const manageRegistriesBtn = "Image Registry Manager";
                vscode.window.showErrorMessage(`Codewind style projects on Kubernetes require an Image Push Registry to be configured. ` +
                    `Add a push registry using the Image Registry Manager.`, manageRegistriesBtn)
                    .then((res) => {
                    if (res === manageRegistriesBtn) {
                        ManageRegistriesCmd_1.default(connection);
                    }
                });
                return true;
            }
            return false;
        });
    }
    RegistryUtils.doesNeedPushRegistry = doesNeedPushRegistry;
    function doesUsePushRegistry(projectType) {
        // these two extension types do not require a push registry
        return projectType !== "appsodyExtension" && projectType !== "odo";
    }
    const newRegistryWizardSteps = [
        {
            promptGenerator: () => `Enter the registry's base address (domain). Do not specify a namespace.`,
            placeholder: `docker.io`,
        },
        {
            promptGenerator: (address) => `Enter the username for ${address}.`,
        },
        {
            promptGenerator: (address, username) => `Enter the password or an API key for ${username} @ ${address}.`,
            password: true,
            allowEmpty: true,
        },
    ];
    /**
     * Run the Add New Image Registry wizard.
     * @returns true if a new registry was successfully added, false if the user cancelled.
     */
    function addNewRegistry(connection, existingRegistries) {
        return __awaiter(this, void 0, void 0, function* () {
            const wizardTitle = "Sign in to a new Image Registry";
            newRegistryWizardSteps[0].validator = (input) => {
                return validateAddress(input, existingRegistries.map((reg) => reg.address));
            };
            const inputResult = yield InputUtil_1.default.runMultiStepInput(wizardTitle, newRegistryWizardSteps);
            if (inputResult == null) {
                return false;
            }
            const [address, username, password] = inputResult;
            // https://github.com/eclipse/codewind/issues/1469
            const hasPushRegistry = existingRegistries.some((registry) => registry.isPushRegistry);
            const needsPushRegistry = !hasPushRegistry && (yield connection.templateSourcesList.hasCodewindSourceEnabled());
            let setAsPushRegistry;
            let namespace;
            if (needsPushRegistry) {
                Logger_1.default.d(`Push registry is required`);
                setAsPushRegistry = true;
            }
            else {
                setAsPushRegistry = yield promptSetAsPushRegistry(address);
                if (setAsPushRegistry == null) {
                    Logger_1.default.d(`Push registry prompt cancelled; not adding registry.`);
                    // cancel
                    return false;
                }
            }
            if (setAsPushRegistry) {
                namespace = yield promptForNamespace(address, username);
                if (namespace == null) {
                    Logger_1.default.d(`Namespace prompt cancelled; not adding registry.`);
                    // cancel
                    return false;
                }
            }
            const newRegistry = yield vscode.window.withProgress({
                cancellable: false,
                location: vscode.ProgressLocation.Notification,
                title: `Creating registry secret for ${username} @ ${address}...`,
            }, () => __awaiter(this, void 0, void 0, function* () {
                const newRegistry_ = yield Requester_1.default.addRegistrySecret(connection, address, username, password);
                Logger_1.default.i(`Successfully added image registry ${address}`);
                return newRegistry_;
            }));
            if (newRegistry == null) {
                Logger_1.default.e(`New registry secret appeared to be created but was null`);
            }
            if (setAsPushRegistry) {
                if (newRegistry == null) {
                    throw new Error(`Unknown error creating new registry secret`);
                }
                let didSetPush = false;
                try {
                    const currentPushRegistry = existingRegistries.find((reg) => reg.isPushRegistry);
                    didSetPush = (yield setPushRegistry(connection, currentPushRegistry, newRegistry, false, namespace)) != null;
                }
                catch (err) {
                    const errMsg = `Failed to set push registry to ${newRegistry.fullAddress} after adding`;
                    Logger_1.default.e(errMsg, err);
                    vscode.window.showErrorMessage(`${errMsg}: ${MCUtil_1.default.errToString(err)}`);
                }
                if (!didSetPush && needsPushRegistry) {
                    vscode.window.showWarningMessage(`You added a registry secret, but did not set it as your image push registry. ` +
                        `Codewind-style projects will be unable to build until you add a push registry.`);
                }
            }
            return true;
        });
    }
    RegistryUtils.addNewRegistry = addNewRegistry;
    function promptSetAsPushRegistry(address) {
        return __awaiter(this, void 0, void 0, function* () {
            const setAsPushOption = {
                label: `Set ${address} as your image push registry`,
                detail: `Codewind-style project images will be pushed to this registry.`
            };
            const dontSetAsPushOption = {
                label: `Don't set ${address} as image push registry`,
                detail: `This registry can still be used to pull private images.`
            };
            const response = yield vscode.window.showQuickPick([
                setAsPushOption, dontSetAsPushOption,
            ], {
                canPickMany: false,
                ignoreFocusOut: true,
                placeHolder: `Would you like to push your built Codewind project images to this registry?`
            });
            if (response == null) {
                return undefined;
            }
            return response === setAsPushOption;
        });
    }
    function promptForNamespace(registryAddress, registryUsername) {
        return __awaiter(this, void 0, void 0, function* () {
            const namespace = yield vscode.window.showInputBox({
                ignoreFocusOut: true,
                placeHolder: registryUsername,
                prompt: `Enter the namespace to push to ${registryAddress} under, or leave the namespace empty. ` +
                    `For example, to push to docker.io/eclipse, enter the "eclipse" namespace.`,
                validateInput: validateIsOnlyUrlChars,
            });
            return namespace;
        });
    }
    function setPushRegistry(connection, currentPushRegistry, newPushRegistry, showProgress, namespace) {
        return __awaiter(this, void 0, void 0, function* () {
            // the secret for this registry must already have been created
            /*
            const yesBtn = "Test";
    
            const doTestResponse = await vscode.window.showInformationMessage(
                `Would you like to test deploying a Hello World image to ${pushRegistry.fullAddress} as ${pushRegistry.username} right now?`,
                yesBtn
            );
    
            // Log.d("doTestResponse", doTestResponse);
    
            if (doTestResponse === yesBtn) {
                const testResult = await testRegistry(connection, pushRegistry);
                if (!testResult) {
                    // user did not get a successful test
                    return;
                }
                else if (testResult === "retry") {
                    return setPushRegistry(connection, pushRegistry);
                }
                // else test succeeded, or they selected to override the failed test
            }
            */
            if (currentPushRegistry) {
                const confirmBtn = "Change Push Registry";
                const confirmRes = yield vscode.window.showInformationMessage(`Codewind project images will not be pushed to ${newPushRegistry.fullAddress} until the next build. ` +
                    `\nAlso note that if you wish to switch your push registry back, you will have to re-enter the namespace.`, { modal: true }, confirmBtn);
                if (confirmRes !== confirmBtn) {
                    return;
                }
            }
            Logger_1.default.d(`Setting push registry to ${newPushRegistry.address}`);
            if (!namespace) {
                namespace = yield promptForNamespace(newPushRegistry.address, newPushRegistry.username);
                if (namespace == null) {
                    return undefined;
                }
            }
            newPushRegistry.namespace = namespace;
            const setPushProm = Requester_1.default.setPushRegistry(connection, newPushRegistry);
            if (showProgress) {
                yield vscode.window.withProgress({
                    cancellable: false,
                    location: vscode.ProgressLocation.Notification,
                    title: `Setting image push registry to ${newPushRegistry.fullAddress}...`,
                }, () => __awaiter(this, void 0, void 0, function* () {
                    yield setPushProm;
                }));
            }
            else {
                yield setPushProm;
            }
            Logger_1.default.i(`Push registry set successfully for ${connection.url} to ${newPushRegistry.fullAddress}`);
            return newPushRegistry;
        });
    }
    RegistryUtils.setPushRegistry = setPushRegistry;
    /*
    async function testRegistry(connection: Connection, registry: ContainerRegistry): Promise<boolean | "retry"> {
        const testResult = await vscode.window.withProgress({
            title: `Pushing a test image to ${registry}...`,
            location: vscode.ProgressLocation.Notification,
            cancellable: false,
        }, () => {
            try {
                return Requester.testPushRegistry(connection, registry);
            }
            catch (err) {
                Log.e("Error testing registry", err);
                return Promise.resolve({ imagePushRegistryTest: false, msg: MCUtil.errToString(err) });
            }
        });

        // tslint:disable-next-line: no-boolean-literal-compare
        if (testResult.imagePushRegistryTest === true) {
            vscode.window.showInformationMessage("Deployment registry test succeeded");
            return true;
        }

        const tryAgainBtn = "Try Again";
        const overrideBtn = "Set Anyway";
        const pushFailedRes = await vscode.window.showWarningMessage(
            `Pushing to "${registry.fullAddress}" failed`,
            { modal: true },
            tryAgainBtn,
            overrideBtn,
        );

        if (pushFailedRes == null) {
            return false;
        }
        else if (pushFailedRes === tryAgainBtn) {
            return testRegistry(connection, registry);
        }
        else {
            // override
            return true;
        }
    }*/
    function validateIsOnlyUrlChars(input) {
        const legal = /^[A-Za-z0-9-._~:/?#\[\]@!\$&'\(\)\*\+;%=,]*$/;
        if (!legal.test(input)) {
            return `Invalid input: "${input}" contains illegal characters.`;
        }
        return undefined;
    }
    RegistryUtils.validateIsOnlyUrlChars = validateIsOnlyUrlChars;
    const PROTOCOL_SEP = "://";
    function validateAddress(input, existingAddresses) {
        const illegalCharsMsg = validateIsOnlyUrlChars(input);
        if (illegalCharsMsg) {
            return illegalCharsMsg;
        }
        if (input.includes(PROTOCOL_SEP)) {
            return "Don't include a protocol in your registry address.";
        }
        const existing = existingAddresses
            .map((addr) => {
            // The backend may have added a protocol.
            const protocolIndex = addr.search(PROTOCOL_SEP);
            if (protocolIndex > 0) {
                return addr.substring(protocolIndex + PROTOCOL_SEP.length);
            }
            return addr;
        })
            .find(((addr) => addr === input));
        if (existing) {
            return `You already have a registry at ${existing}. You can only have one login for a given registry at a time. Remove the existing registry to add a new one at the same address.`;
        }
        return undefined;
    }
    RegistryUtils.validateAddress = validateAddress;
})(RegistryUtils || (RegistryUtils = {}));
exports.default = RegistryUtils;
//# sourceMappingURL=RegistryUtils.js.map