"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Connection_1 = require("./Connection");
const ConnectionState_1 = require("./ConnectionState");
const CLICommandRunner_1 = require("./CLICommandRunner");
const Logger_1 = require("../../Logger");
const codewind_filewatcher_1 = require("codewind-filewatcher");
const FWAuthToken_1 = require("codewind-filewatcher/lib/FWAuthToken");
const ConnectionMemento_1 = require("./ConnectionMemento");
const Requester_1 = require("../project/Requester");
class RemoteConnection extends Connection_1.default {
    constructor(ingressUrl, cliData) {
        super(cliData.id, ingressUrl, cliData.label, true);
        this.updateCredentialsPromise = Promise.resolve();
        this._username = cliData.username;
    }
    enable() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.shouldToggle()) {
                return;
            }
            this.currentToggleOperation = "connecting";
            try {
                yield this.enableInner();
            }
            catch (err) {
                throw err;
            }
            finally {
                this.currentToggleOperation = undefined;
            }
        });
    }
    enableInner() {
        const _super = Object.create(null, {
            enable: { get: () => super.enable }
        });
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`${this.label} starting remote enable`);
            // Make sure the ingress is reachable before trying anything else
            // https://github.com/eclipse/codewind/issues/1547
            let canPing = false;
            try {
                canPing = yield Requester_1.default.ping(this.url);
            }
            catch (err) {
                // ping failed
            }
            if (!canPing) {
                this.setState(ConnectionState_1.ConnectionStates.DISABLED);
                throw new Error(`Failed to connect to ${this.url}. Make sure the Codewind instance is running, and reachable from your machine.`);
            }
            let token;
            try {
                token = (yield this.getAccessToken()).access_token;
            }
            catch (err) {
                this.setState(ConnectionState_1.ConnectionStates.AUTH_ERROR);
                throw err;
            }
            try {
                yield _super.enable.call(this);
                if (!this._socket) {
                    throw new Error(`${this.label} socket was undefined after enabling appeared to succeed`);
                }
            }
            catch (err) {
                this.setState(ConnectionState_1.ConnectionStates.DISABLED);
                throw err;
            }
            try {
                yield this._socket.authenticate(token);
            }
            catch (err) {
                this.setState(ConnectionState_1.ConnectionStates.AUTH_ERROR);
                throw err;
            }
            this.setState(ConnectionState_1.ConnectionStates.READY);
            Logger_1.default.d(`${this} finished remote enable`);
        });
    }
    disable() {
        const _super = Object.create(null, {
            disable: { get: () => super.disable }
        });
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.shouldToggle()) {
                return;
            }
            this.currentToggleOperation = "disconnecting";
            try {
                yield _super.disable.call(this);
                this.setState(ConnectionState_1.ConnectionStates.DISABLED);
            }
            catch (err) {
                throw err;
            }
            finally {
                this.currentToggleOperation = undefined;
            }
        });
    }
    dispose() {
        const _super = Object.create(null, {
            dispose: { get: () => super.dispose }
        });
        return __awaiter(this, void 0, void 0, function* () {
            if (this.overviewPage) {
                this.overviewPage.dispose();
            }
            if (this.sourcesPage) {
                this.sourcesPage.dispose();
            }
            if (this.registriesPage) {
                this.registriesPage.dispose();
            }
            yield _super.dispose.call(this);
        });
    }
    createFileWatcher(cliPath) {
        return __awaiter(this, void 0, void 0, function* () {
            return codewind_filewatcher_1.CreateFileWatcher(this.url.toString(), Logger_1.default.getLogDir, undefined, cliPath, {
                getLatestAuthToken: () => {
                    if (!this._accessToken) {
                        return undefined;
                    }
                    return new FWAuthToken_1.FWAuthToken(this._accessToken.access_token, this._accessToken.token_type);
                },
                informReceivedInvalidAuthToken: () => {
                    this._accessToken = undefined;
                    // Invalidate and retart the process of getting a new access token
                    this.getAccessToken();
                }
            });
        });
    }
    /**
     * Returns true if there is NOT currently an enable/disable operation in progress.
     * If there is one, shows a message, and enable/disable should be blocked by the caller.
     */
    shouldToggle() {
        if (this.currentToggleOperation) {
            vscode.window.showWarningMessage(`${this.label} is already ${this.currentToggleOperation}.`);
            return false;
        }
        return true;
    }
    setState(newState) {
        super.setState(newState);
        this.tryRefreshOverview();
    }
    updateCredentials(username, password) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i(`Start updateCredentials for ${this}`);
            this._username = username;
            yield ConnectionMemento_1.ConnectionMemento.save(this.cliData);
            // Just in case there are multiple quick credentials updates
            yield this.updateCredentialsPromise;
            // Invalidate the old access token which used the old credentials
            this._accessToken = undefined;
            this.updateCredentialsPromise = CLICommandRunner_1.CLICommandRunner.updateKeyringCredentials(this.id, username, password);
            yield this.updateCredentialsPromise;
            Logger_1.default.i("Finished updating keyring credentials");
            try {
                if (this.state !== ConnectionState_1.ConnectionStates.DISABLED) {
                    Logger_1.default.d(`Refreshing access token after credentials update`);
                    yield this.getAccessToken();
                }
            }
            finally {
                this.tryRefreshOverview();
                Logger_1.default.d(`Finished updateCredentials`);
            }
        });
    }
    getAccessToken() {
        return __awaiter(this, void 0, void 0, function* () {
            // if a credential update is in progress, let that complete before trying to get the access token, or we'll get an invalid result
            yield this.updateCredentialsPromise;
            if (this._accessToken) {
                return this._accessToken;
            }
            Logger_1.default.d(`${this.label} looking up access token for user "${this._username}"`);
            try {
                this._accessToken = yield CLICommandRunner_1.CLICommandRunner.getAccessToken(this.id, this._username);
                if (this._socket && this._socket.isConnected && !this._socket.isAuthorized) {
                    yield this._socket.authenticate(this._accessToken.access_token);
                }
                if (this.state === ConnectionState_1.ConnectionStates.AUTH_ERROR) {
                    this.setState(ConnectionState_1.ConnectionStates.READY);
                    yield this.refresh();
                }
                return this._accessToken;
            }
            catch (err) {
                this._accessToken = undefined;
                this.setState(ConnectionState_1.ConnectionStates.AUTH_ERROR);
                throw err;
            }
        });
    }
    get username() {
        return this._username;
    }
    get cliData() {
        return {
            id: this.id,
            label: this.label,
            url: this.url.toString(),
            username: this._username,
        };
    }
    refresh() {
        const _super = Object.create(null, {
            refresh: { get: () => super.refresh }
        });
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isConnected) {
                yield _super.refresh.call(this);
                return;
            }
            if (!this.shouldToggle()) {
                return;
            }
            yield this.disable();
            yield this.enable();
        });
    }
    get overviewPage() {
        return this._activeOverviewPage;
    }
    onDidOpenOverview(overviewPage) {
        this._activeOverviewPage = overviewPage;
    }
    onDidCloseOverview() {
        if (this._activeOverviewPage) {
            this._activeOverviewPage = undefined;
        }
    }
    tryRefreshOverview() {
        if (this._activeOverviewPage) {
            this._activeOverviewPage.refresh();
        }
    }
}
exports.default = RemoteConnection;
//# sourceMappingURL=RemoteConnection.js.map