"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const readline = require("readline");
const fs = require("fs");
const Logger_1 = require("../../../Logger");
const StringNamespaces_1 = require("../../../constants/strings/StringNamespaces");
const translator_1 = require("../../../constants/strings/translator");
const CodewindStates_1 = require("./CodewindStates");
const MCUtil_1 = require("../../../MCUtil");
const Constants_1 = require("../../../constants/Constants");
const CLIWrapper_1 = require("../CLIWrapper");
const Commands_1 = require("../../../constants/Commands");
const LocalCodewindManager_1 = require("./LocalCodewindManager");
const CLILifecycleCommands_1 = require("./CLILifecycleCommands");
const CLICommandRunner_1 = require("../CLICommandRunner");
const util_1 = require("util");
const CWDocs_1 = require("../../../constants/CWDocs");
const STRING_NS = StringNamespaces_1.default.STARTUP;
const TAG_OPTION = "-t";
const TAG_LATEST = "latest";
// Codewind tag to install if CW_TAG is not set in env
// UPDATE THIS on release branches to eg "0.6.0"
const DEFAULT_CW_TAG = "0.8.0";
var CLILifecycleWrapper;
(function (CLILifecycleWrapper) {
    // serves as a lock, only one operation at a time.
    let currentOperation;
    function isRunning() {
        return currentOperation !== undefined;
    }
    CLILifecycleWrapper.isRunning = isRunning;
    const ALREADY_RUNNING_WARNING = "Please wait for the current operation to finish.";
    function runLifecycleCmd(cmd, tagOverride) {
        return __awaiter(this, void 0, void 0, function* () {
            if (currentOperation) {
                vscode.window.showWarningMessage(ALREADY_RUNNING_WARNING);
                throw new Error(CLIWrapper_1.default.CLI_CMD_CANCELLED);
            }
            const args = [];
            const tag = tagOverride || getTag();
            if (tagOverride || cmd.usesTag) {
                args.push(TAG_OPTION, tag);
            }
            const beforeCmdState = LocalCodewindManager_1.default.instance.state;
            const transitionStates = cmd.transitionStates;
            if (transitionStates && transitionStates.during) {
                LocalCodewindManager_1.default.instance.setState(transitionStates.during);
            }
            let progressTitle;
            // For STOP the CLI output looks better by itself, so we don't display any extra title
            if (cmd !== CLILifecycleCommands_1.CLILifecycleCommands.STOP) {
                progressTitle = cmd.getUserActionName(tag);
            }
            try {
                currentOperation = cmd;
                yield CLIWrapper_1.default.cliExec(cmd, args, progressTitle);
            }
            catch (err) {
                if (CLIWrapper_1.default.isCancellation(err)) {
                    // restore original state
                    LocalCodewindManager_1.default.instance.setState(beforeCmdState);
                }
                else if (transitionStates && transitionStates.onError) {
                    LocalCodewindManager_1.default.instance.setState(transitionStates.onError);
                }
                currentOperation = undefined;
                throw err;
            }
            if (transitionStates && transitionStates.after) {
                LocalCodewindManager_1.default.instance.setState(transitionStates.after);
            }
            currentOperation = undefined;
        });
    }
    function getCodewindUrl() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = (yield CLICommandRunner_1.CLICommandRunner.status()).url;
            if (!url) {
                return undefined;
            }
            return vscode.Uri.parse(url);
        });
    }
    CLILifecycleWrapper.getCodewindUrl = getCodewindUrl;
    function getCodewindStartedStatus(status) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!status) {
                try {
                    status = yield CLICommandRunner_1.CLICommandRunner.status();
                }
                catch (err) {
                    if (err.toString().toLowerCase().includes("docker daemon")) {
                        // Work around exception when docker is not running
                        return "stopped";
                    }
                    throw err;
                }
            }
            if (status.started.length > 0) {
                if (status.started.includes(getTag())) {
                    Logger_1.default.i("The correct version of Codewind is already started");
                    return "started-correct-version";
                }
                return "started-wrong-version";
            }
            return "stopped";
        });
    }
    CLILifecycleWrapper.getCodewindStartedStatus = getCodewindStartedStatus;
    function installAndStart() {
        return __awaiter(this, void 0, void 0, function* () {
            const status = yield CLICommandRunner_1.CLICommandRunner.status();
            const tag = getTag();
            let hadOldVersionRunning = false;
            Logger_1.default.i(`Ready to install and start Codewind ${tag}`);
            const startedStatus = yield getCodewindStartedStatus(status);
            if (startedStatus === "stopped") {
                Logger_1.default.i("Codewind is not running");
            }
            else if (startedStatus === "started-wrong-version") {
                Logger_1.default.i(`The wrong version of Codewind ${status.started[0]} is currently started`);
                const okBtn = "OK";
                const resp = yield vscode.window.showWarningMessage(`The locally running version of the Codewind backend (${status.started[0]}) is out-of-date, ` +
                    `and not compatible with this version of the extension. Codewind will now stop and upgrade to the new version.`, { modal: true }, okBtn);
                if (resp !== okBtn) {
                    Logger_1.default.i("User rejected backend upgrade with CW started");
                    vscode.window.showWarningMessage(translator_1.default.t(STRING_NS, "backendUpgradeRequired", { tag }));
                    throw new Error(CLIWrapper_1.default.CLI_CMD_CANCELLED);
                }
                yield stop();
                hadOldVersionRunning = true;
            }
            if (!status["installed-versions"].includes(tag)) {
                Logger_1.default.i(`Codewind ${tag} is NOT installed, installed versions are ${status["installed-versions"].join(", ")}`);
                // If they had an old version running, they have already agreed to update to the new version
                let promptForInstall = !hadOldVersionRunning;
                if (promptForInstall) {
                    const wrongVersionInstalled = status["installed-versions"].length > 0;
                    if (wrongVersionInstalled) {
                        yield onWrongVersionInstalled(status["installed-versions"], tag);
                        promptForInstall = false;
                    }
                }
                try {
                    yield install(promptForInstall);
                }
                catch (err) {
                    if (CLIWrapper_1.default.isCancellation(err)) {
                        throw err;
                    }
                    LocalCodewindManager_1.default.instance.setState(CodewindStates_1.CodewindStates.ERR_INSTALLING);
                    Logger_1.default.e("Error installing codewind", err);
                    throw new Error("Error installing Codewind: " + MCUtil_1.default.errToString(err));
                }
            }
            else {
                Logger_1.default.i(`Codewind ${tag} is already installed`);
            }
            try {
                yield runLifecycleCmd(CLILifecycleCommands_1.CLILifecycleCommands.START);
            }
            catch (err) {
                if (CLIWrapper_1.default.isCancellation(err)) {
                    throw err;
                }
                Logger_1.default.e("Error starting codewind", err);
                throw new Error("Error starting Codewind: " + MCUtil_1.default.errToString(err));
            }
            if (yield isWorkspaceMigrationReqd(status.started, status["installed-versions"])) {
                yield doWorkspaceUpgrade();
            }
        });
    }
    CLILifecycleWrapper.installAndStart = installAndStart;
    function onWrongVersionInstalled(installedVersions, requiredTag) {
        return __awaiter(this, void 0, void 0, function* () {
            const installedVersionsStr = MCUtil_1.default.joinWithAnd(installedVersions);
            const yesBtn = "OK";
            const resp = yield vscode.window.showWarningMessage(`You currently have Codewind ${installedVersionsStr} installed, ` +
                `but ${requiredTag} is required to use this version of the extension. Install Codewind ${requiredTag} now?`, { modal: true }, yesBtn);
            if (resp !== yesBtn) {
                Logger_1.default.i("User rejected backend upgrade");
                vscode.window.showWarningMessage(translator_1.default.t(STRING_NS, "backendUpgradeRequired", { tag: requiredTag }));
                throw new Error(CLIWrapper_1.default.CLI_CMD_CANCELLED);
            }
            // Remove unwanted versions (ie all versions that are installed, since none of them are the required version)
            yield removeAllImages();
        });
    }
    function install(promptForInstall) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i("Installing Codewind");
            const installAffirmBtn = translator_1.default.t(STRING_NS, "installAffirmBtn");
            const moreInfoBtn = translator_1.default.t(STRING_NS, "moreInfoBtn");
            let response;
            if (!promptForInstall || process.env[Constants_1.default.CW_ENV_VAR] === Constants_1.default.CW_ENV_TEST) {
                response = installAffirmBtn;
            }
            else {
                Logger_1.default.d("Prompting for install confirm");
                response = yield vscode.window.showInformationMessage(translator_1.default.t(STRING_NS, "installPrompt"), { modal: true }, installAffirmBtn, moreInfoBtn);
            }
            if (response === installAffirmBtn) {
                try {
                    yield runLifecycleCmd(CLILifecycleCommands_1.CLILifecycleCommands.INSTALL);
                    // success
                    vscode.window.showInformationMessage(translator_1.default.t(StringNamespaces_1.default.STARTUP, "installCompleted", { version: getTag() }), translator_1.default.t(StringNamespaces_1.default.STARTUP, "okBtn"));
                    Logger_1.default.i("Codewind installed successfully");
                }
                catch (err) {
                    if (CLIWrapper_1.default.isCancellation(err)) {
                        throw err;
                    }
                    Logger_1.default.e("Install failed", err);
                    LocalCodewindManager_1.default.instance.setState(CodewindStates_1.CodewindStates.ERR_INSTALLING);
                    CLIWrapper_1.default.showCLIError(MCUtil_1.default.errToString(err));
                    return onInstallFailOrReject(false);
                }
            }
            else if (response === moreInfoBtn) {
                onMoreInfo();
                return onInstallFailOrReject(true);
            }
            else {
                Logger_1.default.i("User rejected installation");
                // they pressed Cancel
                return onInstallFailOrReject(true);
            }
        });
    }
    function stop() {
        return __awaiter(this, void 0, void 0, function* () {
            return runLifecycleCmd(CLILifecycleCommands_1.CLILifecycleCommands.STOP);
        });
    }
    CLILifecycleWrapper.stop = stop;
    function removeAllImages() {
        return __awaiter(this, void 0, void 0, function* () {
            const installedVersions = (yield CLICommandRunner_1.CLICommandRunner.status())["installed-versions"];
            for (const unwantedVersion of installedVersions) {
                yield runLifecycleCmd(CLILifecycleCommands_1.CLILifecycleCommands.REMOVE, unwantedVersion);
            }
        });
    }
    CLILifecycleWrapper.removeAllImages = removeAllImages;
    function onInstallFailOrReject(rejected) {
        return __awaiter(this, void 0, void 0, function* () {
            let msg;
            if (rejected) {
                msg = translator_1.default.t(STRING_NS, "installRejected");
            }
            else {
                msg = translator_1.default.t(STRING_NS, "installFailed");
            }
            const moreInfoBtn = translator_1.default.t(STRING_NS, "moreInfoBtn");
            const tryAgainBtn = translator_1.default.t(StringNamespaces_1.default.STARTUP, "tryAgainBtn");
            return vscode.window.showWarningMessage(msg, moreInfoBtn, tryAgainBtn, translator_1.default.t(STRING_NS, "okBtn"))
                .then((res) => {
                if (res === tryAgainBtn) {
                    return install(false);
                }
                else if (res === moreInfoBtn) {
                    onMoreInfo();
                    return onInstallFailOrReject(true);
                }
                return Promise.reject(CLIWrapper_1.default.CLI_CMD_CANCELLED);
            });
        });
    }
    function onMoreInfo() {
        const moreInfoUrl = CWDocs_1.default.getDocLink(CWDocs_1.default.INSTALL_INFO);
        vscode.commands.executeCommand(Commands_1.default.VSC_OPEN, moreInfoUrl);
    }
    function updateProgress(cmd, stdout, progress) {
        const isInstallCmd = cmd === CLILifecycleCommands_1.CLILifecycleCommands.INSTALL;
        const reader = readline.createInterface(stdout);
        reader.on("line", (line) => {
            if (!line) {
                return;
            }
            if (!isInstallCmd) {
                // simple case for non-install, just update with the output
                progress.report({ message: line });
                return;
            }
            if (line === "Image Tagging Successful") {
                return;
            }
            // With JSON flag, `install` output is JSON we can parse to give good output
            let lineObj;
            try {
                lineObj = JSON.parse(line);
            }
            catch (err) {
                Logger_1.default.e(`Error parsing JSON from CLI output, line was "${line}"`);
                return;
            }
            // we're interested in lines like:
            // {"status":"Pulling from codewind-pfe-amd64","id":"latest"}
            const pullingFrom = "Pulling from";
            if (line.includes(pullingFrom)) {
                const imageTag = lineObj.id;
                const message = lineObj.status + ":" + imageTag;
                progress.report({ message });
            }
        });
    }
    CLILifecycleWrapper.updateProgress = updateProgress;
    function isDevEnv() {
        const env = process.env[Constants_1.default.CW_ENV_VAR];
        return env === Constants_1.default.CW_ENV_DEV || env === Constants_1.default.CW_ENV_TEST;
    }
    function getTag() {
        let tag = DEFAULT_CW_TAG;
        const versionVarValue = process.env[Constants_1.default.CW_ENV_TAG_VAR];
        if (versionVarValue) {
            tag = versionVarValue;
        }
        else if (isDevEnv()) {
            tag = TAG_LATEST;
        }
        return tag;
    }
    /**
     * We have to do a cwctl upgrade (workspace migration) if the old version was older than 0.6, and the workspace was createdƒ
     */
    function isWorkspaceMigrationReqd(startedVersions, installedVersions) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield util_1.promisify(fs.access)(MCUtil_1.default.getCWWorkspacePath());
            }
            catch (err) {
                // no workspace -> no upgrade required
                return false;
            }
            const versions = startedVersions.length > 0 ? startedVersions : installedVersions;
            if (versions.length === 0) {
                // if they have a workspace but no installed verisons, we have to assume it was created by an old version
                return true;
            }
            const newestInstalledVersion = versions.sort()[versions.length - 1];
            const upgradeReqd = [
                "0.1", "0.2", "0.3", "0.4", "0.5"
            ].some((oldVersion) => newestInstalledVersion.startsWith(oldVersion));
            Logger_1.default.i(`The newest installed version ${newestInstalledVersion} requires workspace migration ? ${upgradeReqd}`);
            return upgradeReqd;
        });
    }
    function doWorkspaceUpgrade() {
        return __awaiter(this, void 0, void 0, function* () {
            let upgradeResult;
            try {
                const learnMoreBtn = "Learn More";
                vscode.window.showInformationMessage(`Performing codewind-workspace migration...`, learnMoreBtn)
                    .then((res) => {
                    if (res === learnMoreBtn) {
                        vscode.commands.executeCommand(Commands_1.default.VSC_OPEN, CWDocs_1.default.getDocLink(CWDocs_1.default.WORKSPACE_NEWS));
                    }
                });
                upgradeResult = yield CLICommandRunner_1.CLICommandRunner.upgrade();
            }
            catch (err) {
                Logger_1.default.e("Upgrade errored out", err);
                CLIWrapper_1.default.showCLIError(`Failed to migrate projects from the codewind-workspace.
                Use the Add Existing Project command to add your projects back to Codewind.`);
                return;
            }
            if (upgradeResult.migrated.length === 0 && upgradeResult.failed.length === 0) {
                Logger_1.default.i("No projects were available to be migrated");
            }
            else if (upgradeResult.failed.length === 0) {
                Logger_1.default.i(`Upgrade totally succeeded`);
                vscode.window.showInformationMessage(`Successfully migrated ${upgradeResult.migrated.length} projects: ` +
                    `${MCUtil_1.default.joinWithAnd(upgradeResult.migrated)}.`);
            }
            else if (upgradeResult.migrated.length === 0) {
                Logger_1.default.i(`Upgrade totally failed`);
                CLIWrapper_1.default.showCLIError(`Failed to migrate any projects from the codewind-workspace. ` +
                    `Use the Add Existing Project command to add your projects back to Codewind.`);
            }
            else {
                Logger_1.default.i(`Upgrade partially succeeded`);
                const projectsSucceeded = MCUtil_1.default.joinWithAnd(upgradeResult.migrated);
                const projectsFailed = MCUtil_1.default.joinWithAnd(upgradeResult.failed.map((failed) => failed.projectName));
                CLIWrapper_1.default.showCLIError(`Successfully migrated ${projectsSucceeded}, but failed to migrate ${projectsFailed}. ` +
                    `Use the Add Existing project command to add the failed projects back to Codewind.`);
            }
        });
    }
})(CLILifecycleWrapper = exports.CLILifecycleWrapper || (exports.CLILifecycleWrapper = {}));
exports.default = CLILifecycleWrapper;
//# sourceMappingURL=CLILifecycleWrapper.js.map