"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const CodewindStates_1 = require("./CodewindStates");
const CLIWrapper_1 = require("../CLIWrapper");
const MCUtil_1 = require("../../../MCUtil");
const Logger_1 = require("../../../Logger");
const CLILifecycleWrapper_1 = require("./CLILifecycleWrapper");
const CodewindEventListener_1 = require("../CodewindEventListener");
const ConnectionManager_1 = require("../ConnectionManager");
const Resources_1 = require("../../../constants/Resources");
const Commands_1 = require("../../../constants/Commands");
const Requester_1 = require("../../project/Requester");
const CWDocs_1 = require("../../../constants/CWDocs");
const CHE_CW_URL = "https://localhost:9090";
/**
 * This singleton class wraps the local Codewind connection, which exists if CW is running locally,
 * and controls the local Codewind containers' lifecycle.
 */
class LocalCodewindManager {
    constructor() {
        this._state = CodewindStates_1.CodewindStates.STOPPED;
    }
    static get instance() {
        if (this._instance == null) {
            this._instance = new this();
        }
        return this._instance;
    }
    get localConnection() {
        return this._localConnection;
    }
    get isStarted() {
        return this._state === CodewindStates_1.CodewindStates.STARTED || this._state === CodewindStates_1.CodewindStates.STOPPING;
    }
    get state() {
        return this._state;
    }
    ///// Start/Stop Codewind /////
    startCodewind() {
        return __awaiter(this, void 0, void 0, function* () {
            let cwUrl;
            try {
                cwUrl = yield this.startCodewindInner();
            }
            catch (err) {
                if (!CLIWrapper_1.default.isCancellation(err)) {
                    CLIWrapper_1.default.showCLIError(MCUtil_1.default.errToString(err));
                }
                return;
            }
            this.connect(cwUrl);
        });
    }
    /**
     * Create a local connection to the given codewind URL. Sets this manager's local connection, and sets the state to Started.
     */
    connect(cwUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this._localConnection = yield ConnectionManager_1.default.instance.connectLocal(cwUrl);
                this.setState(CodewindStates_1.CodewindStates.STARTED);
            }
            catch (err) {
                Logger_1.default.e("Error connecting to Codewind after it should have started", err);
                this.setState(CodewindStates_1.CodewindStates.ERR_CONNECTING);
                vscode.window.showErrorMessage(MCUtil_1.default.errToString(err));
            }
        });
    }
    /**
     * Installs and starts Codewind, if required. Will exit immediately if already started.
     * @returns The URL to the running Codewind instance, or undefined if it did not appear to start successfully.
     */
    startCodewindInner() {
        return __awaiter(this, void 0, void 0, function* () {
            yield CLILifecycleWrapper_1.CLILifecycleWrapper.installAndStart();
            let cwUrl;
            try {
                cwUrl = yield CLILifecycleWrapper_1.CLILifecycleWrapper.getCodewindUrl();
                if (!cwUrl) {
                    throw new Error("Could not determine Codewind URL");
                }
            }
            catch (err) {
                Logger_1.default.e("Error getting URL after Codewind should have started", err);
                this.setState(CodewindStates_1.CodewindStates.ERR_CONNECTING);
                throw err;
            }
            Logger_1.default.i("Codewind appears to have started at " + cwUrl);
            return cwUrl;
        });
    }
    stopCodewind() {
        return __awaiter(this, void 0, void 0, function* () {
            yield CLILifecycleWrapper_1.CLILifecycleWrapper.stop();
            this._localConnection = undefined;
        });
    }
    /**
     * Check if the local Codewind URL has changed due to a command-line restart, and recreate the local connection if it has changed.
     */
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            if (global.isTheia) {
                return false;
            }
            const cwUrl = yield CLILifecycleWrapper_1.CLILifecycleWrapper.getCodewindUrl();
            if (this.localConnection && cwUrl && cwUrl !== this.localConnection.url) {
                // If the URL has changed, dispose of the old connection, and connect to the new PFE.
                yield ConnectionManager_1.default.instance.removeConnection(this.localConnection);
                yield this.connect(cwUrl);
                return true;
            }
            return false;
        });
    }
    setState(newState) {
        Logger_1.default.d(`Local Codewind state changing from ${this._state} to ${newState}`);
        this._state = newState;
        CodewindEventListener_1.default.onChange(this);
    }
    /**
     * Theia Only -
     * For the theia case where we do not control CW's lifecycle, we simply wait for it to start.
     */
    waitForCodewindToStartTheia() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i(`In theia; waiting for Codewind to come up on ${CHE_CW_URL}`);
            this.setState(CodewindStates_1.CodewindStates.STARTING);
            const cheCwUrl = vscode.Uri.parse(CHE_CW_URL);
            // This looks awfully similar to Requester.waitForReady, but here we're just testing for a listening port.
            const timeoutS = 180;
            const delayS = 2;
            const maxTries = timeoutS / delayS;
            const longerThanUsualTries = Math.round(maxTries / 2);
            let tries = 0;
            const waitingForReadyProm = new Promise((resolve) => {
                const interval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    tries++;
                    const pingResult = yield Requester_1.default.ping(CHE_CW_URL, delayS);
                    if (pingResult) {
                        clearInterval(interval);
                        return resolve(true);
                    }
                    else if (tries >= maxTries) {
                        clearInterval(interval);
                        return resolve(false);
                    }
                    else if (tries === longerThanUsualTries) {
                        this.onTheiaStartTimeout(false, tries * delayS);
                    }
                }), delayS * 1000);
            }).then((result) => {
                if (result) {
                    Logger_1.default.i(`Codewind pod came up after ${tries} tries, ${tries * delayS}s`);
                }
                else {
                    Logger_1.default.e(`Codewind pod did NOT come up after ${tries} tries, ${tries * delayS}s`);
                }
                return result;
            });
            vscode.window.setStatusBarMessage(`${Resources_1.default.getOcticon(Resources_1.default.Octicons.sync, true)}` +
                `Waiting for Codewind to start...`, waitingForReadyProm);
            yield waitingForReadyProm;
            const isCodewindUp = yield waitingForReadyProm;
            if (!isCodewindUp) {
                this.onTheiaStartTimeout(true, timeoutS);
                this.setState(CodewindStates_1.CodewindStates.ERR_CONNECTING);
                return;
            }
            yield this.connect(cheCwUrl);
        });
    }
    onTheiaStartTimeout(failure, secsElapsed) {
        const helpBtn = "Help";
        const msg = failure ?
            `Codewind failed to come up after ${secsElapsed} seconds. ` +
                `Check the status of the Codewind pod, and click ${helpBtn} to open our documentation. Then, restart the Che workspace.` :
            `Codewind is taking longer than usual to start -  Check the status of the Codewind pod, and click ${helpBtn} to open our documentation.`;
        const showMsgFunc = failure ? vscode.window.showErrorMessage : vscode.window.showWarningMessage;
        showMsgFunc(msg, helpBtn)
            .then((res) => {
            if (res === helpBtn) {
                vscode.commands.executeCommand(Commands_1.default.VSC_OPEN, CWDocs_1.default.getDocLink(CWDocs_1.default.INSTALL_ON_CLOUD));
            }
        });
    }
}
exports.default = LocalCodewindManager;
//# sourceMappingURL=LocalCodewindManager.js.map