"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2020 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const path = require("path");
const fs = require("fs");
const request = require("request-promise-native");
const MCUtil_1 = require("../../MCUtil");
const ProjectState_1 = require("./ProjectState");
const Logger_1 = require("../../Logger");
const translator_1 = require("../../constants/strings/translator");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const ProjectCapabilities_1 = require("./ProjectCapabilities");
const MCLogManager_1 = require("./logs/MCLogManager");
const DebugUtils_1 = require("./DebugUtils");
const ProjectType_1 = require("./ProjectType");
const ProjectPendingRestart_1 = require("./ProjectPendingRestart");
const SocketEvents_1 = require("../connection/SocketEvents");
const Validator_1 = require("./Validator");
const Requester_1 = require("./Requester");
const RemoveProjectCmd_1 = require("../../command/project/RemoveProjectCmd");
const Constants_1 = require("../../constants/Constants");
const Commands_1 = require("../../constants/Commands");
const OpenPerfDashboard_1 = require("../../command/project/OpenPerfDashboard");
const Endpoints_1 = require("../../constants/Endpoints");
/**
 * Used to determine App Monitor URL
 */
const langToPathMap = new Map();
langToPathMap.set("java", "javametrics-dash");
langToPathMap.set("nodejs", "appmetrics-dash");
langToPathMap.set("swift", "swiftmetrics-dash");
const STRING_NS = StringNamespaces_1.default.PROJECT;
class Project {
    constructor(projectInfo, connection) {
        this.connection = connection;
        // does this project appear to have a /metrics endpoint available
        this._metricsAvailable = false;
        // in MS
        this.RESTART_TIMEOUT = 180 * 1000;
        this.deleteFilesOnUnbind = false;
        /**
         * Set this project's status based on the project info event payload passed.
         * This includes checking the appStatus, buildStatus, buildStatusDetail, and startMode.
         * Also updates the appPort and debugPort.
         */
        this.update = (projectInfo) => {
            if (projectInfo.projectID !== this.id) {
                // shouldn't happen, but just in case
                Logger_1.default.e(`Project ${this.id} received status update request for wrong project ${projectInfo.projectID}`);
                // return the old state
                return this._state;
            }
            this.setContainerID(projectInfo.containerId);
            // lastbuild is a number while appImageLastBuild is a string
            this.setLastBuild(projectInfo.lastbuild);
            this.setLastImgBuild(Number(projectInfo.appImageLastBuild));
            this.setAutoBuild(projectInfo.autoBuild);
            this.setInjectMetrics(projectInfo.injectMetrics);
            if (projectInfo.isHttps) {
                this._usesHttps = projectInfo.isHttps === true;
            }
            if (projectInfo.contextRoot) {
                this._contextRoot = projectInfo.contextRoot;
            }
            if (projectInfo.appBaseURL) {
                const asUri = vscode.Uri.parse(projectInfo.appBaseURL);
                if (!asUri.scheme || !asUri.authority) {
                    Logger_1.default.e(`Bad appBaseURL "${projectInfo.appBaseURL}" provided; missing scheme or authority`);
                }
                this.appBaseURL = asUri;
            }
            const oldCapabilitiesReady = this._capabilitiesReady;
            this._capabilitiesReady = projectInfo.capabilitiesReady;
            if (oldCapabilitiesReady !== this._capabilitiesReady) {
                // Log.d(`${this.name} capabilities now ready`);
                this.updateCapabilities();
            }
            const wasEnabled = this.state.isEnabled;
            const oldStateStr = this.state.toString();
            const stateChanged = this.state.update(projectInfo);
            if (stateChanged) {
                const startModeMsg = projectInfo.startMode == null ? "" : `, startMode=${projectInfo.startMode}`;
                Logger_1.default.d(`${this.name} went from ${oldStateStr} to ${this._state}${startModeMsg}`);
                // Check if the project was just enabled or disabled
                if (wasEnabled && !this.state.isEnabled) {
                    this.onDisable();
                }
                else if (!wasEnabled && this.state.isEnabled) {
                    this.onEnable();
                }
            }
            const ports = projectInfo.ports;
            if (ports) {
                this.updatePorts(ports);
            }
            else if (this._state.isStarted) {
                Logger_1.default.e("No ports were provided for an app that is supposed to be started");
            }
            if (this.pendingRestart != null) {
                this.pendingRestart.onStateChange(this.state.appState);
            }
            this.onChange();
            return this._state;
        };
        Logger_1.default.d("Creating project from info:", projectInfo);
        this.name = projectInfo.name;
        this.id = projectInfo.projectID;
        const extensionName = (projectInfo.extension) ? projectInfo.extension.name : undefined;
        this.type = new ProjectType_1.default(projectInfo.projectType, projectInfo.language, extensionName);
        this.language = projectInfo.language || "Unknown";
        this.localPath = vscode.Uri.file(projectInfo.locOnDisk);
        this._contextRoot = projectInfo.contextRoot || projectInfo.contextroot || "";
        this._usesHttps = projectInfo.isHttps === true;
        if (projectInfo.extension && projectInfo.extension.config) {
            this.containerAppRoot = projectInfo.extension.config.containerAppRoot;
        }
        if (!this.containerAppRoot) {
            this.containerAppRoot = projectInfo.containerAppRoot;
        }
        // These will be overridden by the call to update(), but we set them here too so the compiler can see they're always set.
        this._autoBuildEnabled = projectInfo.autoBuild;
        // lastbuild is a number
        this._lastBuild = new Date(projectInfo.lastbuild);
        // appImageLastBuild is a string
        this._lastImgBuild = new Date(Number(projectInfo.appImgLastBuild));
        this._injectMetricsEnabled = projectInfo.injectMetrics || false;
        // this field won't be here pre-0.8
        // https://github.com/eclipse/codewind/pull/1774/files
        this._capabilitiesReady = projectInfo.capabilitiesReady || false;
        this._ports = {
            appPort: undefined,
            debugPort: undefined,
            internalPort: undefined,
            internalDebugPort: undefined,
        };
        this._state = new ProjectState_1.default(this.name);
        this._state = this.update(projectInfo);
        // if the inf data has logs and the project is enabled, logs are available now. Else, we have to wait for logsListChanged events.
        const canGetLogs = this._state.isEnabled && projectInfo.logs != null;
        this.logManager = new MCLogManager_1.default(this, canGetLogs);
        // Do any async initialization work that must be done before the project is ready, here.
        // The function calling the constructor must await on this promise before expecting the project to be ready.
        this.initPromise = Promise.all([
            this.updateCapabilities(),
            this.updateMetricsAvailable()
        ])
            .then(() => Promise.resolve());
        Logger_1.default.i(`Created ${this.type.toString()} project ${this.name} with ID ${this.id} at ${this.localPath.fsPath}`);
    }
    toString() {
        return this.name;
    }
    /**
     * Call when this project's mutable fields change
     * to update the tree view and project info pages.
     */
    onChange() {
        var _a;
        this.connection.onChange(this);
        (_a = this._overviewPage) === null || _a === void 0 ? void 0 : _a.refresh();
    }
    /**
     * Update this project's port fields. Does not call onChange().
     * @param ports - Ports object from a socket event or Project info
     * @returns true if at least one port was changed
     */
    updatePorts(ports) {
        let changed = false;
        changed = this.setPort(ports.exposedPort, "appPort");
        changed = this.setPort(ports.exposedDebugPort, "debugPort") || changed;
        changed = this.setPort(ports.internalPort, "internalPort") || changed;
        changed = this.setPort(ports.internalDebugPort, "internalDebugPort") || changed;
        return changed;
    }
    onSettingsChangedEvent(event) {
        Logger_1.default.d("project settings changed " + this.name, event);
        if (event.status !== SocketEvents_1.default.STATUS_SUCCESS) {
            let errMsg = "Project settings update failed: ";
            Logger_1.default.e(errMsg, event.error);
            if (event.error) {
                errMsg += " " + event.error;
            }
            vscode.window.showErrorMessage(errMsg);
            // We still continue with the update even in the case of error
        }
        // Only one of contextroot, app port, or debug port should be set
        // but there's no reason to treat it differently if multiple are set
        let changed = false;
        if (event.contextRoot) {
            let newContextRoot = event.contextRoot;
            // Remove leading / if present
            if (newContextRoot.startsWith("/")) {
                newContextRoot = newContextRoot.substring(1, newContextRoot.length);
            }
            this._contextRoot = event.contextRoot;
            Logger_1.default.i("ContextRoot now " + this._contextRoot);
            changed = true;
        }
        if (event.ports) {
            if (event.ports.internalPort) {
                changed = this.setPort(event.ports.internalPort, "internalPort");
            }
            else if (event.ports.internalDebugPort) {
                changed = this.setPort(event.ports.internalDebugPort, "internalDebugPort");
            }
            else {
                Logger_1.default.e("Received unexpected ports response:", event.ports);
            }
        }
        if (changed) {
            this.onChange();
        }
    }
    doRestart(mode) {
        if (this.pendingRestart != null) {
            // should be prevented by the RestartProjectCommand
            Logger_1.default.e(this.name + ": doRestart called when already restarting");
            return false;
        }
        this.pendingRestart = new ProjectPendingRestart_1.default(this, mode, this.RESTART_TIMEOUT);
        return true;
    }
    onRestartFinish() {
        Logger_1.default.d(this.name + ": onRestartFinish");
        this.pendingRestart = undefined;
    }
    /**
     * Validate the restart event. If it succeeded, update ports.
     * Notifies the pendingRestart.
     */
    onRestartEvent(event) {
        let success;
        let errMsg;
        if (this.pendingRestart == null) {
            Logger_1.default.e(this.name + ": received restart event without a pending restart", event);
            return;
        }
        if (SocketEvents_1.default.STATUS_SUCCESS !== event.status) {
            Logger_1.default.e(`${this.name}: Restart failed, response is`, event);
            errMsg = translator_1.default.t(STRING_NS, "genericErrorProjectRestart", { thisName: this.name });
            if (event.errorMsg != null) {
                errMsg = event.errorMsg;
            }
            success = false;
        }
        else if (event.ports == null || event.startMode == null ||
            !ProjectCapabilities_1.default.allStartModes.map((mode) => mode.toString()).includes(event.startMode)) {
            // If the status is "success" (as we just checked), these must all be set and valid
            errMsg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "genericErrorProjectRestart", { thisName: this.name });
            Logger_1.default.e(errMsg + ", payload:", event);
            success = false;
        }
        else {
            Logger_1.default.d("Restart event is valid");
            this.updatePorts(event.ports);
            // https://github.com/eclipse/codewind/issues/311
            if (event.containerId) {
                this.setContainerID(event.containerId);
            }
            this.onChange();
            success = true;
        }
        this.pendingRestart.onReceiveRestartEvent(success, errMsg);
    }
    updateCapabilities() {
        return __awaiter(this, void 0, void 0, function* () {
            let capabilities;
            if (!this.state.isEnabled || !this._capabilitiesReady) {
                // The project must refresh the capabilities on re-enable, or when capabilitiesReady becomes true.
                // server will return a 404 in this case
                capabilities = ProjectCapabilities_1.default.NO_CAPABILITIES;
            }
            else {
                try {
                    capabilities = yield Requester_1.default.getCapabilities(this);
                }
                catch (err) {
                    // If the project is enabled and there is an error, we fall back to all capabilities so as not to block any UI actions.
                    // But this should never happen
                    Logger_1.default.e("Error retrieving capabilities for " + this.name, err);
                    capabilities = ProjectCapabilities_1.default.NO_CAPABILITIES;
                }
            }
            this._capabilities = capabilities;
            this.onChange();
        });
    }
    updateMetricsAvailable() {
        return __awaiter(this, void 0, void 0, function* () {
            const oldMetricsAvailable = this._metricsAvailable;
            try {
                this._metricsAvailable = yield Requester_1.default.areMetricsAvailable(this);
                if (oldMetricsAvailable !== this._metricsAvailable) {
                    this.onChange();
                }
            }
            catch (err) {
                Logger_1.default.e(`Error checking metrics status for ${this.name}`, err);
            }
        });
    }
    onConnectionReconnect() {
        this.logManager.onReconnectOrEnable();
    }
    onConnectionDisconnect() {
        if (this.pendingRestart != null) {
            this.pendingRestart.onDisconnectOrDisable(true);
        }
        this.logManager.onDisconnect();
    }
    onEnable() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i(`${this.name} has been enabled`);
            this.logManager.onReconnectOrEnable();
            yield this.updateCapabilities();
        });
    }
    onDisable() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i(`${this.name} has been disabled`);
            if (this.pendingRestart != null) {
                this.pendingRestart.onDisconnectOrDisable(false);
            }
            // this.logManager.destroyAllLogs();
            (_a = this.logManager) === null || _a === void 0 ? void 0 : _a.destroyAllLogs();
        });
    }
    dispose() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all([
                this.clearValidationErrors(),
                (_a = this.logManager) === null || _a === void 0 ? void 0 : _a.destroyAllLogs(),
                this._overviewPage != null ? this._overviewPage.dispose() : Promise.resolve(),
            ]);
            this.connection.onChange(this);
        });
    }
    deleteFromCodewind(deleteFiles) {
        Logger_1.default.d(`Deleting ${this}`);
        this.deleteFilesOnUnbind = deleteFiles;
        const pendingDeletionProm = new Promise((resolve) => {
            this.resolvePendingDeletion = resolve;
            Requester_1.default.requestUnbind(this);
        });
        return pendingDeletionProm;
    }
    onDeletionEvent(event) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.resolvePendingDeletion) {
                Logger_1.default.e(`Received deletion event for ${this} that was not pending deletion`);
                return;
            }
            if (event.status !== SocketEvents_1.default.STATUS_SUCCESS) {
                Logger_1.default.e(`Received bad deletion event for ${this}`, event);
                vscode.window.showErrorMessage(`Error deleting ${this.name}`);
                // resolve the pending deletion because they will have to try again
                this.resolvePendingDeletion();
                return;
            }
            Logger_1.default.i(`${this} was deleted from ${this.connection.label}`);
            DebugUtils_1.default.removeDebugLaunchConfigFor(this);
            const deleteFilesProm = this.deleteFilesOnUnbind ? RemoveProjectCmd_1.deleteProjectDir(this) : Promise.resolve();
            yield Promise.all([
                deleteFilesProm,
                this.dispose(),
            ]);
            this.resolvePendingDeletion();
            Logger_1.default.d(`Finished deleting ${this}`);
        });
    }
    /**
     * Clear all diagnostics for this project's path
     */
    clearValidationErrors() {
        return __awaiter(this, void 0, void 0, function* () {
            Project.diagnostics.delete(this.localPath);
        });
    }
    ///// ProjectOverview
    onDidOpenOverviewPage(overviewPage) {
        this._overviewPage = overviewPage;
    }
    get overviewPage() {
        return this._overviewPage;
    }
    onDidCloseOverviewPage() {
        this._overviewPage = undefined;
    }
    ///// Getters
    // QuickPickItem
    get label() {
        return translator_1.default.t(STRING_NS, "quickPickLabel", { projectName: this.name, projectType: this.type.toString() });
    }
    // QuickPickItem
    get description() {
        const appUrl = this.appUrl;
        if (appUrl != null) {
            return appUrl.toString();
        }
        else {
            return translator_1.default.t(STRING_NS, "quickPickNotRunning");
        }
    }
    // QuickPickItem
    get detail() {
        return this.connection.label;
    }
    get isRestarting() {
        return this.pendingRestart != null;
    }
    get containerID() {
        return this._containerID;
    }
    get contextRoot() {
        return this._contextRoot;
    }
    get ports() {
        return this._ports;
    }
    get autoBuildEnabled() {
        return this._autoBuildEnabled;
    }
    get state() {
        return this._state;
    }
    get capabilities() {
        // This will only happen if this funciton is called before the initPromise resolves, which should never happen
        if (!this._capabilities) {
            this._capabilities = ProjectCapabilities_1.default.ALL_CAPABILITIES;
        }
        return this._capabilities;
    }
    get metricsAvailable() {
        return this._metricsAvailable;
    }
    get hasAppMonitor() {
        return this.type.alwaysHasAppMonitor || this.metricsAvailable;
    }
    get hasPerfDashboard() {
        return this.metricsAvailable || this.injectMetricsEnabled;
    }
    get appUrl() {
        // If the backend has provided us with a baseUrl already, use that
        if (this.appBaseURL) {
            return this.appBaseURL.with({
                path: this._contextRoot,
            });
        }
        if (this._ports.appPort == null || isNaN(this._ports.appPort)) {
            // app is stopped, disabled, etc.
            return undefined;
        }
        const scheme = this._usesHttps ? "https" : "http"; // non-nls
        return this.connection.url.with({
            scheme,
            authority: `${this.connection.host}:${this._ports.appPort}`,
            path: this._contextRoot
        });
    }
    get debugUrl() {
        if (this._ports.debugPort == null || isNaN(this._ports.debugPort)) {
            return undefined;
        }
        return this.connection.host + ":" + this._ports.debugPort; // non-nls
    }
    get lastBuild() {
        return this._lastBuild;
    }
    get lastImgBuild() {
        return this._lastImgBuild;
    }
    get hasContextRoot() {
        return this._contextRoot != null && this._contextRoot.length > 0 && this._contextRoot !== "/";
    }
    get appMonitorUrl() {
        const appMetricsPath = langToPathMap.get(this.type.language);
        const supported = appMetricsPath != null && this.metricsAvailable;
        if ((!this._injectMetricsEnabled) && supported) {
            // open app monitor in Application container
            Logger_1.default.d(`${this.name} supports metrics ? ${supported}`);
            if (this.appUrl === undefined) {
                return undefined;
            }
            let monitorPageUrlStr = this.appUrl.toString();
            if (!monitorPageUrlStr.endsWith("/")) {
                monitorPageUrlStr += "/";
            }
            return monitorPageUrlStr + appMetricsPath + "/?theme=dark";
        }
        try {
            // open app monitor in Performance container
            const cwBaseUrl = global.isTheia ? OpenPerfDashboard_1.getCodewindIngress() : this.connection.url;
            const dashboardUrl = Endpoints_1.default.getPerformanceMonitor(cwBaseUrl, this.language, this.id);
            Logger_1.default.d(`Perf container Monitor Dashboard url for ${this.name} is ${dashboardUrl}`);
            return dashboardUrl.toString();
        }
        catch (err) {
            Logger_1.default.e(`${this} error determining app monitor URL`, err);
            vscode.window.showErrorMessage(MCUtil_1.default.errToString(err));
            return undefined;
        }
    }
    get canContainerShell() {
        return !this.connection.isRemote; // && !!this.containerID;
    }
    get isInVSCodeWorkspace() {
        return !!vscode.workspace.workspaceFolders &&
            vscode.workspace.workspaceFolders.some((folder) => this.localPath.fsPath.startsWith(folder.uri.fsPath));
    }
    get injectMetricsEnabled() {
        return this._injectMetricsEnabled;
    }
    ///// Setters
    /**
     * Set one of this project's Port fields.
     * @param newPort Can be undefined if the caller wishes to "unset" the port (ie, because the app is stopping)
     * @returns true if at least one port was changed.
     */
    setPort(newPort, portType) {
        if (newPort === "") {
            newPort = undefined;
        }
        const newPortNumber = Number(newPort);
        const currentPort = this._ports[portType];
        if (newPort && !MCUtil_1.default.isGoodPort(newPortNumber)) {
            Logger_1.default.w(`Invalid ${portType} port ${newPort} given to project ${this.name}, ignoring it`);
            return false;
        }
        else if (currentPort !== newPortNumber) {
            if (isNaN(newPortNumber)) {
                if (this._ports[portType]) {
                    Logger_1.default.d(`Unset ${portType} for ${this.name}`);
                }
                this._ports[portType] = undefined;
            }
            else if (newPortNumber !== currentPort) {
                Logger_1.default.d(`New ${portType} for ${this.name} is ${newPortNumber}`);
                this._ports[portType] = newPortNumber;
            }
            // the third case is that (the new port === the old port) and neither are null - we don't log anything in this case.
            return true;
        }
        // Log.d(`${portType} port is already ${currentPort}`);
        return false;
    }
    setContainerID(newContainerID) {
        const oldContainerID = this._containerID;
        this._containerID = newContainerID;
        const changed = this._containerID !== oldContainerID;
        if (changed) {
            const asStr = this._containerID == null ? "undefined" : this._containerID.substring(0, 8);
            if (asStr.length === 0) {
                Logger_1.default.w(`Empty containerID for ${this.name}`);
            }
            Logger_1.default.d(`New containerID for ${this.name} is ${asStr}`);
        }
        return changed;
    }
    setLastBuild(newLastBuild) {
        if (newLastBuild == null) {
            return false;
        }
        const oldlastBuild = this._lastBuild;
        this._lastBuild = new Date(newLastBuild);
        const changed = this._lastBuild !== oldlastBuild;
        if (changed) {
            // Log.d(`New lastBuild for ${this.name} is ${this._lastBuild}`);
        }
        return changed;
    }
    setLastImgBuild(newLastImgBuild) {
        if (newLastImgBuild == null) {
            return false;
        }
        const oldlastImgBuild = this._lastImgBuild;
        this._lastImgBuild = new Date(newLastImgBuild);
        const changed = this._lastImgBuild !== oldlastImgBuild;
        if (changed) {
            // Log.d(`New lastImgBuild for ${this.name} is ${this._lastImgBuild}`);
        }
        return changed;
    }
    setAutoBuild(newAutoBuild) {
        if (newAutoBuild == null) {
            return false;
        }
        const oldAutoBuild = this._autoBuildEnabled;
        this._autoBuildEnabled = newAutoBuild;
        const changed = this._autoBuildEnabled !== oldAutoBuild;
        if (changed) {
            // onChange has to be invoked explicitly because this function can be called outside of update()
            Logger_1.default.d(`New autoBuild for ${this.name} is ${this._autoBuildEnabled}`);
            this.onChange();
        }
        return changed;
    }
    setInjectMetrics(newInjectMetrics) {
        return __awaiter(this, void 0, void 0, function* () {
            if (newInjectMetrics == null) {
                return false;
            }
            const oldInjectMetrics = this._injectMetricsEnabled;
            this._injectMetricsEnabled = newInjectMetrics;
            const changed = this._injectMetricsEnabled !== oldInjectMetrics;
            if (changed) {
                // onChange has to be invoked explicitly because this function can be called outside of update()
                Logger_1.default.d(`New autoInjectMetricsEnabled for ${this.name} is ${this._injectMetricsEnabled}`);
                this.updateMetricsAvailable();
            }
            return changed;
        });
    }
    tryOpenSettingsFile() {
        return __awaiter(this, void 0, void 0, function* () {
            const settingsFilePath = path.join(this.localPath.fsPath, Constants_1.default.PROJ_SETTINGS_FILE_NAME);
            let settingsFileExists;
            try {
                yield fs.promises.access(settingsFilePath);
                settingsFileExists = true;
            }
            catch (err) {
                settingsFileExists = false;
            }
            if (settingsFileExists) {
                vscode.commands.executeCommand(Commands_1.default.VSC_OPEN, vscode.Uri.file(settingsFilePath));
            }
            else if (this.type.isExtensionType) {
                // this is expected; https://github.com/eclipse/codewind/issues/649
                vscode.window.showWarningMessage(`Application settings cannot be configured for ${this.type.toString()} projects.`);
            }
            else {
                // fall-back in case the user deleted the file, or something.
                vscode.window.showWarningMessage(`${settingsFilePath} does not exist or was not readable.`);
            }
        });
    }
    /**
     * Extra test for extension projects app monitor - workaround for https://github.com/eclipse/codewind/issues/258
     */
    testPingAppMonitor() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.type.type !== ProjectType_1.default.Types.EXTENSION_APPSODY) {
                // this test is not necessary for non-appsody projects
                return true;
            }
            if (this.appMonitorUrl == null) {
                return false;
            }
            Logger_1.default.i(`Testing extension project's app monitor before opening`);
            try {
                yield request.get(this.appMonitorUrl, { rejectUnauthorized: false });
                return true;
            }
            catch (err) {
                Logger_1.default.w(`Failed to access app monitor for project ${this.name} at ${this.appMonitorUrl}`, err);
                // cache this so we don't have to do this test every time.
                this._metricsAvailable = false;
                // Notify the treeview that this project has changed so it can hide these context actions
                this.onChange();
                return false;
            }
        });
    }
}
exports.default = Project;
Project.diagnostics = vscode.languages.createDiagnosticCollection(Validator_1.default.DIAGNOSTIC_COLLECTION_NAME);
//# sourceMappingURL=Project.js.map