"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2020 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Logger_1 = require("../../Logger");
const ProjectCapabilities_1 = require("./ProjectCapabilities");
/**
 * Represents the project's state. This means app state, build state, and any status details.
 */
class ProjectState {
    constructor(projectName) {
        this.projectName = projectName;
        this.appState = ProjectState.AppStates.UNKNOWN;
        // call update() right away
    }
    update(projectStateInfo) {
        const oldState = Object.assign({}, this);
        this.appState = ProjectState.getAppState(projectStateInfo);
        if (this.appState !== ProjectState.AppStates.DISABLED) {
            this.appDetail = projectStateInfo.detailedAppStatus;
            this.buildState = ProjectState.getBuildState(projectStateInfo.buildStatus);
            this.buildDetail = projectStateInfo.detailedBuildStatus;
        }
        else {
            // the disabled app state has no other states.
            this.appDetail = undefined;
            this.buildState = undefined;
            this.buildDetail = undefined;
        }
        if (this.appDetail && this.appDetail.notify) {
            this.notify();
        }
        return !(this.appState === oldState.appState &&
            this.appDetail === oldState.appDetail &&
            this.buildState === oldState.buildState &&
            this.buildDetail === oldState.buildDetail);
    }
    get isEnabled() {
        return ProjectState.getEnabledStates().includes(this.appState);
    }
    get isStarted() {
        return ProjectState.getStartedStates().includes(this.appState);
    }
    get isStarting() {
        return ProjectState.getStartingStates().includes(this.appState);
    }
    get isDebuggable() {
        return ProjectState.getDebuggableStates().includes(this.appState);
    }
    get isBuilding() {
        return this.buildState === ProjectState.BuildStates.BUILDING;
    }
    toString() {
        if (!this.isEnabled) {
            // Just show Disabled for disabled projects
            return `[${this.appState}]`;
        }
        const buildStatusStr = this.getBuildString();
        if (this.appState === ProjectState.AppStates.UNKNOWN) {
            if (!buildStatusStr) {
                Logger_1.default.w(`${this.projectName} has unknown app and build statuses`);
                return "";
            }
            // Return only the build status if app status is unknown
            return `[${buildStatusStr}]`;
        }
        else {
            // Return both statuses
            return `[${this.appState}] [${buildStatusStr}]`;
        }
    }
    getBuildString() {
        if (!this.isEnabled) {
            return undefined;
        }
        let buildStateStr = "";
        if (this.buildDetail != null && this.buildDetail.trim() !== "") {
            // a detailed status is available
            buildStateStr = `${this.buildState} - ${this.buildDetail}`;
        }
        // Don't display the build state if it's unknown
        else if (this.buildState !== ProjectState.BuildStates.UNKNOWN) {
            buildStateStr = `${this.buildState}`; // non-nls
        }
        return buildStateStr;
    }
    getAppStatusWithDetail() {
        if (!this.appState || this.appState === ProjectState.AppStates.UNKNOWN) {
            return "";
        }
        let status = this.appState.toString();
        if (this.appDetail && this.appDetail.message) {
            status += ` - ${this.appDetail.message}`;
        }
        return status;
    }
    notify() {
        // https://github.com/eclipse/codewind/issues/1297
        if (!this.appDetail || !this.appDetail.notify) {
            return;
        }
        Logger_1.default.i(`Showing user detailed app status ${this.appDetail.message} for project ${this.projectName}`);
        const notificationMsg = `${this.projectName} - ${this.appDetail.message}`;
        if (this.appDetail.severity === "ERROR") {
            vscode.window.showErrorMessage(notificationMsg);
        }
        else if (this.appDetail.severity === "WARN") {
            vscode.window.showWarningMessage(notificationMsg);
        }
        else {
            vscode.window.showInformationMessage(notificationMsg);
        }
    }
}
exports.ProjectState = ProjectState;
(function (ProjectState) {
    // The AppStates and BuildStates string values are all exposed to the user.
    let AppStates;
    (function (AppStates) {
        AppStates["STARTED"] = "Running";
        AppStates["STARTING"] = "Starting";
        AppStates["STOPPING"] = "Stopping";
        AppStates["STOPPED"] = "Stopped";
        AppStates["DEBUGGING"] = "Debugging";
        AppStates["DEBUG_STARTING"] = "Starting - Debug";
        AppStates["DISABLED"] = "Disabled";
        AppStates["UNKNOWN"] = "N/A";
    })(AppStates = ProjectState.AppStates || (ProjectState.AppStates = {}));
    let BuildStates;
    (function (BuildStates) {
        BuildStates["BUILD_SUCCESS"] = "Build Succeeded";
        BuildStates["BUILDING"] = "Building";
        BuildStates["BUILD_FAILED"] = "Build Failed";
        BuildStates["BUILD_QUEUED"] = "Build Queued";
        BuildStates["UNKNOWN"] = "N/A";
    })(BuildStates = ProjectState.BuildStates || (ProjectState.BuildStates = {}));
    function getAllAppStates() {
        return Object.values(AppStates);
    }
    ProjectState.getAllAppStates = getAllAppStates;
    function getEnabledStates() {
        return Object.values(AppStates).filter((state) => state !== ProjectState.AppStates.DISABLED);
    }
    ProjectState.getEnabledStates = getEnabledStates;
    function getStartedStates() {
        return [
            ProjectState.AppStates.STARTED,
            ProjectState.AppStates.DEBUGGING
        ];
    }
    ProjectState.getStartedStates = getStartedStates;
    function getStartingStates() {
        return [
            ProjectState.AppStates.STARTING,
            ProjectState.AppStates.DEBUG_STARTING
        ];
    }
    ProjectState.getStartingStates = getStartingStates;
    function getStartedOrStartingStates() {
        return getStartedStates().concat(getStartingStates());
    }
    ProjectState.getStartedOrStartingStates = getStartedOrStartingStates;
    function getDebuggableStates() {
        return [
            ProjectState.AppStates.DEBUGGING,
            ProjectState.AppStates.DEBUG_STARTING
        ];
    }
    ProjectState.getDebuggableStates = getDebuggableStates;
    /**
     * Convert a project info object into a ProjectState.
     */
    function getAppState(projectStateInfo) {
        // Logger.log("PIP", projectInfoPayload);
        const appStatus = projectStateInfo.appStatus;
        const openClosedState = projectStateInfo.state;
        const startMode = projectStateInfo.startMode;
        // Logger.log(`Convert - appStatus=${appStatus}, closedState=${closedState}, startMode=${startMode}`);
        // First, check if the project is closed (aka Disabled)
        if (openClosedState === "closed") { // non-nls
            return ProjectState.AppStates.DISABLED;
        }
        // Now, check the app states. Compare against both the value we expect from MC,
        // as well as our own possible values, in case we used the fallbackState in the constructor.
        else if (appStatus === "started" || appStatus === AppStates.DEBUGGING || appStatus === AppStates.STARTED) { // non-nls
            if (startMode != null && ProjectCapabilities_1.default.isDebugMode(startMode)) {
                return ProjectState.AppStates.DEBUGGING;
            }
            return ProjectState.AppStates.STARTED;
        }
        else if (appStatus === "starting" || appStatus === AppStates.STARTING || appStatus === AppStates.DEBUG_STARTING) { // non-nls
            if (startMode != null && ProjectCapabilities_1.default.isDebugMode(startMode)) {
                return ProjectState.AppStates.DEBUG_STARTING;
            }
            return ProjectState.AppStates.STARTING;
        }
        else if (appStatus === "stopping" || appStatus === AppStates.STOPPING) { // non-nls
            return ProjectState.AppStates.STOPPING;
        }
        else if (appStatus === "stopped" || appStatus === AppStates.STOPPED) { // non-nls
            return ProjectState.AppStates.STOPPED;
        }
        else if (!appStatus || appStatus === "unknown" || appStatus === AppStates.UNKNOWN) { // non-nls
            return ProjectState.AppStates.UNKNOWN;
        }
        else {
            Logger_1.default.e("Unknown app state:", appStatus);
            return ProjectState.AppStates.UNKNOWN;
        }
    }
    ProjectState.getAppState = getAppState;
    function getBuildState(buildStatus) {
        if (buildStatus === "success" || buildStatus === BuildStates.BUILD_SUCCESS) { // non-nls
            return BuildStates.BUILD_SUCCESS;
        }
        else if (buildStatus === "inProgress" || buildStatus === BuildStates.BUILDING) { // non-nls
            return BuildStates.BUILDING;
        }
        else if (buildStatus === "queued" || buildStatus === BuildStates.BUILD_QUEUED) { // non-nls
            return BuildStates.BUILD_QUEUED;
        }
        else if (buildStatus === "failed" || buildStatus === BuildStates.BUILD_FAILED) { // non-nls
            return BuildStates.BUILD_FAILED;
        }
        else if (buildStatus == null || buildStatus.toLowerCase() === "unknown") { // non-nls
            return BuildStates.UNKNOWN;
        }
        else {
            Logger_1.default.e("Unknown build state:", buildStatus);
            return BuildStates.UNKNOWN;
        }
    }
    ProjectState.getBuildState = getBuildState;
})(ProjectState = exports.ProjectState || (exports.ProjectState = {}));
exports.default = ProjectState;
//# sourceMappingURL=ProjectState.js.map