"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2020 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const request = require("request-promise-native");
const errors_1 = require("request-promise-native/errors");
const ProjectCapabilities_1 = require("./ProjectCapabilities");
const Logger_1 = require("../../Logger");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const translator_1 = require("../../constants/strings/translator");
const MCUtil_1 = require("../../MCUtil");
const Endpoints_1 = require("../../constants/Endpoints");
const RemoteConnection_1 = require("../connection/RemoteConnection");
const RegistryUtils_1 = require("../connection/RegistryUtils");
// tslint:disable-next-line: variable-name
const HttpVerbs = {
    GET: request.get,
    POST: request.post,
    PUT: request.put,
    PATCH: request.patch,
    DELETE: request.delete,
};
const STRING_NS = StringNamespaces_1.default.REQUESTS;
/**
 * These functions wrap all our API calls to the Codewind backend.
 *
 * Each request is perform for either a Connection or a Project - see doConnectionRequest and doProjectRequest.
 *
 * API doc - https://eclipse.github.io/codewind/
 */
var Requester;
(function (Requester) {
    ///// Connection-specific requests
    function getProjects(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            return doConnectionRequest(connection, Endpoints_1.MCEndpoints.PROJECTS, "GET");
        });
    }
    Requester.getProjects = getProjects;
    function getRawEnvironment(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            return doConnectionRequest(connection, Endpoints_1.MCEndpoints.ENVIRONMENT, "GET");
        });
    }
    Requester.getRawEnvironment = getRawEnvironment;
    function getTemplates(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield doConnectionRequest(connection, Endpoints_1.MCEndpoints.TEMPLATES, "GET", { qs: { showEnabledOnly: true } });
            if (result == null) {
                return [];
            }
            return result;
        });
    }
    Requester.getTemplates = getTemplates;
    /**
     * Change the 'enabled' state of the given set of template sources.
     * Should only be called by TemplateSourceList to ensure it is refreshed appropriately.
     */
    function toggleSourceEnablement(connection, enablements) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = enablements.repos.map((repo) => {
                return {
                    op: "enable",
                    url: repo.repoID,
                    value: repo.enable ? "true" : "false",
                };
            });
            // status is always 207, we have to check the individual results for success status
            const result = yield doConnectionRequest(connection, Endpoints_1.MCEndpoints.BATCH_TEMPLATE_REPOS, "PATCH", { body });
            const failures = result.filter((opResult) => opResult.status !== 200);
            if (failures.length > 0) {
                Logger_1.default.e("Repo enablement failure", result);
                failures.forEach((failure) => {
                    const failedOp = failure.requestedOperation;
                    Logger_1.default.e(`Failed to set ${failedOp.op}=${failedOp.value} for ${failedOp.url}: ${failure.status}`);
                });
                const errMsg = `Failed to enable/disable repositories: ${failures.map((failure) => failure.requestedOperation.url).join(", ")}`;
                vscode.window.showErrorMessage(errMsg);
            }
            // Log.d("Repo enablement result", result);
        });
    }
    Requester.toggleSourceEnablement = toggleSourceEnablement;
    function getProjectTypes(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield doConnectionRequest(connection, Endpoints_1.MCEndpoints.PROJECT_TYPES, "GET");
            if (result == null) {
                return [];
            }
            return result;
        });
    }
    Requester.getProjectTypes = getProjectTypes;
    function asContainerRegistry(response) {
        if (!response.address || !response.username) {
            Logger_1.default.e(`Received unexpected container registry response:`, response);
        }
        return new RegistryUtils_1.ContainerRegistry(response.address, response.username);
    }
    function getImageRegistries(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield doConnectionRequest(connection, Endpoints_1.MCEndpoints.REGISTRY_SECRETS, "GET");
            // Log.d(`Container registry response:`, response);
            const registries = response.map((reg) => asContainerRegistry(reg));
            const pushRegistryRes = yield getPushRegistry(connection);
            // Log.d(`Image push registry response`, pushRegistryRes);
            // tslint:disable-next-line: no-boolean-literal-compare
            if (pushRegistryRes.imagePushRegistry === true) {
                const pushRegistry = registries.find((reg) => reg.address === pushRegistryRes.address);
                if (!pushRegistry) {
                    Logger_1.default.e(`Push registry response was ${JSON.stringify(pushRegistryRes)} but no registry with a matching address was found`);
                }
                else {
                    pushRegistry.isPushRegistry = true;
                    pushRegistry.namespace = pushRegistryRes.namespace || "";
                    Logger_1.default.i(`Push registry is ${pushRegistry.address}`);
                }
            }
            else {
                Logger_1.default.d(`No image push registry is set`);
            }
            return registries;
        });
    }
    Requester.getImageRegistries = getImageRegistries;
    function addRegistrySecret(connection, address, username, password) {
        return __awaiter(this, void 0, void 0, function* () {
            const credentialsPlain = { username, password };
            const credentialsEncoded = Buffer.from(JSON.stringify(credentialsPlain)).toString("base64");
            const body = {
                address,
                credentials: credentialsEncoded,
            };
            const response = yield doConnectionRequest(connection, Endpoints_1.MCEndpoints.REGISTRY_SECRETS, "POST", { body });
            const match = response.find((reg) => reg.address === address);
            if (match == null) {
                Logger_1.default.e("Got success response when adding new registry secret, but was not found in api response");
                throw new Error(`Error adding new registry secret`);
            }
            return asContainerRegistry(match);
        });
    }
    Requester.addRegistrySecret = addRegistrySecret;
    function removeRegistrySecret(connection, toRemove) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = {
                address: toRemove.address,
            };
            const response = yield doConnectionRequest(connection, Endpoints_1.MCEndpoints.REGISTRY_SECRETS, "DELETE", { body });
            return response.map(asContainerRegistry);
        });
    }
    Requester.removeRegistrySecret = removeRegistrySecret;
    function getPushRegistry(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            return doConnectionRequest(connection, Endpoints_1.MCEndpoints.PUSH_REGISTRY, "GET");
        });
    }
    Requester.getPushRegistry = getPushRegistry;
    function setPushRegistry(connection, registry) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = {
                operation: "set",
                address: registry.address,
                namespace: registry.namespace,
            };
            yield doConnectionRequest(connection, Endpoints_1.MCEndpoints.PUSH_REGISTRY, "POST", { body });
        });
    }
    Requester.setPushRegistry = setPushRegistry;
    function testPushRegistry(connection, registry) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = {
                operation: "test",
                address: registry.address,
                namespace: registry.namespace,
            };
            return doConnectionRequest(connection, Endpoints_1.MCEndpoints.PUSH_REGISTRY, "POST", { body });
        });
    }
    Requester.testPushRegistry = testPushRegistry;
    // Project-specific requests
    function requestProjectRestart(project, startMode) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = {
                startMode: startMode.toString()
            };
            const restartMsg = translator_1.default.t(STRING_NS, "restartIntoMode", { startMode: ProjectCapabilities_1.default.getUserFriendlyStartMode(startMode) });
            return doProjectRequest(project, Endpoints_1.ProjectEndpoints.RESTART_ACTION, body, "POST", restartMsg, false, true);
        });
    }
    Requester.requestProjectRestart = requestProjectRestart;
    function requestBuild(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const body = {
                action: "build" // non-nls
            };
            // return doProjectRequest(project, url, body, post, "Build");
            const buildMsg = translator_1.default.t(STRING_NS, "build");
            yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.BUILD_ACTION, body, "POST", buildMsg);
        });
    }
    Requester.requestBuild = requestBuild;
    function requestToggleAutoBuild(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const newAutoBuild = !project.autoBuildEnabled;
            // user-friendly action
            const autoBuildMsgKey = newAutoBuild ? "autoBuildEnable" : "autoBuildDisable"; // non-nls
            const newAutoBuildUserStr = translator_1.default.t(STRING_NS, autoBuildMsgKey);
            const newAutoBuildAction = newAutoBuild ? "enableautobuild" : "disableautobuild"; // non-nls
            const body = {
                action: newAutoBuildAction
            };
            // const response = await doProjectRequest(project, ProjectEndpoints.BUILD_ACTION, body, post, newAutoBuildUserStr);
            yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.BUILD_ACTION, body, "POST", newAutoBuildUserStr);
            project.setAutoBuild(newAutoBuild);
        });
    }
    Requester.requestToggleAutoBuild = requestToggleAutoBuild;
    function requestToggleEnablement(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const newEnablement = !project.state.isEnabled;
            const newEnablementMsgKey = newEnablement ? "projectEnable" : "projectDisable"; // non-nls
            const newEnablementStr = translator_1.default.t(STRING_NS, newEnablementMsgKey);
            const endpoint = Endpoints_1.default.getEnablementAction(newEnablement);
            yield doProjectRequest(project, endpoint, {}, "PUT", newEnablementStr);
        });
    }
    Requester.requestToggleEnablement = requestToggleEnablement;
    // Validation appears to have been removed
    // export async function requestValidate(project: Project, silent: boolean): Promise<void> {
    //     const [endpoint, body]: [Endpoint, IValidateRequestBody] = assembleValidateRequest(project, false);
    //     const userOperation = Translator.t(StringNamespaces.CMD_MISC, "validate");
    //     await doProjectRequest(project, endpoint, body, post, userOperation, silent);
    // }
    // export async function requestGenerate(project: Project): Promise<void> {
    //     const [endpoint, body]: [Endpoint, IValidateRequestBody] = assembleValidateRequest(project, true);
    //     const generateMsg = Translator.t(STRING_NS, "generateMissingFiles");
    //     await doProjectRequest(project, endpoint, body, post, generateMsg);
    //     // request a validate after the generate so that the validation errors go away faster
    //     await requestValidate(project, true);
    // }
    // interface IValidateRequestBody {
    //     projectType: string;
    //     projectID?: string;
    //     autoGenerate?: boolean;
    // }
    // /**
    //  * Get the URL and request body for either a Validate or Generate request, they are very similar.
    //  */
    // function assembleValidateRequest(project: Project, generate: boolean): [ProjectEndpoints, IValidateRequestBody] {
    //     const body: IValidateRequestBody = {
    //         projectType: project.type.internalType,
    //     };
    //     if (generate) {
    //         body.autoGenerate = true;
    //     }
    //     const endpoint = generate ? ProjectEndpoints.GENERATE : ProjectEndpoints.VALIDATE;
    //     return [endpoint, body];
    // }
    function requestUnbind(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const msg = translator_1.default.t(STRING_NS, "unbind");
            yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.UNBIND, {}, "POST", msg, true);
        });
    }
    Requester.requestUnbind = requestUnbind;
    function requestAvailableLogs(project) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!project.state.isEnabled) {
                // there are no logs available for disabled projects
                return {
                    build: [], app: []
                };
            }
            const msg = translator_1.default.t(STRING_NS, "checkingAvailableLogs");
            return doProjectRequest(project, Endpoints_1.ProjectEndpoints.LOGS, {}, "GET", msg, true);
        });
    }
    Requester.requestAvailableLogs = requestAvailableLogs;
    function requestToggleLogs(project, enable) {
        return __awaiter(this, void 0, void 0, function* () {
            const verb = enable ? "POST" : "DELETE";
            const onOrOff = enable ? "on" : "off";
            const msg = translator_1.default.t(STRING_NS, "togglingLogs", { onOrOff });
            yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.LOGS, {}, verb, msg, true);
        });
    }
    Requester.requestToggleLogs = requestToggleLogs;
    function getCapabilities(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.CAPABILITIES, {}, "GET", "Getting capabilities", true);
            return new ProjectCapabilities_1.default(result.startModes, result.controlCommands);
        });
    }
    Requester.getCapabilities = getCapabilities;
    function areMetricsAvailable(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const msg = translator_1.default.t(STRING_NS, "checkingMetrics");
            const res = yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.METRICS_STATUS, {}, "GET", msg, true);
            return res.metricsAvailable;
        });
    }
    Requester.areMetricsAvailable = areMetricsAvailable;
    function requestToggleInjectMetrics(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const newInjectMetrics = !project.injectMetricsEnabled;
            // user-friendly action
            const autoInjectMetricsMsgKey = newInjectMetrics ? "autoInjectMetricsEnable" : "autoInjectMetricsDisable"; // non-nls
            const newAutoInjectMetricsUserStr = translator_1.default.t(STRING_NS, autoInjectMetricsMsgKey);
            const body = {
                enable: newInjectMetrics
            };
            yield doProjectRequest(project, Endpoints_1.ProjectEndpoints.METRICS_INJECTION, body, "POST", newAutoInjectMetricsUserStr);
            yield project.setInjectMetrics(newInjectMetrics);
        });
    }
    Requester.requestToggleInjectMetrics = requestToggleInjectMetrics;
    /**
     * Try to connect to the given URL. Returns true if any response, other than 503 unavailable, is returned.
     */
    function ping(url, timeoutS = 10) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`Ping ${url}`);
            if (url instanceof vscode.Uri) {
                url = url.toString();
            }
            try {
                yield req("GET", url, { timeout: timeoutS * 10000 });
                // It succeeded
                return true;
            }
            catch (err) {
                if (err.message === Requester.ERR_LOGIN_PAGE) {
                    Logger_1.default.d(`Received login page when pinging ${url}`);
                    return true;
                }
                else if (err instanceof errors_1.StatusCodeError) {
                    Logger_1.default.d(`Received status ${err.statusCode} when pinging ${url}`);
                    if (err.statusCode === 503) {
                        // We treat 503 as failures, because from a kube cluster it means the hostname is wrong, the ingress/route does not exist, etc.
                        return false;
                    }
                    return true;
                }
                // likely connection refused, socket timeout, etc.
                // so it was not reachable
                Logger_1.default.e(`Error pinging ${url}: ${err.message}`);
                return false;
            }
        });
    }
    Requester.ping = ping;
    /**
     * Repeatedly ping the given connection's 'ready' endpoint. The connection should not be used until that endpoint returns true.
     */
    function waitForReady(connection, timeoutS) {
        return __awaiter(this, void 0, void 0, function* () {
            const READY_DELAY_S = 2;
            const isCWReadyInitially = yield isCodewindReady(connection, false, READY_DELAY_S);
            if (isCWReadyInitially) {
                Logger_1.default.i(`${connection} was ready on first ping`);
                return true;
            }
            const maxTries = timeoutS / READY_DELAY_S;
            let tries = 0;
            return new Promise((resolve) => {
                const interval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    const logStatus = tries % 10 === 0;
                    if (logStatus) {
                        Logger_1.default.d(`Waiting ${connection} to be ready, ${tries * READY_DELAY_S}s have elapsed`);
                    }
                    const ready = yield isCodewindReady(connection, logStatus, READY_DELAY_S);
                    tries++;
                    if (ready) {
                        clearInterval(interval);
                        resolve(true);
                    }
                    else if (tries > maxTries) {
                        clearInterval(interval);
                        resolve(false);
                    }
                }), READY_DELAY_S * 1000);
            }).then((result) => {
                if (result) {
                    Logger_1.default.i(`Codewind was ready after ${tries} tries`);
                }
                else {
                    Logger_1.default.i(`Codewind was NOT ready after ${tries} tries`);
                }
                return result;
            });
        });
    }
    Requester.waitForReady = waitForReady;
    function isCodewindReady(connection, logStatus, timeoutS) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const res = yield doConnectionRequest(connection, Endpoints_1.MCEndpoints.READY, "GET", { timeout: timeoutS * 1000 });
                if (res === true) {
                    return true;
                }
            }
            catch (err) {
                if (logStatus) {
                    Logger_1.default.d("Error contacting ready endpoint", err);
                }
            }
            return false;
        });
    }
    Requester.ERR_LOGIN_PAGE = "Authentication required";
    // By enforcing all requests to go through this function,
    // we can inject options to abstract away required configuration like using json, handling ssl, and authentication.
    function req(verb, url, options = {}, accessToken) {
        return __awaiter(this, void 0, void 0, function* () {
            const optionsCopy = Object.assign({}, options);
            optionsCopy.json = true;
            // we resolve with full response so we can look out for redirects below
            optionsCopy.resolveWithFullResponse = true;
            optionsCopy.followRedirect = false;
            // TODO ...
            optionsCopy.rejectUnauthorized = false;
            if (!optionsCopy.timeout) {
                optionsCopy.timeout = 60000;
            }
            const requestFunc = HttpVerbs[verb];
            Logger_1.default.d(`Doing ${verb} request to ${url}`); // with options:`, options);
            if (accessToken) {
                if (!url.startsWith("https")) {
                    throw new Error(`Refusing to send access token to non-secure URL ${url}`);
                }
                optionsCopy.auth = {
                    bearer: accessToken.access_token,
                };
            }
            const response = yield requestFunc(url, optionsCopy);
            if (response.statusCode === 302 && response.headers.location && response.headers.location.includes("openid-connect/auth")) {
                throw new Error(Requester.ERR_LOGIN_PAGE);
            }
            const body = response.body;
            // Log.d(`Response body is:`, body);
            if (options.resolveWithFullResponse) {
                // Return the full response if it was originally requested in the options
                return response;
            }
            return body;
        });
    }
    function doConnectionRequest(connection, endpoint, verb, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // Re-enable once remote socket works
            // if (!connection.isConnected) {
            //     throw new Error("Can't do API call: Codewind is disconnected.");
            // }
            const url = Endpoints_1.default.resolveMCEndpoint(connection, endpoint);
            let accessToken;
            if (connection instanceof RemoteConnection_1.default) {
                accessToken = yield connection.getAccessToken();
            }
            return req(verb, url, options, accessToken);
        });
    }
    /**
     * Perform a REST request of the type specified by `requestFunc` to the project endpoint for the given project.
     * Displays a message to the user that the request succeeded if userOperationName is not null.
     * Always displays a message to the user in the case of an error.
     * @param body - JSON request body for POST, PUT, PATCH requests. Uses application/json content-type.
     * @param silent - If true, the `userOptionName` will not be displayed when the request is initiated. Error messages are always displayed.
     */
    function doProjectRequest(project, endpoint, body, verb, userOperationName, silent = false, returnFullRes = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = Endpoints_1.default.resolveProjectEndpoint(project.connection, project.id, endpoint);
            // Log.i(`Doing ${userOperationName} request to ${url}`);
            const options = {
                body,
                resolveWithFullResponse: returnFullRes,
            };
            try {
                let accessToken;
                if (project.connection instanceof RemoteConnection_1.default) {
                    accessToken = yield project.connection.getAccessToken();
                }
                const result = yield req(verb, url, options, accessToken);
                if (!silent) {
                    vscode.window.showInformationMessage(translator_1.default.t(STRING_NS, "requestSuccess", { operationName: userOperationName, projectName: project.name }));
                }
                return result;
            }
            catch (err) {
                Logger_1.default.w(`Error doing ${userOperationName} project request for ${project.name}:`, err);
                vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "requestFail", { operationName: userOperationName, projectName: project.name, err: MCUtil_1.default.errToString(err) }));
            }
        });
    }
})(Requester || (Requester = {}));
exports.default = Requester;
//# sourceMappingURL=Requester.js.map