"use strict";
/*******************************************************************************
 * Copyright (c) 2019, 2020 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const MCLog_1 = require("./MCLog");
const Logger_1 = require("../../../Logger");
const Requester_1 = require("../Requester");
const MCUtil_1 = require("../../../MCUtil");
var LogTypes;
(function (LogTypes) {
    LogTypes["APP"] = "app";
    LogTypes["BUILD"] = "build";
})(LogTypes = exports.LogTypes || (exports.LogTypes = {}));
class MCLogManager {
    constructor(project, doInitialize) {
        this.project = project;
        this._logs = [];
        this.initPromise = Promise.resolve();
        this.isShowingAll = false;
        if (doInitialize) {
            this.initPromise = this.initialize();
        }
        this.managerName = `${this.project.name} LogManager`;
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._logs.length > 0) {
                Logger_1.default.e(this.managerName + " logs have already been initialized");
                return;
            }
            // Log.d("Initializing logs");
            try {
                const availableLogs = yield Requester_1.default.requestAvailableLogs(this.project);
                yield this.onLogsListChanged(availableLogs);
                Logger_1.default.i(`${this.managerName} has finished initializing logs: ${this.logs.map((l) => l.displayName).join(", ") || "<none>"}`);
            }
            catch (err) {
                // requester will show the error
                return;
            }
        });
    }
    toString() {
        return this.managerName;
    }
    onLogsListChanged(logs) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            let didOpenNewLog = false;
            (_a = logs.app) === null || _a === void 0 ? void 0 : _a.forEach((log) => {
                didOpenNewLog = this.addLog(log, LogTypes.APP) || didOpenNewLog;
            });
            (_b = logs.build) === null || _b === void 0 ? void 0 : _b.forEach((log) => {
                didOpenNewLog = this.addLog(log, LogTypes.BUILD) || didOpenNewLog;
            });
            if (didOpenNewLog) {
                yield this.toggleLogStreaming(true);
            }
        });
    }
    /**
     * Adds the given log to this manager to track it.
     * The log is shown in the UI if we're showing all, or if it is a new instance of one we're already showing.
     * @returns If a new log was shown (opened).
     */
    addLog(logData, logType) {
        // skip useless container log
        if (logData.logName === "-" || logData.logName === "container") {
            return false;
        }
        const existingIndex = this.logs.findIndex((l) => l.logName === logData.logName);
        const existed = existingIndex !== -1;
        // open the log on creation if we're showing all, or if
        let openOnCreate = this.isShowingAll;
        if (existed) {
            // destroy the old log and replace it with this one
            const existingLog = this.logs.splice(existingIndex, 1)[0];
            openOnCreate = existingLog.isOpen;
            existingLog.removeOutput();
        }
        const newLog = new MCLog_1.default(this.project.name, logData.logName, logType, logData.workspaceLogPath);
        this.logs.push(newLog);
        if (openOnCreate) {
            Logger_1.default.d(`Revealing ${newLog.displayName}`);
            newLog.createOutput(false);
        }
        return openOnCreate;
    }
    showAll() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Showing all logs for " + this.project.name);
            this.isShowingAll = true;
            this.logs.forEach((log) => log.createOutput(true));
            yield this.toggleLogStreaming(true);
        });
    }
    /**
     * Shows the given logs and hides all other logs for this project.
     */
    showSome(toShow) {
        return __awaiter(this, void 0, void 0, function* () {
            if (toShow.length === 0) {
                yield this.hideAll();
                return;
            }
            Logger_1.default.d(`Showing ${toShow.length} logs for ${this.project.name}`);
            this.isShowingAll = false;
            this.logs.forEach((log) => {
                if (toShow.includes(log)) {
                    log.createOutput(true);
                }
                else {
                    log.removeOutput();
                }
            });
            // Refresh all the logs in case one of the ones just enabled is new.
            yield this.toggleLogStreaming(true);
        });
    }
    hideAll() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Hiding all logs for " + this.project.name);
            this.isShowingAll = false;
            this.logs.forEach((log) => log.removeOutput());
            yield this.toggleLogStreaming(false);
        });
    }
    /**
     * @param enable `true` to refresh (ie, restart) all logs for this project, `false` to stop streaming all logs for this project
     */
    toggleLogStreaming(enable) {
        return __awaiter(this, void 0, void 0, function* () {
            // Log.d(`${this.managerName} log streaming now ${enable}`);
            try {
                yield Requester_1.default.requestToggleLogs(this.project, enable);
            }
            catch (err) {
                const errMsg = `Error toggling logs ${enable ? "on" : "off"} for ${this.project.name}`;
                Logger_1.default.e(errMsg, err);
                vscode.window.showErrorMessage(`${errMsg}: ${MCUtil_1.default.errToString(err)}`);
            }
        });
    }
    onNewLogs(event) {
        if (event.projectID !== this.project.id) {
            Logger_1.default.e(`${this.managerName} received logs for other project ${event.projectName}`);
            return;
        }
        const existingLog = this.logs.find((log) => log.logName === event.logName);
        if (existingLog != null) {
            existingLog.onNewLogs(event.reset, event.logs);
        }
    }
    onReconnectOrEnable() {
        // Log.d(`${this.managerName} onReconnectOrEnable`);
        // refresh all streams
        if (this.logs.some((log) => log.isOpen)) {
            this.toggleLogStreaming(true);
        }
    }
    onDisconnect() {
        // Log.d(`${this.managerName} onDisconnectOrDisable`);
        this.logs.forEach((log) => log.onDisconnect());
    }
    get logs() {
        return this._logs;
    }
    destroyAllLogs() {
        this.isShowingAll = false;
        this.logs.forEach((log) => log.removeOutput());
        this._logs = [];
    }
}
exports.default = MCLogManager;
//# sourceMappingURL=MCLogManager.js.map