"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const path = require("path");
const Logger_1 = require("../../Logger");
const MCUtil_1 = require("../../MCUtil");
const SocketEvents_1 = require("../../codewind/connection/SocketEvents");
const ProjectType_1 = require("../../codewind/project/ProjectType");
const RegistryUtils_1 = require("../../codewind/connection/RegistryUtils");
const CLICommandRunner_1 = require("../../codewind/connection/CLICommandRunner");
const Requester_1 = require("../../codewind/project/Requester");
/**
 * @param create true for Create page, false for Import page
 */
function bindProjectCmd(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const defaultPath = vscode.workspace.workspaceFolders ? vscode.workspace.workspaceFolders[0].uri : undefined;
            const dirToBindUri = yield MCUtil_1.default.promptForProjectDir(`Add to ${connection.label}`, defaultPath);
            if (dirToBindUri == null) {
                return;
            }
            const response = yield detectAndBind(connection, dirToBindUri);
            if (response == null) {
                return;
            }
            vscode.window.showInformationMessage(`Added ${MCUtil_1.default.containerPathToFsPath(response.projectPath)} as ${response.projectName}`);
        }
        catch (err) {
            const errMsg = "Error importing project: ";
            Logger_1.default.e(errMsg, err);
            vscode.window.showErrorMessage(errMsg + MCUtil_1.default.errToString(err));
        }
    });
}
exports.default = bindProjectCmd;
function detectAndBind(connection, pathToBindUri) {
    return __awaiter(this, void 0, void 0, function* () {
        const pathToBind = pathToBindUri.fsPath;
        Logger_1.default.i("Binding to", pathToBind);
        const projectName = path.basename(pathToBind);
        const validateRes = yield detectProjectType(connection, pathToBind);
        if (validateRes.status !== SocketEvents_1.default.STATUS_SUCCESS) {
            // failed
            const failedResult = validateRes.result;
            throw new Error(failedResult.error);
        }
        let projectTypeInfo = validateRes.result;
        // if the detection returned the fallback type, or if the user says the detection is wrong
        let detectionFailed = false;
        if (projectTypeInfo.projectType === ProjectType_1.default.InternalTypes.DOCKER) {
            detectionFailed = true;
        }
        else {
            const yesBtn = "Yes";
            const noBtn = "No";
            const confirmRes = yield vscode.window.showInformationMessage(`Please confirm the project type for ${projectName}:\n` +
                `Type: ${projectTypeInfo.projectType}\n` +
                `Language: ${projectTypeInfo.language}`, { modal: true }, yesBtn, noBtn);
            if (confirmRes == null) {
                return;
            }
            else if (confirmRes === noBtn) {
                detectionFailed = true;
            }
            // else they picked 'yes'
        }
        if (detectionFailed) {
            const userProjectType = yield promptForProjectType(connection, projectTypeInfo);
            if (userProjectType == null) {
                return;
            }
            projectTypeInfo = userProjectType;
        }
        if (yield RegistryUtils_1.default.doesNeedPushRegistry(projectTypeInfo.projectType, connection)) {
            return;
        }
        // validate once more with detected type and subtype (if defined),
        // to run any extension defined command involving subtype
        if (projectTypeInfo.projectSubtype) {
            yield detectProjectType(connection, pathToBind, projectTypeInfo.projectType + ":" + projectTypeInfo.projectSubtype);
        }
        yield addProjectToConnection(connection, projectName, pathToBind, projectTypeInfo);
        return { projectName, projectPath: pathToBind };
    });
}
const OTHER_TYPE_OPTION = "Other (Codewind Basic Container)";
/**
 * When detection fails, have the user select the project type that fits best.
 */
function promptForProjectType(connection, detected) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d("Prompting user for project type");
        const projectTypes = yield vscode.window.withProgress({
            cancellable: false,
            location: vscode.ProgressLocation.Notification,
            title: `Fetching project types...`,
        }, () => {
            return Requester_1.default.getProjectTypes(connection);
        });
        const projectTypeQpis = [];
        let dockerType;
        projectTypes.forEach((type, index) => {
            if (type.projectType === ProjectType_1.default.InternalTypes.DOCKER) {
                dockerType = {
                    label: OTHER_TYPE_OPTION,
                    projectType: OTHER_TYPE_OPTION,
                    index: index
                };
            }
            else {
                let label;
                let description;
                // not codewind type if it has a label
                if (type.projectSubtypes.label) {
                    // if only 1 subtype take the label and description of that subtype
                    if (type.projectSubtypes.items.length === 1) {
                        label = type.projectSubtypes.items[0].label;
                        description = type.projectSubtypes.items[0].description;
                    }
                    else {
                        label = type.projectType;
                    }
                }
                else {
                    label = `Codewind ${type.projectType}`;
                }
                projectTypeQpis.push({
                    label,
                    projectType: type.projectType,
                    index: index,
                    description
                });
            }
        });
        projectTypeQpis.sort((a, b) => a.label.localeCompare(b.label));
        // Add "other" option last
        if (dockerType) {
            projectTypeQpis.push(dockerType);
        }
        const projectTypeRes = yield vscode.window.showQuickPick(projectTypeQpis, {
            placeHolder: "Select the project type that best fits your project",
            ignoreFocusOut: true,
        });
        if (projectTypeRes == null) {
            return;
        }
        let language = detected.language;
        const projectType = projectTypeRes.projectType;
        // If project type selection did not change, there's no need to prompt for language/subtype, consider:
        // 1) changing selection of "liberty" to "liberty", it still maps to 1 language (same applies to all known project types)
        // 2) exception: selection of "other" !== "docker", this should allow for the selection of language
        // 3) selecting language is not applicable to entension project, unless selection changes from something else,
        //    in that case we prompt for the subtype
        const projectSubtypeChoices = (projectType !== detected.projectType) ? projectTypes[projectTypeRes.index].projectSubtypes : null;
        let projectSubtype;
        // have choices to potentially present to user
        if (projectSubtypeChoices) {
            // not really, only 1 choice
            if (projectSubtypeChoices.items.length === 1) {
                projectSubtype = projectSubtypeChoices.items[0].id;
            }
            // let's prompt user
            else {
                projectSubtype = yield promptForLanguageOrSubtype(projectSubtypeChoices);
                if (projectSubtype == null) {
                    return;
                }
            }
            // if there's no custom label, we were choosing language
            if (!projectSubtypeChoices.label) {
                language = projectSubtype;
                projectSubtype = undefined;
            }
        }
        return {
            language,
            // map the 'other' back to the docker type
            projectType: projectType === OTHER_TYPE_OPTION ? ProjectType_1.default.InternalTypes.DOCKER : projectType,
            projectSubtype
        };
    });
}
const OTHER_LANG_BTN = "Other";
function promptForLanguageOrSubtype(choices) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d("Prompting user for project language or subtype");
        const languageQpis = choices.items.map((choice) => {
            return {
                id: choice.id,
                label: choice.label,
                description: choice.description
            };
        });
        // remove duplicates
        languageQpis.sort((a, b) => a.label.localeCompare(b.label));
        if (!choices.label) {
            languageQpis.push({ id: "", label: OTHER_LANG_BTN });
        }
        const language = yield vscode.window.showQuickPick(languageQpis, {
            placeHolder: choices.label ? `Select the ${choices.label}` : "Select the language that best fits your project",
            ignoreFocusOut: true,
        });
        if (language == null) {
            return;
        }
        else if (language.label === OTHER_LANG_BTN) {
            return yield vscode.window.showInputBox({
                ignoreFocusOut: true,
                prompt: "Enter the programming language for your project",
                validateInput: (input) => {
                    const matchResult = input.match(/^[a-zA-Z0-9\s]+$/);
                    if (matchResult == null) {
                        return `The programming language cannot be empty, and can only contain alphanumeric characters and spaces.`;
                    }
                    return undefined;
                }
            });
        }
        return language.id;
    });
}
function detectProjectType(connection, pathToBind, desiredType) {
    return __awaiter(this, void 0, void 0, function* () {
        return CLICommandRunner_1.CLICommandRunner.detectProjectType(connection.id, pathToBind, desiredType);
        // Log.d("Detection response", detectResponse);
        // return detectResponse;
    });
}
function addProjectToConnection(connection, projectName, pathToBind, projectTypeInfo) {
    return __awaiter(this, void 0, void 0, function* () {
        yield vscode.window.withProgress({
            cancellable: false,
            location: vscode.ProgressLocation.Notification,
            title: `Adding ${projectName} to ${connection.label}...`
        }, () => {
            return CLICommandRunner_1.CLICommandRunner.bindProject(connection.id, projectName, pathToBind, projectTypeInfo);
        });
    });
}
exports.addProjectToConnection = addProjectToConnection;
//# sourceMappingURL=BindProjectCmd.js.map