"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const path = require("path");
const Logger_1 = require("../../Logger");
const MCUtil_1 = require("../../MCUtil");
const Requester_1 = require("../../codewind/project/Requester");
const Configurations_1 = require("../../constants/Configurations");
const RegistryUtils_1 = require("../../codewind/connection/RegistryUtils");
const Resources_1 = require("../../constants/Resources");
const CLICommandRunner_1 = require("../../codewind/connection/CLICommandRunner");
const ManageSourcesCmd_1 = require("./ManageSourcesCmd");
const SocketEvents_1 = require("../../codewind/connection/SocketEvents");
const CREATE_PROJECT_WIZARD_NO_STEPS = 2;
const BACK_BTN_MSG = "Back button";
const HAS_SELECTED_SOURCE_KEY = "first-create-done";
function createProjectCmd(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!connection.isRemote) {
            // On initial project create, prompt the user to select a template source
            const extState = global.extGlobalState;
            const hasSelectedSource = extState.get(HAS_SELECTED_SOURCE_KEY);
            if (!hasSelectedSource) {
                try {
                    const selectedSource = yield showTemplateSourceQuickpick(connection);
                    if (selectedSource == null) {
                        return;
                    }
                    if (connection.sourcesPage) {
                        connection.sourcesPage.refresh();
                    }
                    extState.update(HAS_SELECTED_SOURCE_KEY, true);
                    if (selectedSource === "managed") {
                        ManageSourcesCmd_1.default(connection);
                        // Don't continue with the create in this case.
                        return;
                    }
                }
                catch (err) {
                    const errMsg = `Error fetching template sources`;
                    Logger_1.default.e(errMsg, err);
                    vscode.window.showErrorMessage(`${errMsg}: ${MCUtil_1.default.errToString(err)}`);
                }
            }
        }
        try {
            let template;
            let projectName;
            while (!template || !projectName) {
                template = yield promptForTemplate(connection);
                if (template == null) {
                    return;
                }
                else if (yield RegistryUtils_1.default.doesNeedPushRegistry(template.projectType, connection)) {
                    // The user needs to configure a push registry before they can create this type of project
                    return;
                }
                try {
                    projectName = yield promptForProjectName(connection, template);
                    if (projectName == null) {
                        return;
                    }
                }
                catch (err) {
                    if (err !== BACK_BTN_MSG) {
                        // unexpected error
                        throw err;
                    }
                    // else user clicked back, return to top of loop
                }
            }
            // Get the parent directory to create the project under
            let parentDir;
            if (vscode.workspace.workspaceFolders
                && (global.isTheia || Configurations_1.CWConfigurations.ALWAYS_CREATE_IN_WORKSPACE.get())) {
                if (vscode.workspace.workspaceFolders.length === 1) {
                    parentDir = vscode.workspace.workspaceFolders[0].uri;
                }
                else {
                    const selection = yield showWorkspaceFolderSelection();
                    if (!selection) {
                        return undefined;
                    }
                    parentDir = selection;
                }
            }
            else {
                // Have the user select the parent dir from anywhere on disk
                const defaultUri = vscode.workspace.workspaceFolders && vscode.workspace.workspaceFolders[0] ?
                    vscode.workspace.workspaceFolders[0].uri : undefined;
                const selection = yield MCUtil_1.default.promptForProjectDir("Select Parent Directory", defaultUri);
                if (!selection) {
                    return undefined;
                }
                parentDir = selection;
            }
            const response = yield createProject(connection, template, parentDir, projectName);
            if (!response) {
                // user cancelled
                return;
            }
            vscode.window.showInformationMessage(`Created project ${response.projectName} at ${MCUtil_1.default.containerPathToFsPath(response.projectPath)}`);
        }
        catch (err) {
            const errMsg = "Error creating new project: ";
            Logger_1.default.e(errMsg, err);
            vscode.window.showErrorMessage(errMsg + MCUtil_1.default.errToString(err));
        }
    });
}
exports.default = createProjectCmd;
function showWorkspaceFolderSelection() {
    return __awaiter(this, void 0, void 0, function* () {
        const selection = yield vscode.window.showWorkspaceFolderPick({
            ignoreFocusOut: true,
            placeHolder: "Select the parent directory for your new project",
        });
        if (selection == null) {
            return undefined;
        }
        return selection.uri;
    });
}
const MANAGE_SOURCES_ITEM = "Manage Template Sources";
function showTemplateSourceQuickpick(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        const sources = yield connection.templateSourcesList.get();
        if (sources.length === 0) {
            // Should not be possible because Codewind templates are always present.
            throw new Error("No template sources are configured. Use the Manage Template Sources command to add the Codewind templates.");
        }
        if (sources.length === 1) {
            // if there is exactly one repo, just enable it and move on.
            yield connection.templateSourcesList.toggleEnablement({
                repos: [{
                        enable: true,
                        repoID: sources[0].url,
                    }]
            });
            return "selected";
        }
        const qpis = sources.map((repo) => {
            const label = repo.name || repo.description || "No name available";
            const description = repo.name ? repo.description : undefined;
            return {
                url: repo.url,
                label,
                description,
                detail: repo.url,
            };
        });
        qpis.push({
            label: MANAGE_SOURCES_ITEM,
            detail: "Select multiple sources and learn more about template sources.",
            // special case
            url: "",
        });
        const selection = yield vscode.window.showQuickPick(qpis, {
            placeHolder: `Select one of the template sources below, or select "${MANAGE_SOURCES_ITEM}".`
        });
        if (selection == null) {
            return undefined;
        }
        if (selection.label === MANAGE_SOURCES_ITEM) {
            return "managed";
        }
        // enable the selected repo, only
        const repoEnablement = sources.map((repo) => {
            return {
                enable: repo.url === selection.url,
                repoID: repo.url,
            };
        });
        yield connection.templateSourcesList.toggleEnablement({ repos: repoEnablement });
        vscode.window.showInformationMessage(`Set template source to ${selection.label}. The other sources have been disabled. You can change this setting at any time with the Manage Template Sources command. `);
        return "selected";
    });
}
function getWizardTitle(connection) {
    return `(${connection.label}) Create a New Project`;
}
const MANAGE_SOURCES_QP_BTN = "Manage Template Sources";
function promptForTemplate(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        const qp = vscode.window.createQuickPick();
        // busy and enabled have no effect in theia https://github.com/eclipse-theia/theia/issues/5059
        qp.busy = true;
        qp.enabled = false;
        qp.placeholder = "Fetching available project templates...";
        qp.buttons = [{
                iconPath: Resources_1.default.getIconPaths(Resources_1.default.Icons.Edit),
                tooltip: MANAGE_SOURCES_QP_BTN,
            }];
        qp.matchOnDetail = true;
        qp.canSelectMany = false;
        qp.step = 1;
        qp.totalSteps = CREATE_PROJECT_WIZARD_NO_STEPS;
        qp.title = getWizardTitle(connection);
        qp.ignoreFocusOut = true;
        if (!global.isTheia) {
            // Theia quickpicks misbehave if the quickpick is shown before populating the items
            // https://github.com/eclipse-theia/theia/issues/6221#issuecomment-533268856
            // In VS Code, the items are populated after showing, so we can show the quickpick sooner, which looks better.
            qp.show();
        }
        let templateQpis;
        try {
            templateQpis = yield getTemplateQpis(connection);
        }
        catch (err) {
            qp.hide();
            throw err;
        }
        if (templateQpis == null) {
            // getTemplateQpis will have shown the error message
            qp.hide();
            return undefined;
        }
        qp.items = templateQpis;
        qp.placeholder = "Select the project type to create";
        qp.busy = false;
        qp.enabled = true;
        if (global.isTheia) {
            // it wasn't shown above, so show it now
            qp.show();
        }
        const qpiSelection = yield new Promise((resolve) => {
            qp.onDidTriggerButton((btn) => {
                if (btn.tooltip === MANAGE_SOURCES_QP_BTN) {
                    ManageSourcesCmd_1.default(connection);
                    resolve(undefined);
                }
            });
            qp.onDidHide((_e) => {
                resolve(undefined);
            });
            // it looks funny to use onDidChangeSelection instead of onDidAccept,
            // but it behaves the same when there's just one item since we can only make one selection.
            // this is a workaround for https://github.com/eclipse-theia/theia/issues/6221
            // qp.onDidAccept(() => {
            //     Log.d("onDidAccept, qp.selectedItems are", qp.selectedItems);
            //     resolve(qp.selectedItems);
            // });
            qp.onDidChangeSelection((selection) => {
                // Log.d("onDidChangeSelection, qp.selectedItems are", qp.selectedItems);
                // Log.d("onDidChangeSelection, selection is ", selection);
                resolve(selection);
            });
        })
            .finally(() => qp.dispose());
        // there are either 1 or 0 items selected because canSelectMany is false
        if (qpiSelection == null || qpiSelection.length === 0 || qpiSelection[0] == null) {
            return undefined;
        }
        const selected = qpiSelection[0];
        // map the selected QPI back to the template it represents
        const selectedProjectType = templateQpis.find((type) => selected.label === type.label);
        if (selectedProjectType == null) {
            // should never happen
            throw new Error(`Could not find template ${selected.label}`);
        }
        return selectedProjectType;
    });
}
function getTemplateQpis(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        const templates = yield Requester_1.default.getTemplates(connection);
        Logger_1.default.d(`Finished GETing templates`);
        // if there are multiple sources enabled, we append the source name to the template label to clarify where the template is from
        const areMultipleSourcesEnabled = new Set(templates.map((template) => template.source)).size > 1;
        if (areMultipleSourcesEnabled) {
            templates.forEach((template) => {
                const source = template.source || "Unnamed source";
                template.label += ` (${source})`;
            });
        }
        const templateQpis = templates.map((template) => {
            return Object.assign(Object.assign({}, template), { detail: template.language, extension: template.url });
        });
        if (templateQpis.length === 0) {
            // The user has no repos or has disabled all repos
            const manageReposBtn = "Manage Template Sources";
            yield vscode.window.showErrorMessage("You have no enabled template sources. You must enable at least one template source in order to create projects.", manageReposBtn)
                .then((res) => {
                if (res === manageReposBtn) {
                    ManageSourcesCmd_1.default(connection);
                }
            });
            return undefined;
        }
        return templateQpis;
    });
}
function promptForProjectName(connection, template) {
    return __awaiter(this, void 0, void 0, function* () {
        const projNamePlaceholder = `my-${template.language}-project`;
        const projNamePrompt = `Enter a name for your new ${template.language} project`;
        const ib = vscode.window.createInputBox();
        ib.title = getWizardTitle(connection);
        ib.step = 2;
        ib.totalSteps = CREATE_PROJECT_WIZARD_NO_STEPS;
        ib.buttons = [vscode.QuickInputButtons.Back];
        ib.placeholder = projNamePlaceholder;
        ib.prompt = projNamePrompt;
        ib.ignoreFocusOut = true;
        ib.onDidChangeValue((projName) => {
            ib.validationMessage = validateProjectName(projName);
        });
        return new Promise((resolve, reject) => {
            ib.show();
            ib.onDidHide((_e) => {
                resolve(undefined);
            });
            ib.onDidAccept((_e) => {
                resolve(ib.value);
            });
            ib.onDidTriggerButton((_btn) => {
                // back button is the only button
                reject(BACK_BTN_MSG);
            });
        })
            .finally(() => ib.dispose());
    });
}
const ILLEGAL_CHARS = [
    `"`, "/", "\\", "?", "%", "*", ":", "|", "<", ">", "&", " "
];
function validateProjectName(projectName) {
    const firstIllegalChar = [...projectName].find((c) => ILLEGAL_CHARS.includes(c));
    if (firstIllegalChar != null) {
        return `Invalid project name "${projectName}". Project names may not contain "${firstIllegalChar}"`;
    }
    // const matches: boolean = /^[a-z0-9_.-]+$/.test(projectName);
    // if (!matches) {
    // tslint:disable-next-line: max-line-length
    //     return `Invalid project name "${projectName}". Project name can only contain numbers, lowercase letters, periods, hyphens, and underscores.`;
    // }
    return undefined;
}
function createProject(connection, template, parentDir, projectName) {
    return __awaiter(this, void 0, void 0, function* () {
        const projectPath = path.join(parentDir.fsPath, projectName);
        yield vscode.window.withProgress({
            cancellable: false,
            location: vscode.ProgressLocation.Notification,
            title: `Creating ${projectName}...`
        }, () => __awaiter(this, void 0, void 0, function* () {
            const creationRes = yield CLICommandRunner_1.CLICommandRunner.createProject(connection.id, projectPath, template.url);
            if (creationRes.status !== SocketEvents_1.default.STATUS_SUCCESS) {
                // failed
                let failedReason = `Unknown error creating ${projectName} at ${projectPath}`;
                try {
                    failedReason = creationRes.result.error || creationRes.result;
                }
                // tslint:disable-next-line: no-empty
                catch (err) { }
                throw new Error(failedReason);
            }
            // create succeeded, now we bind
            const projectType = { projectType: template.projectType, language: template.language };
            yield CLICommandRunner_1.CLICommandRunner.bindProject(connection.id, projectName, projectPath, projectType);
        }));
        return { projectName, projectPath };
    });
}
exports.createProject = createProject;
//# sourceMappingURL=CreateUserProjectCmd.js.map