"use strict";
/*******************************************************************************
 * Copyright (c) 2020 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Logger_1 = require("../../Logger");
const ConnectionManager_1 = require("../../codewind/connection/ConnectionManager");
const ToggleEnablementCmd_1 = require("./ToggleEnablementCmd");
const BindProjectCmd_1 = require("../connection/BindProjectCmd");
const RegistryUtils_1 = require("../../codewind/connection/RegistryUtils");
const RemoveProjectCmd_1 = require("./RemoveProjectCmd");
const MCUtil_1 = require("../../MCUtil");
function changeProjectConnectionCmd(project) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const selectableConnections = ConnectionManager_1.default.instance.connections.filter((conn) => 
            // we show a given connection if the other connection is connected and does not already have this project
            conn.isConnected && conn !== project.connection && !conn.hasProjectAtPath(project.localPath));
            if (selectableConnections.length === 0) {
                vscode.window.showWarningMessage(`No suitable target for moving ${project.name} - ` +
                    `there is no connection that is connected and does not already have this project.`);
                return;
            }
            const targetConnection = yield vscode.window.showQuickPick(selectableConnections, {
                canPickMany: false,
                ignoreFocusOut: true,
                matchOnDetail: true,
                placeHolder: `Select the connection to move ${project.name} to.`,
            });
            if (targetConnection == null) {
                return;
            }
            if (yield RegistryUtils_1.default.doesNeedPushRegistry(project.type.internalType, targetConnection)) {
                return;
            }
            // Ask what to do with the existing instance of the project
            const optionRemove = {
                label: "Remove",
                detail: `Remove the project from ${project.connection.label}. The source code is not affected.`,
            };
            const optionDisable = {
                label: "Disable",
                detail: `Keep the project on ${project.connection.label}, but Disable it so it doesn't build there.`,
            };
            const optionNothing = {
                label: "Leave it alone",
                detail: `Leave the project running on ${project.connection.label}. It will be built on both Codewind instances.`
            };
            const options = [optionRemove];
            if (project.state.isEnabled) {
                options.push(optionDisable);
            }
            options.push(optionNothing);
            const existingActionResponse = yield vscode.window.showQuickPick(options, {
                canPickMany: false,
                ignoreFocusOut: true,
                placeHolder: `Select what to do with the existing instance of ${project.name} deployed on ${project.connection.label}.`,
            });
            if (existingActionResponse == null) {
                return;
            }
            Logger_1.default.i(`Moving ${project.name} from ${project.connection.label} to ${targetConnection.label}`);
            const projectType = {
                language: project.language,
                projectType: project.type.internalType,
            };
            try {
                yield BindProjectCmd_1.addProjectToConnection(targetConnection, project.name, project.localPath.fsPath, projectType);
            }
            catch (err) {
                const errMsg = `Failed to add ${project.name} to ${targetConnection.label}`;
                Logger_1.default.e(errMsg, err);
                // We use regex instead .includes here because for some reason this error message uses non-breaking spaces
                if (/type\sis\sinvalid/.test(err.toString())) {
                    Logger_1.default.i(`Cannot add ${project.type.internalType} project to ${targetConnection.label}`);
                    vscode.window.showErrorMessage(`${errMsg}: ${targetConnection.label} does not support ${project.type.type} projects.`);
                }
                else {
                    vscode.window.showErrorMessage(`${errMsg}: ${MCUtil_1.default.errToString(err)}`);
                }
                // Don't clean up the project from the old instance if it failed to get added to the new one
                return;
            }
            if (existingActionResponse === optionRemove) {
                yield RemoveProjectCmd_1.removeProject(project, false);
            }
            else if (existingActionResponse === optionDisable) {
                yield ToggleEnablementCmd_1.default(project);
            }
            // else, they selected to leave the existing project
        }
        catch (err) {
            const errMsg = `Error moving ${project.name}`;
            Logger_1.default.e(errMsg, err);
            vscode.window.showErrorMessage(`${errMsg}: ${MCUtil_1.default.errToString(err)}`);
        }
    });
}
exports.default = changeProjectConnectionCmd;
//# sourceMappingURL=ChangeProjectConnectionCmd.js.map