"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Resources_1 = require("../../constants/Resources");
const ConnectionOverviewPage_1 = require("./pages/ConnectionOverviewPage");
const Logger_1 = require("../../Logger");
const ConnectionManager_1 = require("../../codewind/connection/ConnectionManager");
const MCUtil_1 = require("../../MCUtil");
const url_1 = require("url");
const Requester_1 = require("../../codewind/project/Requester");
const RemoveConnectionCmd_1 = require("../connection/RemoveConnectionCmd");
const ToggleConnectionEnablement_1 = require("../connection/ToggleConnectionEnablement");
const ManageRegistriesCmd_1 = require("../connection/ManageRegistriesCmd");
const ManageSourcesCmd_1 = require("../connection/ManageSourcesCmd");
const WebviewWrapper_1 = require("./WebviewWrapper");
var ConnectionOverviewWVMessages;
(function (ConnectionOverviewWVMessages) {
    ConnectionOverviewWVMessages["HELP"] = "help";
    ConnectionOverviewWVMessages["SAVE_CONNECTION_INFO"] = "save-connection";
    ConnectionOverviewWVMessages["TOGGLE_CONNECTED"] = "toggleConnected";
    ConnectionOverviewWVMessages["DELETE"] = "delete";
    ConnectionOverviewWVMessages["CANCEL"] = "cancel";
    ConnectionOverviewWVMessages["REGISTRY"] = "registry";
    ConnectionOverviewWVMessages["SOURCES"] = "sources";
})(ConnectionOverviewWVMessages = exports.ConnectionOverviewWVMessages || (exports.ConnectionOverviewWVMessages = {}));
class ConnectionOverviewWrapper extends WebviewWrapper_1.WebviewWrapper {
    /////
    constructor(connectionInfo, connection) {
        super(connectionInfo.label, Resources_1.default.Icons.Logo);
        this.handleWebviewMessage = (msg) => __awaiter(this, void 0, void 0, function* () {
            try {
                switch (msg.type) {
                    case ConnectionOverviewWVMessages.HELP: {
                        vscode.window.showInformationMessage("Help about this page");
                        break;
                    }
                    case ConnectionOverviewWVMessages.SAVE_CONNECTION_INFO: {
                        const newInfo = msg.data;
                        if (this.connection) {
                            if (!newInfo.username) {
                                vscode.window.showErrorMessage(`Enter a username`);
                            }
                            else if (!newInfo.password) {
                                vscode.window.showErrorMessage(`Enter a password`);
                            }
                            else {
                                try {
                                    yield this.connection.updateCredentials(newInfo.username, newInfo.password);
                                }
                                catch (err) {
                                    const errMsg = `Error updating ${this.connection.label}:`;
                                    vscode.window.showErrorMessage(`${errMsg} ${MCUtil_1.default.errToString(err)}`);
                                    Logger_1.default.e(errMsg, err);
                                }
                                this.refresh();
                            }
                        }
                        else {
                            try {
                                const newConnection = yield this.createNewConnection(newInfo, this.label);
                                this.connection = newConnection;
                                this.connection.onDidOpenOverview(this);
                                vscode.window.showInformationMessage(`Successfully created new connection ${this.label} to ${newConnection.url}`);
                                this.refresh();
                            }
                            catch (err) {
                                // the err from createNewConnection is user-friendly
                                Logger_1.default.w(`Error creating new connection to ${newInfo.url}`, err);
                                vscode.window.showErrorMessage(`${MCUtil_1.default.errToString(err)}`);
                            }
                        }
                        break;
                    }
                    case ConnectionOverviewWVMessages.TOGGLE_CONNECTED: {
                        if (this.connection) {
                            yield ToggleConnectionEnablement_1.default(this.connection, !this.connection.enabled);
                        }
                        else {
                            Logger_1.default.e("Received Toggle Connected event but there is no connection");
                        }
                    }
                    case ConnectionOverviewWVMessages.CANCEL: {
                        if (this.connection) {
                            this.refresh();
                        }
                        else {
                            this.dispose();
                        }
                        break;
                    }
                    case ConnectionOverviewWVMessages.DELETE: {
                        if (this.connection) {
                            const didRemove = yield RemoveConnectionCmd_1.default(this.connection);
                            if (didRemove) {
                                this.dispose();
                            }
                        }
                        else {
                            vscode.window.showInformationMessage(`Creating new connection cancelled`);
                        }
                        break;
                    }
                    case ConnectionOverviewWVMessages.REGISTRY: {
                        if (this.connection) {
                            ManageRegistriesCmd_1.default(this.connection);
                        }
                        else {
                            vscode.window.showInformationMessage("Create your new connection by pressing Save before proceeding to the next step.");
                        }
                        break;
                    }
                    case ConnectionOverviewWVMessages.SOURCES: {
                        if (this.connection) {
                            ManageSourcesCmd_1.default(this.connection);
                        }
                        else {
                            vscode.window.showInformationMessage("Create your new connection by pressing Save before proceeding to the next step.");
                        }
                        break;
                    }
                    default:
                        Logger_1.default.e("Received unexpected WebView message in Connection Overview page", msg);
                }
            }
            catch (err) {
                const errMsg = `Connection Overview error: ${MCUtil_1.default.errToString(err)}`;
                vscode.window.showErrorMessage(errMsg);
                Logger_1.default.e(errMsg, err);
            }
        });
        this.label = connectionInfo.label;
        this.connection = connection;
        this.refresh();
    }
    static showForNewConnection(label) {
        return new ConnectionOverviewWrapper({ label });
    }
    static showForExistingConnection(connection) {
        if (connection.overviewPage) {
            connection.overviewPage.reveal();
            return connection.overviewPage;
        }
        return new ConnectionOverviewWrapper(connection.cliData, connection);
    }
    getTitle() {
        return this.label;
    }
    generateHtml(resourceProvider) {
        return __awaiter(this, void 0, void 0, function* () {
            let isConnnected = false;
            if (this.connection) {
                isConnnected = this.connection.isConnected;
            }
            const connectionInfo = this.connection ? this.connection.cliData : { label: this.label };
            return ConnectionOverviewPage_1.default(resourceProvider, connectionInfo, isConnnected);
        });
    }
    onDidDispose() {
        var _a;
        (_a = this.connection) === null || _a === void 0 ? void 0 : _a.onDidCloseOverview();
    }
    /**
     * Tries to create a new connection from the given info.
     * Returns the new Connection if it succeeds. Returns undefined if user cancels. Throws errors.
     */
    createNewConnection(newConnectionInfo, label) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!newConnectionInfo.url) {
                throw new Error("Enter a Codewind Gatekeeper ingress host");
            }
            Logger_1.default.d("Ingress host is", newConnectionInfo.url);
            let ingressUrlStr = newConnectionInfo.url.trim();
            try {
                if (!ingressUrlStr.includes("://")) {
                    Logger_1.default.d(`No protocol; assuming https`);
                    ingressUrlStr = `https://${ingressUrlStr}`;
                }
                const ingressAsUrl = new url_1.URL(ingressUrlStr);
                if (!ingressAsUrl.protocol.startsWith("https")) {
                    throw new Error(`Protocol must be https, or omitted.`);
                }
            }
            catch (err) {
                throw new Error(`"${ingressUrlStr}" is not a valid URL: ${MCUtil_1.default.errToString(err)}`);
            }
            if (!newConnectionInfo.username) {
                throw new Error(`Enter a username for ${ingressUrlStr}`);
            }
            else if (!newConnectionInfo.password) {
                throw new Error(`Enter a password for ${ingressUrlStr}`);
            }
            const ingressUrl = vscode.Uri.parse(ingressUrlStr);
            const username = newConnectionInfo.username;
            const password = newConnectionInfo.password;
            return vscode.window.withProgress({
                cancellable: false,
                location: vscode.ProgressLocation.Notification,
                title: `Creating ${label}...`,
            }, () => __awaiter(this, void 0, void 0, function* () {
                const canPing = yield Requester_1.default.ping(ingressUrl);
                if (!canPing) {
                    throw new Error(`Failed to contact ${ingressUrl}. Make sure this URL is reachable.`);
                }
                const newConnection = yield ConnectionManager_1.default.instance.createRemoteConnection(ingressUrl, this.label, username, password);
                return newConnection;
            }));
        });
    }
}
exports.default = ConnectionOverviewWrapper;
//# sourceMappingURL=ConnectionOverviewPageWrapper.js.map