"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Resources_1 = require("../../constants/Resources");
const WebviewUtil_1 = require("./WebviewUtil");
const Logger_1 = require("../../Logger");
const RegistriesPage_1 = require("./pages/RegistriesPage");
const Requester_1 = require("../../codewind/project/Requester");
const MCUtil_1 = require("../../MCUtil");
const RegistryUtils_1 = require("../../codewind/connection/RegistryUtils");
const CWDocs_1 = require("../../constants/CWDocs");
const Commands_1 = require("../../constants/Commands");
const WebviewWrapper_1 = require("./WebviewWrapper");
const ConnectionOverviewCmd_1 = require("../connection/ConnectionOverviewCmd");
var ManageRegistriesWVMessages;
(function (ManageRegistriesWVMessages) {
    // EDIT = "edit",
    ManageRegistriesWVMessages["CHANGE_PUSH"] = "change-push";
})(ManageRegistriesWVMessages = exports.ManageRegistriesWVMessages || (exports.ManageRegistriesWVMessages = {}));
function getTitle(connection) {
    let title = "Image Registry Manager";
    if (!global.isTheia) {
        title += ` (${connection.label})`;
    }
    return title;
}
class RegistriesPageWrapper extends WebviewWrapper_1.WebviewWrapper {
    constructor(connection) {
        super(getTitle(connection), Resources_1.default.Icons.Logo);
        this.connection = connection;
        this.registries = [];
        this.handleWebviewMessage = (msg) => __awaiter(this, void 0, void 0, function* () {
            switch (msg.type) {
                case ManageRegistriesWVMessages.CHANGE_PUSH: {
                    const data = msg.data;
                    const newPushRegistry = this.lookupRegistry(data.fullAddress);
                    if (newPushRegistry.isPushRegistry) {
                        // This is not ideal but once there is a push registry the only way to remove it is to select a new one
                        vscode.window.showWarningMessage(`To stop using the current push registry ${newPushRegistry.fullAddress}, select a new push registry.`);
                        return;
                    }
                    try {
                        const currentPushRegistry = this.registries.find((reg) => reg.isPushRegistry);
                        const didUpdatePushRegistry = yield RegistryUtils_1.default.setPushRegistry(this.connection, currentPushRegistry, newPushRegistry, true);
                        if (!didUpdatePushRegistry) {
                            Logger_1.default.d(`User cancelled changing push registry`);
                            return;
                        }
                        vscode.window.showInformationMessage(`Successfully changed push registry to ${didUpdatePushRegistry.fullAddress}`);
                    }
                    catch (err) {
                        const errMsg = `Failed to update push registry to ${newPushRegistry.fullAddress}`;
                        Logger_1.default.e(errMsg, err);
                        vscode.window.showErrorMessage(`${errMsg}: ${MCUtil_1.default.errToString(err)}`);
                    }
                    yield this.refresh();
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.ADD_NEW: {
                    try {
                        const added = yield RegistryUtils_1.default.addNewRegistry(this.connection, this.registries);
                        if (!added) {
                            Logger_1.default.d(`User cancelled added new image registry`);
                            return;
                        }
                    }
                    catch (err) {
                        const errMsg = `Failed to add new image registry`;
                        Logger_1.default.e(errMsg, err);
                        vscode.window.showErrorMessage(`${errMsg}: ${MCUtil_1.default.errToString(err)}`);
                    }
                    yield this.refresh();
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.DELETE: {
                    const data = msg.data;
                    const registry = this.lookupRegistry(data.fullAddress);
                    if (registry.isPushRegistry) {
                        const confirmBtn = "Remove Anyway";
                        const confirmRes = yield vscode.window.showWarningMessage(`${registry.fullAddress} is currently set as your image push registry. \n` +
                            `Removing it will cause Codewind-style project builds to fail until a new image push registry is selected.`, { modal: true }, confirmBtn);
                        if (confirmRes !== confirmBtn) {
                            return;
                        }
                    }
                    else {
                        const confirmBtn = "Remove";
                        const confirmRes = yield vscode.window.showWarningMessage(`Are you sure you want to remove ${registry.fullAddress}?`, { modal: true }, confirmBtn);
                        if (confirmRes !== confirmBtn) {
                            return;
                        }
                    }
                    try {
                        yield Requester_1.default.removeRegistrySecret(this.connection, registry);
                    }
                    catch (err) {
                        const errMsg = `Failed to remove registry ${registry.fullAddress}`;
                        Logger_1.default.e(errMsg, err);
                        vscode.window.showErrorMessage(`${errMsg}: ${MCUtil_1.default.errToString(err)}`);
                    }
                    yield this.refresh();
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.HELP: {
                    vscode.commands.executeCommand(Commands_1.default.VSC_OPEN, CWDocs_1.default.getDocLink(CWDocs_1.default.REGISTRIES));
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.REFRESH: {
                    yield this.refresh();
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.OPEN_CONNECTION: {
                    ConnectionOverviewCmd_1.default(this.connection);
                    break;
                }
                default: {
                    Logger_1.default.e("Received unknown event from manage templates webview:", msg);
                }
            }
        });
        connection.onDidOpenRegistriesPage(this);
        this.refresh();
    }
    generateHtml(resourceProvider) {
        return __awaiter(this, void 0, void 0, function* () {
            this.registries = yield vscode.window.withProgress({
                location: vscode.ProgressLocation.Notification,
                cancellable: false,
                title: `Fetching image registries...`,
            }, () => __awaiter(this, void 0, void 0, function* () {
                return Requester_1.default.getImageRegistries(this.connection);
            }));
            const html = RegistriesPage_1.default(resourceProvider, this.connection.label, this.connection.isRemote, this.registries, this.connection.isKubeConnection);
            return html;
        });
    }
    onDidDispose() {
        this.connection.onDidCloseRegistriesPage();
    }
    lookupRegistry(fullAddress) {
        const matchingRegistry = this.registries.find((registry) => registry.fullAddress === fullAddress);
        if (!matchingRegistry) {
            Logger_1.default.e(`No matching registry found, expected to find fullAddress ${fullAddress}, registries are:`, this.registries);
            throw new Error(`No registry was found with full address "${fullAddress}"`);
        }
        return matchingRegistry;
    }
}
exports.RegistriesPageWrapper = RegistriesPageWrapper;
//# sourceMappingURL=RegistriesPageWrapper.js.map