"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
// import * as fs from "fs";
const url_1 = require("url");
const Resources_1 = require("../../constants/Resources");
const WebviewUtil_1 = require("./WebviewUtil");
const Logger_1 = require("../../Logger");
const MCUtil_1 = require("../../MCUtil");
const Commands_1 = require("../../constants/Commands");
const CWDocs_1 = require("../../constants/CWDocs");
const WebviewWrapper_1 = require("./WebviewWrapper");
const SourcesPage_1 = require("./pages/SourcesPage");
const ConnectionOverviewCmd_1 = require("../connection/ConnectionOverviewCmd");
var ManageSourcesWVMessages;
(function (ManageSourcesWVMessages) {
    ManageSourcesWVMessages["ENABLE_DISABLE"] = "enableOrDisable";
})(ManageSourcesWVMessages = exports.ManageSourcesWVMessages || (exports.ManageSourcesWVMessages = {}));
const SOURCES_PAGE_TITLE = "Template Source Manager";
function getTitle(connection) {
    let title = SOURCES_PAGE_TITLE;
    if (!global.isTheia) {
        title += ` (${connection.label})`;
    }
    return title;
}
class SourcesPageWrapper extends WebviewWrapper_1.WebviewWrapper {
    constructor(connection) {
        super(getTitle(connection), Resources_1.default.Icons.Logo);
        this.connection = connection;
        this.handleWebviewMessage = (msg) => __awaiter(this, void 0, void 0, function* () {
            switch (msg.type) {
                case ManageSourcesWVMessages.ENABLE_DISABLE: {
                    const enablement = msg.data;
                    yield this.enableDisable(enablement);
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.ADD_NEW: {
                    yield this.addNew();
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.DELETE: {
                    const sourceUrl = msg.data;
                    yield this.removeSource(sourceUrl);
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.HELP: {
                    vscode.commands.executeCommand(Commands_1.default.VSC_OPEN, CWDocs_1.default.getDocLink(CWDocs_1.default.TEMPLATE_MANAGEMENT));
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.REFRESH: {
                    // vscode.window.showInformationMessage("Refreshed repository list");
                    yield this.refresh();
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.OPEN_CONNECTION: {
                    ConnectionOverviewCmd_1.default(this.connection);
                    break;
                }
                case WebviewUtil_1.CommonWVMessages.OPEN_WEBLINK: {
                    WebviewUtil_1.default.openWeblink(msg.data);
                    break;
                }
                default: {
                    Logger_1.default.e("Received unknown event from manage templates webview:", msg);
                }
            }
        });
        connection.onDidOpenSourcesPage(this);
        this.refresh();
    }
    generateHtml(resourceProvider) {
        return __awaiter(this, void 0, void 0, function* () {
            const sources = yield this.connection.templateSourcesList.get(true);
            return SourcesPage_1.default(resourceProvider, this.connection.label, this.connection.isRemote, sources);
        });
    }
    onDidDispose() {
        this.connection.onDidCloseSourcesPage();
    }
    enableDisable(enablement) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i("Enable/Disable repos:", enablement);
            try {
                yield this.connection.templateSourcesList.toggleEnablement(enablement);
            }
            catch (err) {
                // If any of the enablements fail, the checkboxes will be out of sync with the backend state, so refresh the page to reset
                yield this.refresh();
            }
        });
    }
    addNew() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d("Adding new repo to " + this.connection.url);
            const repoInfo = yield promptForNewRepo();
            if (!repoInfo) {
                // cancelled
                return;
            }
            try {
                yield this.connection.templateSourcesList.add(repoInfo.repoUrl, repoInfo.repoName, repoInfo.repoDescr);
                yield this.refresh();
            }
            catch (err) {
                vscode.window.showErrorMessage(`Error adding new template source: ${MCUtil_1.default.errToString(err)}`, err);
                Logger_1.default.e(`Error adding new template repo ${JSON.stringify(repoInfo)}`, err);
            }
        });
    }
    removeSource(sourceUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.d(`Delete source ${sourceUrl} from ${this.connection.url}`);
            const yesBtn = "Yes";
            // TODO add name?
            const res = yield vscode.window.showWarningMessage(`Are you sure you wish to delete this template repository?`, { modal: true }, yesBtn);
            if (res !== yesBtn) {
                return;
            }
            try {
                yield this.connection.templateSourcesList.remove(sourceUrl);
                yield this.refresh();
            }
            catch (err) {
                vscode.window.showErrorMessage(`Error deleting template source ${sourceUrl}: ${MCUtil_1.default.errToString(err)}`, err);
                Logger_1.default.e(`Error removing template repo ${sourceUrl}`, err);
            }
        });
    }
}
exports.SourcesPageWrapper = SourcesPageWrapper;
function promptForNewRepo() {
    return __awaiter(this, void 0, void 0, function* () {
        const repoUrl = yield vscode.window.showInputBox({
            ignoreFocusOut: true,
            placeHolder: `https://raw.githubusercontent.com/codewind-resources/codewind-templates/master/devfiles/index.json`,
            prompt: "Enter the URL to your template source's index file.",
            validateInput: validateRepoInput,
        });
        if (!repoUrl) {
            return undefined;
        }
        let repoName = yield vscode.window.showInputBox({
            ignoreFocusOut: true,
            placeHolder: "My Templates",
            prompt: `Enter a name for ${repoUrl}`,
        });
        if (!repoName) {
            return undefined;
        }
        repoName = repoName.trim();
        let repoDescr = yield vscode.window.showInputBox({
            ignoreFocusOut: true,
            placeHolder: "Description of My Templates",
            prompt: `(Optional) Enter a description for ${repoName}`,
        });
        if (repoDescr) {
            repoDescr = repoDescr.trim();
        }
        return { repoUrl, repoName, repoDescr };
    });
}
function validateRepoInput(input) {
    let asUrl;
    try {
        // We use URL instead of vscode.Uri because the latter appears to throw errors irregularly.
        asUrl = new url_1.URL(input);
    }
    catch (err) {
        // not a url
    }
    if (!asUrl) {
        return "The repository URL must be a valid URL.";
    }
    else if (!asUrl.protocol.startsWith("http")) {
        return "The repository URL must be a valid http(s) URL.";
    }
    return undefined;
}
//# sourceMappingURL=SourcesPageWrapper.js.map