"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const fs = require("fs");
const path = require("path");
const Resources_1 = require("../../constants/Resources");
const MCUtil_1 = require("../../MCUtil");
const Logger_1 = require("../../Logger");
const Commands_1 = require("../../constants/Commands");
var CommonWVMessages;
(function (CommonWVMessages) {
    CommonWVMessages["OPEN_CONNECTION"] = "openConnection";
    CommonWVMessages["ADD_NEW"] = "addNew";
    CommonWVMessages["DELETE"] = "delete";
    CommonWVMessages["HELP"] = "help";
    CommonWVMessages["REFRESH"] = "refresh";
    CommonWVMessages["OPEN_WEBLINK"] = "openWebLink";
})(CommonWVMessages = exports.CommonWVMessages || (exports.CommonWVMessages = {}));
var WebviewUtil;
(function (WebviewUtil) {
    function getWebviewOptions() {
        return {
            enableScripts: true,
            retainContextWhenHidden: true,
            localResourceRoots: [
                Resources_1.default.getBaseResourcePath()
            ],
        };
    }
    WebviewUtil.getWebviewOptions = getWebviewOptions;
    function getCSP() {
        if (global.isTheia || MCUtil_1.default.isDevEnv()) {
            return "";
        }
        return `<meta http-equiv="Content-Security-Policy"
            content="default-src 'none'; img-src vscode-resource: https:; script-src vscode-resource: 'unsafe-inline'; style-src vscode-resource: 'unsafe-inline';"
        >`;
    }
    WebviewUtil.getCSP = getCSP;
    function buildTitleSection(rp, title, connectionLabel, isRemoteConnection) {
        // the subtitle is ommitted in theia since there is only one connection
        const hasSubtitle = !global.isTheia;
        return `<div class="title-section ${hasSubtitle ? "" : "title-section-subtitled"}">
            <img id="logo" alt="Codewind Logo" src="${rp.getIcon(Resources_1.default.Icons.Logo)}"/>
            <div>
                <h1 id="title">${title}</h1>
                ${hasSubtitle ? buildConnectionSubtitle(connectionLabel, isRemoteConnection) : ""}
            </div>
        </div>`;
    }
    WebviewUtil.buildTitleSection = buildTitleSection;
    function buildConnectionSubtitle(connectionLabel, isRemoteConnection) {
        let classAttr = "";
        let onClick = "";
        if (isRemoteConnection) {
            classAttr = `class="clickable"`;
            onClick = `onclick="sendMsg('${CommonWVMessages.OPEN_CONNECTION}')"`;
        }
        return `<h2 id="subtitle" ${classAttr} ${onClick}>${connectionLabel}</h2>`;
    }
    WebviewUtil.ATTR_ID = "data-id";
    WebviewUtil.ATTR_ENABLED = "data-enabled";
    WebviewUtil.TOGGLE_BTN_CLASS = "toggle-btn";
    function buildToggleTD(rp, enabled, title, idAttrValue) {
        return `<td class="btn-cell">
            <input type="image" title="${title}" alt="${title}" ${WebviewUtil.ATTR_ID}="${idAttrValue}" ${WebviewUtil.ATTR_ENABLED}="${enabled}"
                class="${WebviewUtil.TOGGLE_BTN_CLASS} btn" src="${getStatusToggleIconSrc(rp, enabled)}" onclick="onToggle(this)"/>
        </td>`;
    }
    WebviewUtil.buildToggleTD = buildToggleTD;
    function getStatusToggleIconSrc(rp, enabled, escapeBackslash = false) {
        const toggleIcon = rp.getIcon(enabled ? Resources_1.default.Icons.ToggleOnThin : Resources_1.default.Icons.ToggleOffThin);
        if (escapeBackslash) {
            return getEscapedPath(toggleIcon);
        }
        return toggleIcon;
    }
    WebviewUtil.getStatusToggleIconSrc = getStatusToggleIconSrc;
    /**
     * Paths to be opened that are embedded into the webview HTML require an extra escape on Windows.
     * https://github.com/eclipse/codewind/issues/476
     */
    function getEscapedPath(path) {
        if (MCUtil_1.default.getOS() === "windows") {
            return path.replace(/\\/g, "\\\\");
        }
        return path;
    }
    WebviewUtil.getEscapedPath = getEscapedPath;
    /**
     * To work around https://github.com/eclipse/codewind/issues/2273 we use this instead of <a href> to open links that may not use https.
     */
    function openWeblink(link) {
        const asUri = vscode.Uri.parse(link);
        if (!asUri.scheme) {
            Logger_1.default.e(`Received bad link to open from webview`, link);
            vscode.window.showErrorMessage(`Cannot open "${link}"`);
            return;
        }
        else if (asUri.scheme === "file") {
            Logger_1.default.w(`Refusing to open file URI`, asUri.toString());
            vscode.window.showWarningMessage(`This link points to a file, and cannot be opened.`);
            return;
        }
        vscode.commands.executeCommand(Commands_1.default.VSC_OPEN, asUri);
    }
    WebviewUtil.openWeblink = openWeblink;
    /**
     * For debugging in the browser, write out the html to an html file on disk and point to the resources on disk.
     * The file will be stored in the path in process.env.WEBVIEW_DEBUG_DIR, or ~.
     *
     * If WEBVIEW_DEBUG_DIR is not set, this function does nothing.
     */
    function debugWriteOutWebview(html, filename) {
        return __awaiter(this, void 0, void 0, function* () {
            const destDir = process.env.WEBVIEW_DEBUG_DIR;
            if (!destDir) {
                return;
                // destDir = process.env.HOME || ((MCUtil.getOS() === "windows") ? "C:\\" : "/");
            }
            if (!filename.endsWith(".html")) {
                filename = filename + ".html";
            }
            try {
                yield fs.promises.access(destDir);
            }
            catch (err) {
                Logger_1.default.d(`Creating ${destDir}`);
                yield fs.promises.mkdir(destDir, { recursive: true });
            }
            const destFile = path.join(destDir, filename);
            const htmlWithFileProto = html.replace(/vscode-resource:\/\/file\/\//g, "file://");
            try {
                yield fs.promises.writeFile(destFile, htmlWithFileProto);
                Logger_1.default.d(`Wrote out debug webview to ${destFile}`);
            }
            catch (err) {
                Logger_1.default.e(`Error writing out debug webview ${filename}`, err);
            }
        });
    }
    WebviewUtil.debugWriteOutWebview = debugWriteOutWebview;
})(WebviewUtil || (WebviewUtil = {}));
exports.default = WebviewUtil;
//# sourceMappingURL=WebviewUtil.js.map