"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Resources_1 = require("../../constants/Resources");
const WebviewUtil_1 = require("./WebviewUtil");
const Logger_1 = require("../../Logger");
const MCUtil_1 = require("../../MCUtil");
const VSC_RESOURCE_SCHEME = "vscode-resource:";
class WebviewWrapper {
    /**
     * Create and open the webview.
     * Subclass must call `this.refresh()` after performing all initialization.
     */
    constructor(title, titleIcon) {
        this.title = title;
        this.webPanel = vscode.window.createWebviewPanel(title, title, vscode.ViewColumn.Active, WebviewUtil_1.default.getWebviewOptions());
        this.webPanel.reveal();
        this.webPanel.onDidDispose(() => {
            this.onDidDispose();
        });
        this.webPanel.iconPath = Resources_1.default.getIconPaths(titleIcon);
        this.webPanel.webview.onDidReceiveMessage((msg) => {
            try {
                this.handleWebviewMessage(msg);
            }
            catch (err) {
                vscode.window.showErrorMessage(`${this.title}: Error running action ${msg.type} - ${MCUtil_1.default.errToString(err)}`);
                Logger_1.default.e(`Error processing message from webview ${this.title}`, err);
                Logger_1.default.e("Message was", msg);
            }
        });
        this.resourceProvider = {
            getIcon: (icon) => {
                const fsUri = Resources_1.default.getIconPaths(icon).dark;
                if (global.isTheia) {
                    return VSC_RESOURCE_SCHEME + fsUri.fsPath;
                }
                return this.webPanel.webview.asWebviewUri(fsUri).toString();
            },
            getStylesheet: (filename) => {
                const fsUri = Resources_1.default.getCssPath(filename);
                if (global.isTheia) {
                    return VSC_RESOURCE_SCHEME + fsUri.fsPath;
                }
                return this.webPanel.webview.asWebviewUri(fsUri).toString();
            }
        };
    }
    reveal() {
        this.webPanel.reveal();
    }
    dispose() {
        this.webPanel.dispose();
        this.onDidDispose();
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            let newHtml;
            try {
                newHtml = yield this.generateHtml(this.resourceProvider);
            }
            catch (err) {
                const errMsg = `${this.title} error:`;
                Logger_1.default.e(errMsg, err);
                vscode.window.showErrorMessage(`${errMsg} ${MCUtil_1.default.errToString(err)}`);
                return;
            }
            WebviewUtil_1.default.debugWriteOutWebview(newHtml, `${MCUtil_1.default.slug(this.title)}.html`);
            // Setting the html to "" seems to clear the page state, otherwise there is some caching done
            // which causes eg. the selected radiobutton to not be updated https://github.com/eclipse/codewind/issues/1413
            this.webPanel.webview.html = "";
            this.webPanel.webview.html = newHtml;
        });
    }
}
exports.WebviewWrapper = WebviewWrapper;
//# sourceMappingURL=WebviewWrapper.js.map