"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Resources_1 = require("../../../constants/Resources");
const MCUtil_1 = require("../../../MCUtil");
const CWDocs_1 = require("../../../constants/CWDocs");
const ProjectOverviewPageWrapper_1 = require("../ProjectOverviewPageWrapper");
const WebviewUtil_1 = require("../WebviewUtil");
const NOT_AVAILABLE = "Not available";
const NOT_RUNNING = "Not running";
const NOT_DEBUGGING = "Not debugging";
function getProjectOverviewHtml(rp, project) {
    return `
    <!DOCTYPE html>

    <html>
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        ${WebviewUtil_1.default.getCSP()}

        <link rel="stylesheet" href="${rp.getStylesheet("common.css")}"/>
        <link rel="stylesheet" href="${rp.getStylesheet("project-overview.css")}"/>
        ${global.isTheia ?
        `<link rel="stylesheet" href="${rp.getStylesheet("theia.css")}"/>` : ""}
    </head>
    <body>

    ${WebviewUtil_1.default.buildTitleSection(rp, project.name, project.connection.label, project.connection.isRemote)}

    <div id="main">
        <div id="top-section">
            <input type="button" value="Build"
                class="btn btn-prominent ${project.state.isEnabled ? "" : "btn-disabled"}"
                onclick="${project.state.isEnabled ? `sendMsg('${ProjectOverviewPageWrapper_1.ProjectOverviewWVMessages.BUILD}')` : ""}"/>

            <div id="top-right-btns">
                <input id="enablement-btn" class="btn btn-prominent" type="button"
                    onclick="sendMsg('${ProjectOverviewPageWrapper_1.ProjectOverviewWVMessages.TOGGLE_ENABLEMENT}')"
                    value="${(project.state.isEnabled ? "Disable" : "Enable") + " project"}"
                />
                <input class="btn btn-red" type="button"
                    onclick="sendMsg('${ProjectOverviewPageWrapper_1.ProjectOverviewWVMessages.UNBIND}')"
                    value="Remove project"
                />
            </div>
        </div>
        <div class="section">
            <h3>Project Information</h3>
            <table>
                ${buildRow(rp, "Type", project.type.toString())}
                ${buildRow(rp, "Language", MCUtil_1.default.uppercaseFirstChar(project.language))}
                ${buildRow(rp, "Project ID", project.id)}
                ${buildRow(rp, "Local Path", project.localPath.fsPath, { openable: global.isTheia ? undefined : "folder" })}
            </table>
        </div>
        <div class="section">
            <h3>Project Status</h3>
            <table>
                <tr>
                    <td class="info-label">Auto build:</td>
                    <td>
                        <input id="auto-build-toggle" type="checkbox" class="btn"
                            onclick="sendMsg('${ProjectOverviewPageWrapper_1.ProjectOverviewWVMessages.TOGGLE_AUTOBUILD}')"
                            ${project.autoBuildEnabled ? "checked" : ""}
                            ${project.state.isEnabled ? " " : " disabled"}
                        />
                    </td>
                </tr>
                <tr>
                    <td class="info-label">Inject Appmetrics:</td>
                    <td>
                        <input id="auto-inject-metrics-toggle" type="checkbox" class="btn"
                            onclick="sendMsg('${ProjectOverviewPageWrapper_1.ProjectOverviewWVMessages.TOGGLE_INJECT_METRICS}')"
                            ${project.injectMetricsEnabled ? "checked" : ""}
                            ${project.type.canInjectMetrics && project.state.isEnabled ? " " : " disabled"}
                        />
                    </td>
                </tr>
                ${buildRow(rp, "Application Status", normalize(project.state.getAppStatusWithDetail(), NOT_AVAILABLE))}
                ${buildRow(rp, "Build Status", normalize(project.state.getBuildString(), NOT_AVAILABLE))}
                ${buildRow(rp, "Last Image Build", normalizeDate(project.lastImgBuild, NOT_AVAILABLE))}
                ${buildRow(rp, "Last Build", normalizeDate(project.lastBuild, NOT_AVAILABLE))}
            </table>
        </div>
        <div class="section">
            <div id="app-info-header-section">
                <h3>Application Information</h3>
                <div id="about-project-settings">
                    <a href="${CWDocs_1.default.getDocLink(CWDocs_1.default.PROJECT_SETTINGS)}" title="More Info">More Info</a>
                </div>
            </div>
            <!-- Hide Container ID when it doesn't apply -->
            ${project.connection.isKubeConnection ? "" : `
                <table>
                    ${buildRow(rp, "Container ID", normalize(project.containerID, NOT_AVAILABLE, 32))}
                </table>`}

            <table>
                ${buildRow(rp, "Application Endpoint", normalize(project.appUrl, NOT_RUNNING), {
        editable: true,
        openable: project.appUrl != null ? "web" : undefined
    })}
                ${buildRow(rp, "Exposed App Port", normalize(project.ports.appPort, NOT_RUNNING))}
                ${buildRow(rp, "Internal App Port", normalize(project.ports.internalPort, NOT_AVAILABLE), { editable: true })}

                <!-- buildDebugSection must also close the <table> -->
                ${buildDebugSection(rp, project)}
            <!-- /table -->
        </div>
    <!-- end main -->
    </div>

    <script type="text/javascript">
        const vscode = acquireVsCodeApi();

        function sendMsg(type, data = undefined) {
            // See IWebViewMsg in ProjectOverviewCmd
            vscode.postMessage({ type: type, data: data });
        }
    </script>

    </body>
    </html>
    `;
}
exports.getProjectOverviewHtml = getProjectOverviewHtml;
const DEFAULT_ROW_OPTIONS = {
    copyable: false,
    editable: false,
    openable: undefined,
};
function buildRow(rp, label, data, options = DEFAULT_ROW_OPTIONS) {
    let secondColTdContents = "";
    let thirdColTd = "";
    let fourthColTd = "";
    if (options.openable) {
        let classAttr = "";
        // let href: string = "";
        let onclick = "";
        if (options.openable === "web") {
            // href = `href="${data}"`;
            classAttr = `class="url"`;
            onclick = `onclick="sendMsg('${WebviewUtil_1.CommonWVMessages.OPEN_WEBLINK}', '${data}')"`;
        }
        else {
            // it is a folder
            const folderPath = WebviewUtil_1.default.getEscapedPath(data);
            onclick = `onclick="sendMsg('${ProjectOverviewPageWrapper_1.ProjectOverviewWVMessages.OPEN_FOLDER}', '${folderPath}')"`;
        }
        secondColTdContents += `<a title="${data}" ${classAttr} ${onclick}>${data}</a>`;
    }
    else {
        secondColTdContents = `${data}`;
    }
    if (options.editable) {
        const tooltip = `title="Edit Project Settings"`;
        const onClick = `onclick="sendMsg('${ProjectOverviewPageWrapper_1.ProjectOverviewWVMessages.EDIT}')"`;
        thirdColTd = `
            <td class="btn-cell">
                <input type="image" id="edit-${MCUtil_1.default.slug(label)}" class="edit-btn" ${tooltip} ${onClick} src="${rp.getIcon(Resources_1.default.Icons.Edit)}"/>
            </td>
        `;
    }
    if (options.openable === "web") {
        // add an 'open' button if this row's data is a web link
        fourthColTd = `
            <td class="btn-cell">
                <a title="${data}" onclick="sendMsg('${WebviewUtil_1.CommonWVMessages.OPEN_WEBLINK}', '${data}')">
                    <input type="image" title="Open" src="${rp.getIcon(Resources_1.default.Icons.OpenExternal)}"/>
                </a>
            </td>
        `;
    }
    const secondTd = `<td title="${label}">${secondColTdContents}</td>`;
    return `
        <tr class="info-row">
            <td class="info-label">${label}:</td>
            ${secondTd}
            ${thirdColTd}
            ${fourthColTd}
        </tr>
    `;
}
/**
 * Convert `item` to a user-friendly string, or the fallback if `item` is undefined or invalid.
 */
function normalize(item, fallback, maxLength) {
    let result;
    if (item == null || item === "" || (typeof item === typeof 0 && isNaN(Number(item)))) {
        result = fallback;
    }
    else if (item instanceof vscode.Uri && item.scheme.includes("file")) {
        result = item.fsPath;
    }
    else {
        result = item.toString();
    }
    if (maxLength != null) {
        result = result.substring(0, maxLength);
    }
    return result;
}
function normalizeDate(d, fallback) {
    if (MCUtil_1.default.isGoodDate(d)) {
        let dateStr = d.toLocaleDateString();
        if (dateStr === (new Date()).toLocaleDateString()) {
            dateStr = "Today";
        }
        return `${dateStr} at ${d.toLocaleTimeString()}`;
    }
    else {
        return fallback;
    }
}
function buildDebugSection(rp, project) {
    if (global.isTheia) {
        return `
            </table>
        `;
    }
    let noDebugMsg;
    if (project.connection.isRemote) {
        noDebugMsg = "Remote projects do not support debug.";
    }
    else if (!project.capabilities.supportsDebug) {
        noDebugMsg = `${project.type} projects do not support debug.`;
    }
    if (noDebugMsg) {
        return `
            </table>
            ${noDebugMsg}
        `;
    }
    return `
        ${buildRow(rp, "Exposed Debug Port", normalize(project.ports.debugPort, NOT_DEBUGGING))}
        ${buildRow(rp, "Internal Debug Port", normalize(project.ports.internalDebugPort, NOT_AVAILABLE), { editable: true })}
        </table>
    `;
    // ${buildRow(rp, "Debug URL", normalize(project.debugUrl, NOT_DEBUGGING))}
}
//# sourceMappingURL=ProjectOverviewPage.js.map