"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
const fs = require("fs");
const path = require("path");
const Logger_1 = require("../Logger");
// non-nls-file
const RES_FOLDER_NAME = "res";
const STYLE_FOLDER_NAME = "css";
const IMG_FOLDER_NAME = "img";
const LIGHT_FOLDER_NAME = "light";
const DARK_FOLDER_NAME = "dark";
// for icons that are the same on all themes
const THEMELESS_FOLDER_NAME = "themeless";
// const PROJECT_TYPES_FOLDER = "project-types";
var Resources;
(function (Resources) {
    function getBaseResourcePath() {
        return vscode_1.Uri.file(path.join(global.__extRoot, RES_FOLDER_NAME));
    }
    Resources.getBaseResourcePath = getBaseResourcePath;
    function getImagePath(...paths) {
        const imagePath = path.join(getBaseResourcePath().fsPath, IMG_FOLDER_NAME, ...paths);
        return vscode_1.Uri.file(imagePath);
    }
    function getCssPath(filename) {
        const cssPath = path.join(getBaseResourcePath().fsPath, STYLE_FOLDER_NAME, filename);
        return vscode_1.Uri.file(cssPath);
    }
    Resources.getCssPath = getCssPath;
    /**
     * Resolve the icon with the given name and return the paths to it,
     * which can then be assigned to a vscode iconPath (eg on a TreeItem).
     * If an icon cannot be found, an error is will be logged.
     *
     * If a matching icon exists in the 'themeless' folder, the icon will be returned from there, with the dark and light paths matching.
     * Else, a requested icon is assumed to have a file in both 'dark' and 'light' folders.
     *
     */
    function getIconPaths(icon) {
        const themeless = getImagePath(THEMELESS_FOLDER_NAME, icon);
        try {
            fs.accessSync(themeless.fsPath, fs.constants.R_OK);
            return {
                dark: themeless,
                light: themeless,
            };
        }
        catch (err) {
            // if it doesn't exist in the themeless folder, it must exist in both dark and light folders
        }
        const darkPath = getImagePath(DARK_FOLDER_NAME, icon);
        // make sure the file exists and is readable
        fs.access(darkPath.fsPath, fs.constants.R_OK, (err) => {
            if (err) {
                Logger_1.default.e(`Dark icon not found! ${icon} - error:`, err);
            }
        });
        const lightPath = getImagePath(LIGHT_FOLDER_NAME, icon);
        fs.access(lightPath.fsPath, fs.constants.R_OK, (err) => {
            if (err) {
                Logger_1.default.e(`Light icon not found! ${icon} - error:`, err);
            }
        });
        return {
            dark: darkPath,
            light: lightPath,
        };
    }
    Resources.getIconPaths = getIconPaths;
    let Icons;
    (function (Icons) {
        Icons["Logo"] = "codewind.svg";
        Icons["Connect"] = "connect.svg";
        Icons["ConnectionConnected"] = "connection_connected.svg";
        Icons["ConnectionDisconnected"] = "connection_disconnected.svg";
        Icons["ConnectionConnectedCheckmark"] = "connection_connected_checkmark.svg";
        Icons["ConnectionDisconnectedCheckmark"] = "connection_disconnected_checkmark.svg";
        Icons["ConnectionWarningCheckmark"] = "connection_warning.svg";
        Icons["Copy"] = "copy.svg";
        Icons["ServerError"] = "server_error.svg";
        Icons["Error"] = "error.svg";
        Icons["Edit"] = "edit.svg";
        Icons["Help"] = "help.svg";
        Icons["Info"] = "info.svg";
        Icons["LocalConnected"] = "local_connected.svg";
        Icons["LocalDisconnected"] = "local_disconnected.svg";
        Icons["New"] = "new.svg";
        Icons["OpenExternal"] = "launch.svg";
        Icons["Play"] = "play.svg";
        Icons["Refresh"] = "refresh.svg";
        Icons["Stop"] = "stop.svg";
        Icons["ToggleOnThin"] = "toggle_on_thin.svg";
        Icons["ToggleOffThin"] = "toggle_off_thin.svg";
        Icons["Trash"] = "trash.svg";
        Icons["Delete"] = "trash_can.svg";
        Icons["Edit_Connection"] = "edit_connection.svg";
        Icons["Warning"] = "warning.svg";
        Icons["Generic"] = "project-types/generic.svg";
        Icons["Go"] = "project-types/go.svg";
        Icons["Java"] = "project-types/java.svg";
        Icons["Microprofile"] = "project-types/microprofile.svg";
        Icons["NodeJS"] = "project-types/nodejs.svg";
        Icons["Python"] = "project-types/python.svg";
        Icons["Spring"] = "project-types/spring.svg";
        Icons["Swift"] = "project-types/swift.svg";
    })(Icons = Resources.Icons || (Resources.Icons = {}));
    // https://octicons.github.com/
    let Octicons;
    (function (Octicons) {
        Octicons["sync"] = "sync";
        Octicons["bug"] = "bug";
    })(Octicons = Resources.Octicons || (Resources.Octicons = {}));
    /**
     * Get a string which can be included in a status bar message to render an octicon.
     * The returned value will be "$(iconName)" or "$(iconName~spin)"
     *
     * @param iconName - One of the octicons from https://octicons.github.com
     */
    function getOcticon(iconName, spinning = false) {
        return `$(${iconName}${spinning ? "~spin" : ""})`;
    }
    Resources.getOcticon = getOcticon;
})(Resources || (Resources = {}));
exports.default = Resources;
//# sourceMappingURL=Resources.js.map