/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const CommandUtil_1 = require("./command/CommandUtil");
const InitViews_1 = require("./view/InitViews");
const Logger_1 = require("./Logger");
const translator_1 = require("./constants/strings/translator");
const StringNamespaces_1 = require("./constants/strings/StringNamespaces");
const StartCodewindCmd_1 = require("./command/StartCodewindCmd");
const Constants_1 = require("./constants/Constants");
const ConnectionManager_1 = require("./codewind/connection/ConnectionManager");
// configures json as the language of the codewind settings file.
function setSettingsFileLanguage(doc) {
    // sometimes the path has .git appended, see https://github.com/Microsoft/vscode/issues/22561
    // since we are using the uri, the path separator will always be a forward slash.
    if ((doc.uri.scheme === "file" && doc.uri.path.endsWith(`/${Constants_1.default.PROJ_SETTINGS_FILE_NAME}`)) ||
        doc.uri.scheme === "git" && doc.uri.path.endsWith(`/${Constants_1.default.PROJ_SETTINGS_FILE_NAME}.git`)) {
        vscode.languages.setTextDocumentLanguage(doc, "json");
    }
}
// this method is called when your extension is activated
// your extension is activated the very first time the command is executed
function activate(context) {
    return __awaiter(this, void 0, void 0, function* () {
        process.on("unhandledRejection", (err) => Logger_1.default.e("Unhandled promise rejection:", err));
        // Initialize our globals
        global.__extRoot = context.extensionPath;
        // Declared as 'any' type, but will always be assigned globalState which is a vscode.Memento
        global.extGlobalState = context.globalState;
        global.isTheia = vscode.env.appName.toLowerCase().includes("theia");
        Logger_1.default.setLogFilePath(context);
        Logger_1.default.i("Finished activating logger");
        try {
            const thisExtension = vscode.extensions.getExtension("IBM.codewind");
            global.extVersion = thisExtension.packageJSON.version;
        }
        catch (err) {
            Logger_1.default.e("Error determining extension version", err);
            global.extVersion = "unknown";
        }
        Logger_1.default.i("Extension version is " + global.extVersion);
        try {
            yield translator_1.default.init();
        }
        catch (err) {
            // This string can't be translated for obvious reasons :)
            const errmsg = "Error initializing i18next - placeholder strings will be used! " + (err.message || err); // non-nls
            Logger_1.default.e(errmsg, err);
            vscode.window.showErrorMessage(errmsg);
        }
        const msg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "activeMsg");
        // Make sure i18next loaded the strings properly here.
        Logger_1.default.i("activeMsg:", msg);
        // vscode.window.showInformationMessage(msg);
        const subscriptions = [
            ...InitViews_1.default(),
            ...CommandUtil_1.createCommands(),
        ];
        subscriptions.push(ConnectionManager_1.default.instance);
        // configure json as the language of the codewind settings file.  ensure that this is applied
        // to any settings file active in the editor at the time this extension activates.
        context.subscriptions.push(vscode.workspace.onDidOpenTextDocument((doc) => setSettingsFileLanguage(doc)));
        setImmediate(() => {
            if (vscode.window.activeTextEditor) {
                setSettingsFileLanguage(vscode.window.activeTextEditor.document);
            }
        });
        yield ConnectionManager_1.default.instance.activate();
        // Connect to local codewind if it's started, but don't start it automatically.
        StartCodewindCmd_1.default(false);
        subscriptions.forEach((e) => {
            context.subscriptions.push(e);
        });
        Logger_1.default.d("Finished activating");
    });
}
exports.activate = activate;
// this method is called when your extension is deactivated
function deactivate() {
    // nothing here
}
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map