"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * The functions in this file generate the TreeItems' contextIDs,
 * which are checked against the regex in package.nls.json to determine command enablement in context menus.
 */
var TreeItemContextValues;
(function (TreeItemContextValues) {
    TreeItemContextValues["BASE"] = "ext.cw";
    TreeItemContextValues["ROOT"] = "root";
    TreeItemContextValues["NO_PROJECTS"] = "noProjects";
    // Local Codewind status
    TreeItemContextValues["LOCAL_CW_STOPPED"] = "local.stopped";
    TreeItemContextValues["LOCAL_CW_STARTED"] = "local.started";
    // Connection
    TreeItemContextValues["CONN_BASE"] = "connection";
    TreeItemContextValues["CONN_CONNECTED"] = "connection-good";
    // CONN_CONNECTED must not be a substring of CONN_DISCONNECTED
    TreeItemContextValues["CONN_DISCONNECTED"] = "connection-bad";
    TreeItemContextValues["REMOTECONN_ENABLED"] = "remote.enabled";
    TreeItemContextValues["REMOTECONN_DISABLED"] = "remote.disabled";
    TreeItemContextValues["CONN_WITH_REGISTRY"] = "registry";
    TreeItemContextValues["CONN_WITH_TEKTON"] = "tekton";
    // Project
    TreeItemContextValues["PROJ_BASE"] = "project";
    // (en|dis)abled are mutex
    TreeItemContextValues["PROJ_ENABLED"] = "enabled";
    TreeItemContextValues["PROJ_DISABLED"] = "disabled";
    // debuggable, started both imply enabled
    TreeItemContextValues["PROJ_DEBUGGABLE"] = "debuggable";
    TreeItemContextValues["PROJ_STARTED"] = "started";
    // auto build statuses are mutex
    TreeItemContextValues["PROJ_AUTOBUILD_ON"] = "autoBuildOn";
    TreeItemContextValues["PROJ_AUTOBUILD_OFF"] = "autoBuildOff";
    TreeItemContextValues["PROJ_RESTARTABLE"] = "restartable";
    TreeItemContextValues["PROJ_APP_MONITOR"] = "appMonitor";
    TreeItemContextValues["PROJ_PERF_DASHBOARD"] = "perfDashboard";
    TreeItemContextValues["PROJ_SHELLABLE"] = "shellable";
    TreeItemContextValues["PROJ_INJECT_METRICS_ON"] = "injectMetricsOn";
    TreeItemContextValues["PROJ_INJECT_METRICS_OFF"] = "injectMetricsOff";
})(TreeItemContextValues || (TreeItemContextValues = {}));
var TreeItemContext;
(function (TreeItemContext) {
    function getRootContext() {
        return buildContextValue([TreeItemContextValues.ROOT]);
    }
    TreeItemContext.getRootContext = getRootContext;
    function getLocalCWContext(localCW) {
        const contextValue = localCW.isStarted ? TreeItemContextValues.LOCAL_CW_STARTED : TreeItemContextValues.LOCAL_CW_STOPPED;
        const connectionContext = localCW.localConnection ? getConnectionContextInner(localCW.localConnection) : [];
        const cv = buildContextValue([...connectionContext, contextValue]);
        // Log.d("Local connection context " + cv);
        return cv;
    }
    TreeItemContext.getLocalCWContext = getLocalCWContext;
    function getConnectionContextInner(connection) {
        const contextValues = [TreeItemContextValues.CONN_BASE];
        if (connection.isRemote) {
            if (connection.enabled) {
                contextValues.push(TreeItemContextValues.REMOTECONN_ENABLED);
            }
            else {
                contextValues.push(TreeItemContextValues.REMOTECONN_DISABLED);
            }
        }
        if (connection.isConnected) {
            if (connection.isKubeConnection) {
                contextValues.push(TreeItemContextValues.CONN_WITH_TEKTON);
                contextValues.push(TreeItemContextValues.CONN_WITH_REGISTRY);
            }
            contextValues.push(TreeItemContextValues.CONN_CONNECTED);
        }
        else if (connection.enabled) {
            // Enabled but not connected -> Disconnected
            contextValues.push(TreeItemContextValues.CONN_DISCONNECTED);
        }
        return contextValues;
    }
    function getConnectionContext(connection) {
        const cv = buildContextValue(getConnectionContextInner(connection));
        // Log.d(`The context value for ${connection} is ${cv}`);
        return cv;
    }
    TreeItemContext.getConnectionContext = getConnectionContext;
    function getProjectContext(project) {
        const contextValues = [TreeItemContextValues.PROJ_BASE];
        if (project.state.isEnabled) {
            contextValues.push(TreeItemContextValues.PROJ_ENABLED);
            if (project.state.isStarted) {
                contextValues.push(TreeItemContextValues.PROJ_STARTED);
            }
            if (project.state.isDebuggable) {
                contextValues.push(TreeItemContextValues.PROJ_DEBUGGABLE);
            }
        }
        else {
            contextValues.push(TreeItemContextValues.PROJ_DISABLED);
        }
        if (project.autoBuildEnabled) {
            contextValues.push(TreeItemContextValues.PROJ_AUTOBUILD_ON);
        }
        else {
            contextValues.push(TreeItemContextValues.PROJ_AUTOBUILD_OFF);
        }
        if (project.capabilities.supportsRestart) {
            contextValues.push(TreeItemContextValues.PROJ_RESTARTABLE);
        }
        if (project.hasAppMonitor) {
            contextValues.push(TreeItemContextValues.PROJ_APP_MONITOR);
        }
        if (project.hasPerfDashboard) {
            contextValues.push(TreeItemContextValues.PROJ_PERF_DASHBOARD);
        }
        if (project.canContainerShell) {
            contextValues.push(TreeItemContextValues.PROJ_SHELLABLE);
        }
        if (project.type.canInjectMetrics) {
            if (project.injectMetricsEnabled) {
                contextValues.push(TreeItemContextValues.PROJ_INJECT_METRICS_ON);
            }
            else {
                contextValues.push(TreeItemContextValues.PROJ_INJECT_METRICS_OFF);
            }
        }
        // The final result will look like eg: "ext.cw.project.enabled.autoBuildOn"
        const cv = buildContextValue(contextValues);
        // Log.d(`The context value for ${project.name} is ${cv}`);
        return cv;
    }
    TreeItemContext.getProjectContext = getProjectContext;
    function buildContextValue(subvalues) {
        return [TreeItemContextValues.BASE, ...subvalues].join(".");
    }
})(TreeItemContext || (TreeItemContext = {}));
exports.default = TreeItemContext;
//# sourceMappingURL=TreeItemContext.js.map