"use strict";
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const TreeItemContext_1 = require("./TreeItemContext");
const Resources_1 = require("../constants/Resources");
const LocalCodewindManager_1 = require("../codewind/connection/local/LocalCodewindManager");
const translator_1 = require("../constants/strings/translator");
const Commands_1 = require("../constants/Commands");
const ConnectionState_1 = require("../codewind/connection/ConnectionState");
const STRING_NS = StringNamespaces_1.default.TREEVIEW;
var TreeItemFactory;
(function (TreeItemFactory) {
    function getRoot() {
        return {
            collapsibleState: vscode.TreeItemCollapsibleState.Expanded,
            iconPath: Resources_1.default.getIconPaths(Resources_1.default.Icons.Logo),
            label: "Codewind",
            contextValue: TreeItemContext_1.default.getRootContext(),
        };
    }
    TreeItemFactory.getRoot = getRoot;
    const CW_STOPPED_NODE_ID = "ext.cw.localTreeStopped";
    const CW_STARTED_NODE_ID = "ext.cw.localTreeStarted";
    function getLocalCWTI() {
        const cwState = LocalCodewindManager_1.default.instance.state;
        const cwStarted = LocalCodewindManager_1.default.instance.isStarted;
        let label = "Local";
        if (global.isTheia) {
            // it's confusing to say "local" in Theia
            label = "Projects";
        }
        // Show state except when started (since it's obvious in that case).
        if (!cwStarted) {
            label += ` (${cwState})`;
        }
        const cwUrl = LocalCodewindManager_1.default.instance.localConnection ? LocalCodewindManager_1.default.instance.localConnection.url : undefined;
        const tooltip = (cwUrl || "Stopped").toString();
        // we use the ID only in the started case so that when CW starts the new TreeItem can auto-expand after it starts
        const id = cwStarted ? CW_STARTED_NODE_ID : CW_STOPPED_NODE_ID;
        const collapsibleState = cwStarted ? vscode.TreeItemCollapsibleState.Expanded : vscode.TreeItemCollapsibleState.None;
        const icon = cwStarted ? Resources_1.default.Icons.LocalConnected : Resources_1.default.Icons.LocalDisconnected;
        const cwStatusItem = {
            id,
            label,
            tooltip,
            collapsibleState,
            iconPath: Resources_1.default.getIconPaths(icon),
            contextValue: TreeItemContext_1.default.getLocalCWContext(LocalCodewindManager_1.default.instance),
        };
        return cwStatusItem;
    }
    TreeItemFactory.getLocalCWTI = getLocalCWTI;
    function getConnectionTI(connection) {
        const collapsibleState = connection.state.hasChildrenInTree ? vscode.TreeItemCollapsibleState.Expanded : vscode.TreeItemCollapsibleState.None;
        // change ID so it refreshes the collapsiblestate
        const id = `${connection.label}.${connection.state.hasChildrenInTree ? "connected" : "disconnected"}`;
        const icon = connection.isConnected ? Resources_1.default.Icons.ConnectionConnected : Resources_1.default.Icons.ConnectionDisconnected;
        const iconPath = Resources_1.default.getIconPaths(icon);
        return {
            collapsibleState,
            contextValue: TreeItemContext_1.default.getConnectionContext(connection),
            iconPath,
            id,
            label: connection.label,
            tooltip: `${connection.enabled ? "" : "[Disabled] "}${connection.url}`,
        };
    }
    TreeItemFactory.getConnectionTI = getConnectionTI;
    function getConnectionChildren(connection) {
        if (connection.state === ConnectionState_1.ConnectionStates.AUTH_ERROR) {
            return [{
                    label: "Authentication error",
                    iconPath: Resources_1.default.getIconPaths(Resources_1.default.Icons.ServerError),
                    tooltip: "Click here to open this connection's Settings",
                    collapsibleState: vscode.TreeItemCollapsibleState.None,
                    command: {
                        command: Commands_1.default.CONNECTION_OVERVIEW,
                        title: "",
                        arguments: [connection]
                    }
                }];
        }
        else if (connection.state === ConnectionState_1.ConnectionStates.INITIALIZING) {
            return [{
                    label: "Connecting...",
                    iconPath: Resources_1.default.getIconPaths(Resources_1.default.Icons.ServerError),
                    collapsibleState: vscode.TreeItemCollapsibleState.None,
                }];
        }
        else if (!connection.isConnected) {
            return [{
                    label: translator_1.default.t(STRING_NS, "disconnectedConnectionLabel"),
                    iconPath: Resources_1.default.getIconPaths(Resources_1.default.Icons.ServerError),
                    collapsibleState: vscode.TreeItemCollapsibleState.None,
                }];
        }
        else if (connection.isConnected) {
            if (connection.projects.length > 0) {
                return connection.projects.sort((a, b) => a.name.localeCompare(b.name));
            }
            return [{
                    label: "No projects (Click here to create a project)",
                    iconPath: Resources_1.default.getIconPaths(Resources_1.default.Icons.Error),
                    tooltip: "Click here to create a project",
                    collapsibleState: vscode.TreeItemCollapsibleState.None,
                    command: {
                        command: Commands_1.default.CREATE_PROJECT,
                        title: "",
                        arguments: [connection]
                    }
                }];
        }
        // it is disabled
        return [];
    }
    TreeItemFactory.getConnectionChildren = getConnectionChildren;
    function getProjectTI(project) {
        const label = translator_1.default.t(STRING_NS, "projectLabel", { projectName: project.name, state: project.state.toString() });
        // won't work in theia until https://github.com/eclipse-theia/theia/pull/5590
        const command = {
            command: Commands_1.default.VSC_REVEAL_EXPLORER,
            title: "",
            arguments: [project.localPath]
        };
        return {
            id: project.id,
            label,
            collapsibleState: vscode.TreeItemCollapsibleState.None,
            tooltip: label,
            contextValue: TreeItemContext_1.default.getProjectContext(project),
            iconPath: Resources_1.default.getIconPaths(project.type.icon),
            // command run on single-click (or double click - depends on a user setting - https://github.com/Microsoft/vscode/issues/39601)
            command,
        };
    }
    TreeItemFactory.getProjectTI = getProjectTI;
})(TreeItemFactory || (TreeItemFactory = {}));
exports.default = TreeItemFactory;
//# sourceMappingURL=TreeItemFactory.js.map